@@ -1,38 +1,36 @@
-/************************************************************************
+/***********************************************************************
     filename:   Sample_Demo4.cpp
     created:    20/8/2005
     author:     Paul D Turner
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "Sample_Demo4.h"
 #include "CEGUI.h"
 #include "CEGuiBaseApplication.h"
+#include <cstdlib>
 
-#if defined( __WIN32__ ) || defined( _WIN32 )
-#define WIN32_LEAN_AND_MEAN
-#include "windows.h"
-
-int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,int nCmdShow)
-#else
 int main(int argc, char *argv[])
-#endif
 {
     // This is a basic start-up for the sample application which is
     // object orientated in nature, so we just need an instance of
@@ -54,23 +52,23 @@ bool Demo4Sample::initialiseSample()
     WindowManager& winMgr = WindowManager::getSingleton();
 
     // load scheme and set up defaults
-    SchemeManager::getSingleton().loadScheme("../datafiles/schemes/TaharezLook.scheme");
+    SchemeManager::getSingleton().loadScheme("TaharezLook.scheme");
     System::getSingleton().setDefaultMouseCursor("TaharezLook", "MouseArrow");
-    FontManager::getSingleton().createFont("../datafiles/fonts/Commonwealth-10.font");
+    FontManager::getSingleton().createFont("Commonwealth-10.font");
 
     // load an image to use as a background
-    ImagesetManager::getSingleton().createImagesetFromImageFile("BackgroundImage", "../datafiles/imagesets/GPN-2000-001437.tga");
+    ImagesetManager::getSingleton().createImagesetFromImageFile("BackgroundImage", "GPN-2000-001437.tga");
 
     // here we will use a StaticImage as the root, then we can use it to place a background image
-    StaticImage* background = static_cast<StaticImage*>(winMgr.createWindow("TaharezLook/StaticImage", "background_wnd"));
+    Window* background = winMgr.createWindow("TaharezLook/StaticImage", "background_wnd");
     // set position and size
-    background->setPosition(Point(0, 0));
-    background->setSize(Size(1, 1));
+    background->setPosition(UVector2(cegui_reldim(0), cegui_reldim( 0)));
+    background->setSize(UVector2(cegui_reldim(1), cegui_reldim( 1)));
     // disable frame and standard background
-    background->setFrameEnabled(false);
-    background->setBackgroundEnabled(false);
+    background->setProperty("FrameEnabled", "false");
+    background->setProperty("BackgroundEnabled", "false");
     // set the background image
-    background->setImage("BackgroundImage", "full_image");
+    background->setProperty("Image", "set:BackgroundImage image:full_image");
     // install this as the root GUI sheet
     System::getSingleton().setGUISheet(background);
 
@@ -90,16 +88,16 @@ bool Demo4Sample::initialiseSample()
     PushButton* btn = static_cast<PushButton*>(winMgr.createWindow("TaharezLook/Button", "QuitButton"));
     sheet->addChildWindow(btn);
     btn->setText("Quit!");
-    btn->setPosition(Point(0.035f, 0.0f));
-    btn->setSize(Size(0.1f, 0.036f));
+    btn->setPosition(UVector2(cegui_reldim(0.035f), cegui_reldim( 0.0f)));
+    btn->setSize(UVector2(cegui_reldim(0.1f), cegui_reldim( 0.036f)));
     btn->subscribeEvent(PushButton::EventClicked, Event::Subscriber(&Demo4Sample::handleQuit, this));
     btn->setAlwaysOnTop(true);
 
     // Alpha-slider
     Slider* slider = static_cast<Slider*>(winMgr.createWindow("TaharezLook/Slider", "my slider"));
     sheet->addChildWindow(slider);
-    slider->setPosition(Point(0.136f, 0.0f));
-    slider->setSize(Size(0.02f, 0.075f));
+    slider->setPosition(UVector2(cegui_reldim(0.136f), cegui_reldim( 0.0f)));
+    slider->setSize(UVector2(cegui_reldim(0.02f), cegui_reldim( 0.075f)));
     slider->setAlwaysOnTop(true);
     // here we disable inherited alpha so that we will always be able to see the slider
     slider->setInheritsAlpha(false);
@@ -112,10 +110,10 @@ bool Demo4Sample::initialiseSample()
     // big scroll bar
     Scrollbar* vscb = static_cast<Scrollbar*>(winMgr.createWindow("TaharezLook/LargeVerticalScrollbar", "Vert Scroll 1"));
     sheet->addChildWindow(vscb);
-    vscb->setPosition(Point(0.0f, 0.0f));
-    vscb->setMinimumSize(Size(0.01f, 0.01f));
-    vscb->setMaximumSize(Size(1.0f, 1.0f));
-    vscb->setSize(Size(0.035f, 0.83f));
+    vscb->setPosition(UVector2(cegui_reldim(0.0f), cegui_reldim( 0.0f)));
+    vscb->setMinSize(UVector2(cegui_reldim(0.01f), cegui_reldim( 0.01f)));
+    vscb->setMaxSize(UVector2(cegui_reldim(1.0f), cegui_reldim( 1.0f)));
+    vscb->setSize(UVector2(cegui_reldim(0.035f), cegui_reldim( 0.83f)));
     vscb->setDocumentSize(100);
     vscb->setPageSize(5);
     vscb->setStepSize(1);
@@ -124,10 +122,10 @@ bool Demo4Sample::initialiseSample()
     // mini vert scroll bar
     Scrollbar* mvsb = static_cast<Scrollbar*>(winMgr.createWindow("TaharezLook/VerticalScrollbar", "MiniVertScroll 1"));
     sheet->addChildWindow(mvsb);
-    mvsb->setPosition(Point(0.99f, 0.015f));
-    mvsb->setMinimumSize(Size(0.01f, 0.01f));
-    mvsb->setMaximumSize(Size(1.0f, 1.0f));
-    mvsb->setSize(Size(0.01f, 0.5f));
+    mvsb->setPosition(UVector2(cegui_reldim(0.99f), cegui_reldim( 0.015f)));
+    mvsb->setMinSize(UVector2(cegui_reldim(0.01f), cegui_reldim( 0.01f)));
+    mvsb->setMaxSize(UVector2(cegui_reldim(1.0f), cegui_reldim( 1.0f)));
+    mvsb->setSize(UVector2(cegui_reldim(0.01f), cegui_reldim( 0.5f)));
     mvsb->setDocumentSize(360);
     mvsb->setPageSize(45);
     mvsb->setStepSize(1);
@@ -136,10 +134,10 @@ bool Demo4Sample::initialiseSample()
     // mini horz scroll bar
     Scrollbar* mhsb = static_cast<Scrollbar*>(winMgr.createWindow("TaharezLook/HorizontalScrollbar", "MiniHorzScroll 1"));
     sheet->addChildWindow(mhsb);
-    mhsb->setPosition(Point(0.485f, 0.0f));
-    mhsb->setMinimumSize(Size(0.01f, 0.01f));
-    mhsb->setMaximumSize(Size(1.0f, 1.0f));
-    mhsb->setSize(Size(0.5f, 0.015f));
+    mhsb->setPosition(UVector2(cegui_reldim(0.485f), cegui_reldim( 0.0f)));
+    mhsb->setMinSize(UVector2(cegui_reldim(0.01f), cegui_reldim( 0.01f)));
+    mhsb->setMaxSize(UVector2(cegui_reldim(1.0f), cegui_reldim( 1.0f)));
+    mhsb->setSize(UVector2(cegui_reldim(0.5f), cegui_reldim( 0.015f)));
     mhsb->setDocumentSize(360);
     mhsb->setPageSize(45);
     mhsb->setStepSize(1);
@@ -150,50 +148,50 @@ bool Demo4Sample::initialiseSample()
     //
     FrameWindow* textwnd = static_cast<FrameWindow*>(winMgr.createWindow("TaharezLook/FrameWindow", "TextWindow"));
     sheet->addChildWindow(textwnd);
-    textwnd->setPosition(Point(0.2f, 0.2f));
-    textwnd->setMaximumSize(Size(0.75f, 0.75f));
-    textwnd->setMinimumSize(Size(0.1f, 0.1f));
-    textwnd->setSize(Size(0.5f, 0.5f));
+    textwnd->setPosition(UVector2(cegui_reldim(0.2f), cegui_reldim( 0.2f)));
+    textwnd->setMaxSize(UVector2(cegui_reldim(0.75f), cegui_reldim( 0.75f)));
+    textwnd->setMinSize(UVector2(cegui_reldim(0.1f), cegui_reldim( 0.1f)));
+    textwnd->setSize(UVector2(cegui_reldim(0.5f), cegui_reldim( 0.5f)));
     textwnd->setCloseButtonEnabled(false);
     textwnd->setText("Crazy Eddie's GUI - Demo 4");
 
-    StaticText* st = static_cast<StaticText*>(winMgr.createWindow("TaharezLook/StaticText", "TextWindow/Static"));
+    Window* st = winMgr.createWindow("TaharezLook/StaticText", "TextWindow/Static");
     textwnd->addChildWindow(st);
-    st->setPosition(Point(0.1f, 0.2f));
-    st->setSize(Size(0.5f, 0.6));
+    st->setPosition(UVector2(cegui_reldim(0.1f), cegui_reldim( 0.2f)));
+    st->setSize(UVector2(cegui_reldim(0.5f), cegui_reldim( 0.6f)));
 
-    st = static_cast<StaticText*>(winMgr.createWindow("TaharezLook/StaticText", "TextWindow/Group label 1"));
+    st = winMgr.createWindow("TaharezLook/StaticText", "TextWindow/Group label 1");
     textwnd->addChildWindow(st);
-    st->setPosition(Point(0.65f, 0.23f));
-    st->setSize(Size(0.35f, 0.05f));
+    st->setPosition(UVector2(cegui_reldim(0.65f), cegui_reldim( 0.23f)));
+    st->setSize(UVector2(cegui_reldim(0.35f), cegui_reldim( 0.05f)));
     st->setText("Horz. Formatting");
     // disable frame and background on static control
-    st->setFrameEnabled(false);
-    st->setBackgroundEnabled(false);
+    st->setProperty("FrameEnabled", "false");
+    st->setProperty("BackgroundEnabled", "false");
 
-    st = static_cast<StaticText*>(winMgr.createWindow("TaharezLook/StaticText", "TextWindow/Group label 2"));
+    st = winMgr.createWindow("TaharezLook/StaticText", "TextWindow/Group label 2");
     textwnd->addChildWindow(st);
-    st->setPosition(Point(0.65f, 0.53f));
-    st->setSize(Size(0.35f, 0.05f));
+    st->setPosition(UVector2(cegui_reldim(0.65f), cegui_reldim( 0.53f)));
+    st->setSize(UVector2(cegui_reldim(0.35f), cegui_reldim( 0.05f)));
     st->setText("Vert. Formatting");
     // disable frame and background on static control
-    st->setFrameEnabled(false);
-    st->setBackgroundEnabled(false);
+    st->setProperty("FrameEnabled", "false");
+    st->setProperty("BackgroundEnabled", "false");
 
-    st = static_cast<StaticText*>(winMgr.createWindow("TaharezLook/StaticText", "TextWindow/Box label"));
+    st = winMgr.createWindow("TaharezLook/StaticText", "TextWindow/Box label");
     textwnd->addChildWindow(st);
-    st->setPosition(Point(0.12f, 0.13f));
-    st->setSize(Size(0.35f, 0.05f));
+    st->setPosition(UVector2(cegui_reldim(0.12f), cegui_reldim( 0.13f)));
+    st->setSize(UVector2(cegui_reldim(0.35f), cegui_reldim( 0.05f)));
     st->setText("Formatted Output");
     // disable frame and background on static control
-    st->setFrameEnabled(false);
-    st->setBackgroundEnabled(false);
+    st->setProperty("FrameEnabled", "false");
+    st->setProperty("BackgroundEnabled", "false");
 
     // word-wrap checkbox
     Checkbox* cb = static_cast<Checkbox*>(winMgr.createWindow("TaharezLook/Checkbox", "TextWindow/CB1"));
     textwnd->addChildWindow(cb);
-    cb->setPosition(Point(0.65f, 0.13f));
-    cb->setSize(Size(0.35f, 0.05f));
+    cb->setPosition(UVector2(cegui_reldim(0.65f), cegui_reldim( 0.13f)));
+    cb->setSize(UVector2(cegui_reldim(0.35f), cegui_reldim( 0.05f)));
     cb->setText("Word Wrap");
     // subscribe a handler to listen for when the check-box button select state changes
     cb->subscribeEvent(Checkbox::EventCheckStateChanged, &formatChangedHandler);
@@ -201,8 +199,8 @@ bool Demo4Sample::initialiseSample()
     // horizontal formatting radio group
     RadioButton* rb = static_cast<RadioButton*>(winMgr.createWindow("TaharezLook/RadioButton", "TextWindow/RB1"));
     textwnd->addChildWindow(rb);
-    rb->setPosition(Point(0.65f, 0.3f));
-    rb->setSize(Size(0.35f, 0.05f));
+    rb->setPosition(UVector2(cegui_reldim(0.65f), cegui_reldim( 0.3f)));
+    rb->setSize(UVector2(cegui_reldim(0.35f), cegui_reldim( 0.05f)));
     rb->setGroupID(1);
     rb->setText("Left Aligned");
     // subscribe a handler to listen for when the radio button select state changes
@@ -210,8 +208,8 @@ bool Demo4Sample::initialiseSample()
 
     rb = static_cast<RadioButton*>(winMgr.createWindow("TaharezLook/RadioButton", "TextWindow/RB2"));
     textwnd->addChildWindow(rb);
-    rb->setPosition(Point(0.65f, 0.35f));
-    rb->setSize(Size(0.35f, 0.05f));
+    rb->setPosition(UVector2(cegui_reldim(0.65f), cegui_reldim( 0.35f)));
+    rb->setSize(UVector2(cegui_reldim(0.35f), cegui_reldim( 0.05f)));
     rb->setGroupID(1);
     rb->setText("Right Aligned");
     // subscribe a handler to listen for when the radio button select state changes
@@ -219,8 +217,8 @@ bool Demo4Sample::initialiseSample()
 
     rb = static_cast<RadioButton*>(winMgr.createWindow("TaharezLook/RadioButton", "TextWindow/RB3"));
     textwnd->addChildWindow(rb);
-    rb->setPosition(Point(0.65f, 0.4f));
-    rb->setSize(Size(0.35f, 0.05f));
+    rb->setPosition(UVector2(cegui_reldim(0.65f), cegui_reldim( 0.4f)));
+    rb->setSize(UVector2(cegui_reldim(0.35f), cegui_reldim( 0.05f)));
     rb->setGroupID(1);
     rb->setText("Centred");
     // subscribe a handler to listen for when the radio button select state changes
@@ -229,8 +227,8 @@ bool Demo4Sample::initialiseSample()
     // vertical formatting radio group
     rb = static_cast<RadioButton*>(winMgr.createWindow("TaharezLook/RadioButton", "TextWindow/RB4"));
     textwnd->addChildWindow(rb);
-    rb->setPosition(Point(0.65f, 0.6f));
-    rb->setSize(Size(0.35f, 0.05f));
+    rb->setPosition(UVector2(cegui_reldim(0.65f), cegui_reldim( 0.6f)));
+    rb->setSize(UVector2(cegui_reldim(0.35f), cegui_reldim( 0.05f)));
     rb->setGroupID(2);
     rb->setText("Top Aligned");
     // subscribe a handler to listen for when the radio button select state changes
@@ -238,8 +236,8 @@ bool Demo4Sample::initialiseSample()
 
     rb = static_cast<RadioButton*>(winMgr.createWindow("TaharezLook/RadioButton", "TextWindow/RB5"));
     textwnd->addChildWindow(rb);
-    rb->setPosition(Point(0.65f, 0.65f));
-    rb->setSize(Size(0.35f, 0.05f));
+    rb->setPosition(UVector2(cegui_reldim(0.65f), cegui_reldim( 0.65f)));
+    rb->setSize(UVector2(cegui_reldim(0.35f), cegui_reldim( 0.05f)));
     rb->setGroupID(2);
     rb->setText("Bottom Aligned");
     // subscribe a handler to listen for when the radio button select state changes
@@ -247,8 +245,8 @@ bool Demo4Sample::initialiseSample()
 
     rb = static_cast<RadioButton*>(winMgr.createWindow("TaharezLook/RadioButton", "TextWindow/RB6"));
     textwnd->addChildWindow(rb);
-    rb->setPosition(Point(0.65f, 0.7f));
-    rb->setSize(Size(0.35f, 0.05f));
+    rb->setPosition(UVector2(cegui_reldim(0.65f), cegui_reldim( 0.7f)));
+    rb->setSize(UVector2(cegui_reldim(0.35f), cegui_reldim( 0.05f)));
     rb->setGroupID(2);
     rb->setText("Centred");
     // subscribe a handler to listen for when the radio button select state changes
@@ -257,9 +255,9 @@ bool Demo4Sample::initialiseSample()
     // Edit box for text entry
     Editbox* eb = static_cast<Editbox*>(winMgr.createWindow("TaharezLook/Editbox", "TextWindow/Editbox1"));
     textwnd->addChildWindow(eb);
-    eb->setPosition(Point(0.05f, 0.85f));
-    eb->setMaximumSize(Size(1.0f, 0.04f));
-    eb->setSize(Size(0.90f, 0.08f));
+    eb->setPosition(UVector2(cegui_reldim(0.05f), cegui_reldim( 0.85f)));
+    eb->setMaxSize(UVector2(cegui_reldim(1.0f), cegui_reldim( 0.04f)));
+    eb->setSize(UVector2(cegui_reldim(0.90f), cegui_reldim( 0.08f)));
     // subscribe a handler to listen for when the text changes
     eb->subscribeEvent(Window::EventTextChanged, &textChangedHandler);
 
@@ -320,25 +318,25 @@ bool formatChangedHandler(const CEGUI::EventArgs& e)
     const RadioButton* rb6 = static_cast<const RadioButton*>(winMgr.getWindow("TextWindow/RB6"));
     const Checkbox*    cb1 = static_cast<const Checkbox*>(winMgr.getWindow("TextWindow/CB1"));
     // and also the static text for which we will set the formatting options
-    StaticText*  st  = static_cast<StaticText*>(winMgr.getWindow("TextWindow/Static"));
+    Window*  st  = winMgr.getWindow("TextWindow/Static");
 
     // handle vertical formatting settings
     if (rb4->isSelected())
-        st->setVerticalFormatting(StaticText::TopAligned);
+        st->setProperty("VertFormatting", "TopAligned");
     else if (rb5->isSelected())
-        st->setVerticalFormatting(StaticText::BottomAligned);
+        st->setProperty("VertFormatting", "BottomAligned");
     else if (rb6->isSelected())
-        st->setVerticalFormatting(StaticText::VertCentred);
+        st->setProperty("VertFormatting", "VertCentred");
 
     // handle horizontal formatting settings
     bool wrap = cb1->isSelected();
 
     if (rb1->isSelected())
-        st->setHorizontalFormatting(wrap ? StaticText::WordWrapLeftAligned : StaticText::LeftAligned);
+        st->setProperty("HorzFormatting", wrap ? "WordWrapLeftAligned" : "LeftAligned");
     else if (rb2->isSelected())
-        st->setHorizontalFormatting(wrap ? StaticText::WordWrapRightAligned : StaticText::RightAligned);
+        st->setProperty("HorzFormatting", wrap ? "WordWrapRightAligned" : "RightAligned");
     else if (rb3->isSelected())
-        st->setHorizontalFormatting(wrap ? StaticText::WordWrapCentred : StaticText::HorzCentred);
+        st->setProperty("HorzFormatting", wrap ? "WordWrapCentred" : "HorzCentred");
 
     // event was handled
     return true;
@@ -352,7 +350,7 @@ bool textChangedHandler(const CEGUI::EventArgs& e)
     using namespace CEGUI;
 
     //find the static box
-    StaticText* st = static_cast<StaticText*>(WindowManager::getSingleton().getWindow("TextWindow/Static"));
+    Window* st = WindowManager::getSingleton().getWindow("TextWindow/Static");
 
     // set text from the edit box...
     st->setText(static_cast<const WindowEventArgs&>(e).window->getText());
