@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUI.h
 	created:	21/2/2004
 	author:		Paul D Turner
 	
 	purpose:	Main system include for client code
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUI_h_
 #define _CEGUI_h_
 
@@ -49,14 +53,17 @@
 #include "CEGUIWindowFactory.h"
 #include "CEGUIWindowFactoryManager.h"
 #include "CEGUIWindowManager.h"
+#include "CEGUIWindowRenderer.h"
+#include "CEGUIWindowRendererManager.h"
 #include "CEGUIScheme.h"
 #include "CEGUISchemeManager.h"
 #include "CEGUISystem.h"
 #include "CEGUIScriptModule.h"
+#include "CEGUICoordConverter.h"
+#include "CEGUIPropertyHelper.h"
 
-// mid-level renderables
-#include "CEGUIRenderableFrame.h"
-#include "CEGUIRenderableImage.h"
+// Falagard core system include
+#include "falagard/CEGUIFalWidgetLookManager.h"
 
 // gui elements
 #include "elements/CEGUIButtonBase.h"
@@ -70,19 +77,15 @@
 #include "elements/CEGUIThumb.h"
 #include "elements/CEGUISlider.h"
 #include "elements/CEGUIScrollbar.h"
-#include "elements/CEGUIStatic.h"
 #include "elements/CEGUIListbox.h"
 #include "elements/CEGUICombobox.h"
 #include "elements/CEGUIListHeader.h"
 #include "elements/CEGUIMultiColumnList.h"
-#include "elements/CEGUIStaticText.h"
-#include "elements/CEGUIStaticImage.h"
 #include "elements/CEGUIGUISheet.h"
 #include "elements/CEGUIListboxTextItem.h"
 #include "elements/CEGUIComboDropList.h"
 #include "elements/CEGUIMultiLineEditbox.h"
 #include "elements/CEGUITabButton.h"
-#include "elements/CEGUITabPane.h"
 #include "elements/CEGUITabControl.h"
 #include "elements/CEGUISpinner.h"
 #include "elements/CEGUIDragContainer.h"
@@ -90,11 +93,12 @@
 #include "elements/CEGUIScrollablePane.h"
 #include "elements/CEGUITooltip.h"
 #include "elements/CEGUIItemEntry.h"
-#include "elements/CEGUITextItem.h"
 #include "elements/CEGUIMenuItem.h"
 #include "elements/CEGUIItemListBase.h"
 #include "elements/CEGUIMenuBase.h"
 #include "elements/CEGUIMenubar.h"
 #include "elements/CEGUIPopupMenu.h"
+#include "elements/CEGUIScrolledItemListBase.h"
+#include "elements/CEGUIItemListbox.h"
 
 #endif	// end of guard _CEGUI_h_
