@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIFontManager.h
 	created:	21/2/2004
 	author:		Paul D Turner
 	
 	purpose:	Defines interface for the FontManager class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUIFontManager_h_
 #define _CEGUIFontManager_h_
 
@@ -68,26 +72,6 @@ public:
 	~FontManager(void);
 
 
-	/*!
-	\brief
-		Return singleton FontManager object
-
-	\return
-		Singleton FontManager object
-	*/
-	static	FontManager&	getSingleton(void);
-
-
-	/*!
-	\brief
-		Return pointer to singleton FontManager object
-
-	\return
-		Pointer to singleton FontManager object
-	*/
-	static	FontManager*	getSingletonPtr(void);
-
-
 	/*!
 	\brief
 		Creates a new font from a font definition file, and returns a pointer to the new Font object.
@@ -108,25 +92,23 @@ public:
 	\exception	RendererException			thrown if the Renderer can't support a texture large enough to hold the requested glyph imagery.
 	\exception	MemoryException				thrown if allocation of imagery construction buffer fails.
 	*/
-	Font*	createFont(const String& filename, const String& resourceGroup = "");
+    Font *createFont(const String& filename, const String& resourceGroup = "");
 
 
 	/*!
 	\brief
 		Creates a new Font based on a true-type font, and returns a pointer to the new Font object.
 
+	\param type
+		String object containing the type of the font to be created (same as
+		in the "Type" attribute of the font XML).
+
 	\param name
 		String object containing a unique name for the new font.
 
 	\param fontname
 		String object containing the name and path of the true-type font to access.
 
-	\param size
-		Specifies the glyph size (point-size) for the new font.
-
-	\param flags
-		Some combination of FontFlag values to be used for the creation of this font.
-
     \param resourceGroup
         Resource group identifier to be passed to the resource provider when loading the font definition file.
 
@@ -138,7 +120,18 @@ public:
 	\exception	RendererException			thrown if the Renderer can't support a texture large enough to hold the requested glyph imagery.
 	\exception	MemoryException				thrown if allocation of imagery construction buffer fails.
 	*/
-	Font*	createFont(const String& name, const String& fontname, uint size, uint flags, const String& resourceGroup = "");
+    Font *createFont (const String &type, const String& name, const String& fontname,
+                      const String& resourceGroup = "");
+
+
+    /*!
+    \brief
+        Create a new font object given its type and the XML attributes.
+
+    \return
+        The new font object or NULL.
+    */
+    Font *createFont (const String &type, const XMLAttributes& attributes);
 
 
 	/*!
@@ -235,27 +228,23 @@ public:
 
 
 private:
-	/*************************************************************************
-		Implementation Data
-	*************************************************************************/
-	typedef	std::map<String, Font*>		FontRegistry;
-	FontRegistry		d_fonts;
-
-	struct FontManagerImplData;
-	FontManagerImplData*	d_implData;
-
+    /*************************************************************************
+        Implementation Data
+    *************************************************************************/
+    typedef std::map<String, Font*, String::FastLessCompare> FontRegistry;
+    FontRegistry d_fonts;
 
 public:
-	/*************************************************************************
-		Iterator stuff
-	*************************************************************************/
-	typedef	ConstBaseIterator<FontRegistry>	FontIterator;
+    /*************************************************************************
+        Iterator stuff
+    *************************************************************************/
+    typedef ConstBaseIterator<FontRegistry> FontIterator;
 
-	/*!
-	\brief
-		Return a FontManager::FontIterator object to iterate over the available Font objects.
-	*/
-	FontIterator	getIterator(void) const;
+    /*!
+    \brief
+        Return a FontManager::FontIterator object to iterate over the available Font objects.
+    */
+    FontIterator getIterator (void) const;
 };
 
 } // End of  CEGUI namespace section
