@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIForwardRefs.h
 	created:	21/2/2004
 	author:		Paul D Turner
 	
 	purpose:	Forward declares all core system classes
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUIForwardRefs_h_
 #define _CEGUIForwardRefs_h_
 
@@ -63,11 +67,14 @@ class FactoryModule;
 class ScriptModule;
 class ResourceProvider;
 class DefaultResourceProvider;
+class DynamicModule;
 class PropertyReceiver;
 class GlobalEventSet;
 class XMLAttributes;
 class XMLHandler;
 class XMLParser;
+class WindowRenderer;
+class WindowRendererManager;
 
 /*************************************************************************
 	Forward reference declarations for GUI element base classes
@@ -88,9 +95,6 @@ class ProgressBar;
 class Thumb;
 class Scrollbar;
 class Slider;
-class Static;
-class StaticText;
-class StaticImage;
 class ListboxItem;
 class ListboxTextItem;
 class ComboDropList;
@@ -102,12 +106,15 @@ class ScrollablePane;
 class ScrolledContainer;
 class Tooltip;
 class ItemEntry;
-class TextItem;
 class MenuItem;
 class ItemListBase;
 class MenuBase;
 class Menubar;
 class PopupMenu;
+class ScrolledItemListBase;
+class ItemListbox;
+class TabButton;
+class TabControl;
 
 } // End of  CEGUI namespace section
 
