@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIImage.h
 	created:	13/3/2004
 	author:		Paul D Turner
 	
 	purpose:	Defines interface for Image class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUIImage_h_
 #define _CEGUIImage_h_
 
@@ -33,6 +37,7 @@
 #include "CEGUIVector.h"
 #include "CEGUISize.h"
 #include "CEGUIRenderer.h"
+#include "CEGUIXMLSerializer.h" 
 #include <map>
 
 
@@ -131,6 +136,15 @@ public:
 	*/
 	const String&	getImagesetName(void) const;
 
+	/*!
+	\brief
+		Return the parent Imageset object that contains this Image
+
+	\return
+		The parent Imageset object.
+	*/
+	const Imageset*	getImageset(void) const {return d_owner;}
+
     /*!
     \brief
         Return Rect describing the source texture area used by this Image.
@@ -371,16 +385,17 @@ public:
     \brief
         Writes an xml representation of this Image object to \a out_stream.
 
-    \param out_stream
+    \param xml_stream
         Stream where xml data should be output.
 
+
     \return
         Nothing.
     */
-    void writeXMLToStream(OutStream& out_stream) const;
+    void writeXMLToStream(XMLSerializer& xml_stream) const;
 
 
-	friend class std::map<String, Image>;
+	friend class std::map<String, Image, String::FastLessCompare>;
 	friend struct std::pair<const String, Image>;
 
 
