@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUILogger.h
 	created:	21/2/2004
 	author:		Paul D Turner
-	
+
 	purpose:	Defines interface for the Logger class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUILogger_h_
 #define _CEGUILogger_h_
 
@@ -60,7 +64,10 @@ enum LoggingLevel
 
 /*!
 \brief
-	Class that implements logging for the GUI system
+	Abstract class that defines the interface of a logger object for the GUI system.
+    The default implementation of this interface is the DefaultLogger class; if you
+    want to perform special logging, derive your own class from Logger and initialize
+    a object of that type before you create the CEGUI::System singleton.
 */
 class CEGUIEXPORT Logger : public Singleton <Logger>
 {
@@ -74,17 +81,7 @@ public:
 	/*!
 	\brief Destructor for Logger object.
 	*/
-	~Logger(void);
-
-	
-	/*!
-	\brief
-		Return the singleton Logger object
-
-	\return
-		Reference to the one and only Logger object
-	*/
-	static Logger&	getSingleton(void);
+	virtual ~Logger(void);
 
 
 	/*!
@@ -123,11 +120,13 @@ public:
 	\return
 		Nothing
 	*/
-	void	logEvent(const String& message, LoggingLevel level = Standard);
+	virtual void logEvent(const String& message, LoggingLevel level = Standard) = 0;
 
     /*!
     \brief
         Set the name of the log file where all subsequent log entries should be written.
+        The interpretation of file name may differ depending on the concrete logger
+        implementation.
 
     \note
         When this is called, and the log file is created, any cached log entries are
@@ -140,23 +139,16 @@ public:
         - true if events should be added to the end of the current file.
         - false if the current contents of the file should be discarded.
      */
-    void    setLogFilename(const String& filename, bool append = false);
+    virtual void setLogFilename(const String& filename, bool append = false) = 0;
 
 protected:
-    /*************************************************************************
-		Implementation Data
-	*************************************************************************/
 	LoggingLevel	d_level;		//!< Holds current logging level
-	std::ofstream	d_ostream;		//!< Stream used to implement the logger
-    std::vector<std::pair<String, LoggingLevel> > d_cache;    //!< Used to cache log entries before log file is created.
-    std::ostringstream d_workstream;//!< Used to build log entry strings. 
-    bool d_caching;                 //!< true while log entries are beign cached (prior to logfile creation)
-    
+
 private:
 	/*************************************************************************
 		Copy constructor and assignment usage is denied.
 	*************************************************************************/
-	Logger(const Logger& logger) {}
+	Logger(const Logger& logger) : Singleton <Logger>() {}
 	Logger& operator=(const Logger& logger) {return *this;}
 
 };
