@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUISystem.h
 	created:	20/2/2004
 	author:		Paul D Turner
 
 	purpose:	Defines interface for main GUI system class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUISystem_h_
 #define _CEGUISystem_h_
 
@@ -82,144 +86,32 @@ public:
 	/*************************************************************************
 		Construction and Destruction
 	*************************************************************************/
-	/*!
-	\brief
-		Constructs a System object.
-
-	\param renderer
-		Pointer to the valid Renderer object that will be used to render GUI imagery
-
-	\param logFile
-		pointer to a utf8 encoded buffer containing the name to use for the log file.
-	*/
-	System(Renderer* renderer, const utf8* logFile = (const utf8*)"CEGUI.log");
-
-	/*!
-	\brief
-		Construct a new System object
-
-	\param renderer
-		Pointer to the valid Renderer object that will be used to render GUI imagery
-
-	\param resourceProvider
-		Pointer to a ResourceProvider object.
-
-	\param logFile
-		pointer to a utf8 encoded buffer containing the name to use for the log file.
-	*/
-    System(Renderer* renderer, ResourceProvider* resourceProvider, const utf8* logFile = (const utf8*)"CEGUI.log");
-
-
-	/*!
-	\brief
-		Construct a new System object
-
-	\param renderer
-		Pointer to the valid Renderer object that will be used to render GUI imagery
-
-	\param scriptModule
-		Pointer to a ScriptModule object.  may be NULL.
-
-	\param configFile
-		pointer to a utf8 encoded buffer containing the name to use for the configuration file.
-	*/
-	System(Renderer* renderer, ScriptModule* scriptModule, const utf8* configFile = (const utf8*)"cegui.config");
-
-
-	/*!
-	\brief
-		Construct a new System object
-
-	\param renderer
-		Pointer to the valid Renderer object that will be used to render GUI imagery
-
-	\param scriptModule
-		Pointer to a ScriptModule object.  may be NULL.
-
-	\param resourceProvider
-		Pointer to a ResourceProvider object.
-
-	\param configFile
-		pointer to a utf8 encoded buffer containing the name to use for the configuration file.
-	*/
-	System(Renderer* renderer, ScriptModule* scriptModule, ResourceProvider* resourceProvider, const utf8* configFile = (const utf8*)"cegui.config");
-
-
-    /*!
-    \brief
-        Constructs a System object.
-
-    \param renderer
-        Pointer to the valid Renderer object that will be used to render GUI imagery
-
-    \param xmlParser
-        Pointer to a valid XMLParser object to be used when parsing XML files, or NULL to use a default parser.
-
-    \param logFile
-        pointer to a utf8 encoded buffer containing the name to use for the log file.
-    */
-    System(Renderer* renderer, XMLParser* xmlParser, const utf8* logFile = (const utf8*)"CEGUI.log");
-
-    
-    /*!
-    \brief
-        Constructs a System object.
-
-    \param renderer
-        Pointer to the valid Renderer object that will be used to render GUI imagery
-
-    \param resourceProvider
-        Pointer to a ResourceProvider object.
-    
-    \param xmlParser
-        Pointer to a valid XMLParser object to be used when parsing XML files, or NULL to use a default parser.
-
-    \param logFile
-        pointer to a utf8 encoded buffer containing the name to use for the log file.
-    */
-    System(Renderer* renderer, ResourceProvider* resourceProvider, XMLParser* xmlParser, const utf8* logFile = (const utf8*)"CEGUI.log");
-
-    
     /*!
     \brief
         Construct a new System object
 
     \param renderer
-        Pointer to the valid Renderer object that will be used to render GUI imagery
-
-    \param xmlParser
-        Pointer to a valid XMLParser object to be used when parsing XML files, or NULL to use a default parser.
-    
-    \param scriptModule
-        Pointer to a ScriptModule object.  may be NULL.
-
-    \param configFile
-        pointer to a utf8 encoded buffer containing the name to use for the configuration file.
-    */
-    System(Renderer* renderer, XMLParser* xmlParser, ScriptModule* scriptModule, const utf8* configFile = (const utf8*)"cegui.config");
-
-        
-    /*!
-    \brief
-        Construct a new System object
-
-    \param renderer
-        Pointer to the valid Renderer object that will be used to render GUI imagery
+        Pointer to the valid Renderer object that will be used to render GUI
+        imagery.
 
     \param resourceProvider
-        Pointer to a ResourceProvider object.
-    
+        Pointer to a ResourceProvider object, or NULL to use whichever default
+        the Renderer provides.
+
     \param xmlParser
-        Pointer to a valid XMLParser object to be used when parsing XML files, or NULL to use a default parser.
-    
+        Pointer to a valid XMLParser object to be used when parsing XML files,
+        or NULL to use a default parser.
+
     \param scriptModule
-        Pointer to a ScriptModule object.  may be NULL.
+        Pointer to a ScriptModule object.  may be NULL for none.
 
     \param configFile
-        pointer to a utf8 encoded buffer containing the name to use for the configuration file.
-    */
-    System(Renderer* renderer, ResourceProvider* resourceProvider, XMLParser* xmlParser, ScriptModule* scriptModule, const utf8* configFile = (const utf8*)"cegui.config");
+        String object containing the name of a configuration file to use.
 
+    \param logFile
+        String object containing the name to use for the log file.
+    */
+    System(Renderer* renderer, ResourceProvider* resourceProvider = 0, XMLParser* xmlParser = 0, ScriptModule* scriptModule = 0, const String& configFile = "", const String& logFile = "CEGUI.log");
 
 	/*!
 	\brief
@@ -511,6 +403,18 @@ public:
 	*/
 	ScriptModule*	getScriptingModule(void) const;
 
+    /*!
+    \brief
+        Set the ScriptModule to be used for scripting within the GUI system.
+
+    \param scriptModule
+        Pointer to a ScriptModule based object, or 0 for none (be careful!)
+
+    \return
+        Nothing
+    */
+    void setScriptingModule(ScriptModule* scriptModule);
+
 	/*!
 	\brief
 		Return a pointer to the ResourceProvider being used within the GUI system.
@@ -623,7 +527,7 @@ public:
     \return
         Nothing.
     */
-    void setTooltip(Tooltip* tooltip);
+    void setDefaultTooltip(Tooltip* tooltip);
 
     /*!
     \brief
@@ -640,7 +544,7 @@ public:
     \return
         Nothing.
     */
-    void setTooltip(const String& tooltipType);
+    void setDefaultTooltip(const String& tooltipType);
 
     /*!
     \brief
@@ -672,6 +576,62 @@ public:
 	*/
 	Window* getModalTarget(void) const		{return d_modalTarget;}
 
+    /*!
+    \brief
+        Static member to set the name of the default XML parser module that
+        should be used.
+
+        If you want to modify the default parser from the one compiled in, you
+        need to call this static member prior to instantiating the main
+        CEGUI::System object.
+
+        Note that calling this member to change the name of the default module
+        after CEGUI::System, and therefore the default xml parser, has been
+        created will have no real effect - the default parser name will be
+        updated, though no actual changes to the xml parser module will occur.
+
+        The built-in options for this are:
+         - XercesParser
+         - ExpatParser
+         - LibxmlParser
+         - TinyXMLParser
+
+        Whether these are actually available, depends upon how you built the
+        system.  If you have some custom parser, you can provide the name of
+        that here to have it used as the default, though note that the
+        final filename of the parser module should be of the form:
+
+        [prefix]CEGUI[parserName][suffix]
+
+        where:
+        - [prefix] is some optional prefix; like 'lib' on linux.
+        - CEGUI is a required prefix.
+        - [parserName] is the name of the parser, as supplied to this function.
+        - [suffix] is the filename suffix, like .dll or .so
+
+        Final module filenames are, thus, of the form:
+        - CEGUIXercesParser.dll
+        - libCEGUIXercesParser.so
+
+    \param parserName
+        String describing the name of the xml parser module to be used as the
+        default.
+
+    \return
+        Nothing.
+    */
+    static void setDefaultXMLParserName(const String& parserName);
+
+    /*!
+    \brief
+        Return the name of the currently set default xml parser module.
+
+    \return
+        String holding the currently set default xml parser name.  Note that if
+        this name has been changed after instantiating the system, the name
+        returned may not actually correspond to the module in use.
+    */
+    static const String getDefaultXMLParserName();
 
 	/*************************************************************************
 		Input injection interface
@@ -899,26 +859,29 @@ private:
 	SystemKey	keyCodeToSyskey(Key::Scan key, bool direction);
 
 
-	/*!
-	\brief
-		Method to do the work of the constructor
-	*/
-       void	constructor_impl(Renderer* renderer, ResourceProvider* resourceProvider, XMLParser* xmlParser, ScriptModule* scriptModule, const String& configFile, const String& logFile);
-
-
-	/*!
-	\brief
-		add events for the System object
-	*/
-	void	addSystemEvents(void);
-
-
 	/*!
 	\brief
 		Handler method for display size change notifications
 	*/
 	bool	handleDisplaySizeChange(const EventArgs& e);
 
+    //! output the standard log header
+    void outputLogHeader();
+
+    //! adds factories for all the basic window types
+    void addStandardWindowFactories();
+
+    //! create the other core system singleton objects (except the logger)
+    void createSingletons();
+
+    //! cleanup the core system singleton objects
+    void destroySingletons();
+
+    //! handle creation and initialisation of the XML parser.
+    void setupXMLParser();
+
+    //! handle cleanup of the XML parser
+    void cleanupXMLParser();
 
 	/*************************************************************************
 		Handlers for System events
@@ -1013,9 +976,15 @@ private:
 
     XMLParser*  d_xmlParser;        //!< XMLParser object we use to process xml files.
     bool        d_ourXmlParser;     //!< true when we created the xml parser.
+    DynamicModule* d_parserModule;  //! pointer to parser module.
 
     Tooltip* d_defaultTooltip;      //!< System default tooltip object.
     bool     d_weOwnTooltip;        //!< true if System created the custom Tooltip.
+
+    //!< Holds the connection to Renderer::EventDisplaySizeChanged so we can unsubscribe before we die.
+    Event::Connection d_rendererCon;
+
+    static String   d_defaultXMLParserName; //!< Holds name of default XMLParser
 };
 
 } // End of  CEGUI namespace section
