@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIWindowManager.h
 	created:	21/2/2004
 	author:		Paul D Turner
 	
 	purpose:	Defines the interface for the WindowManager object
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUIWindowManager_h_
 #define _CEGUIWindowManager_h_
 
@@ -108,26 +112,6 @@ public:
 	~WindowManager(void);
 
 
-	/*!
-	\brief
-		Return singleton WindowManager object
-
-	\return
-		Singleton WindowManager object
-	*/
-	static	WindowManager&	getSingleton(void);
-
-
-	/*!
-	\brief
-		Return pointer to singleton WindowManager object
-
-	\return
-		Pointer to singleton WindowManager object
-	*/
-	static	WindowManager*	getSingletonPtr(void);
-
-
 	/*************************************************************************
 		Window Related Methods
 	*************************************************************************/
@@ -252,7 +236,7 @@ public:
 	\exception FileIOException			thrown if something goes wrong while processing the file \a filename.
 	\exception InvalidRequestException	thrown if \a filename appears to be invalid.
 	*/
-	Window*	loadWindowLayout(const String& filename, const String& name_prefix = "", const String& resourceGroup = "", PropertyCallback* callback = NULL, void* userdata = NULL);
+	Window*	loadWindowLayout(const String& filename, const String& name_prefix = "", const String& resourceGroup = "", PropertyCallback* callback = 0, void* userdata = 0);
 
     /*!
     \brief
@@ -348,6 +332,30 @@ public:
     */
     void renameWindow(Window* window, const String& new_name);
 
+    /*!
+    \brief
+        Returns the default resource group currently set for layouts.
+
+    \return
+        String describing the default resource group identifier that will be
+        used when loading layouts.
+    */
+    static const String& getDefaultResourceGroup()
+        { return d_defaultResourceGroup; }
+
+    /*!
+    \brief
+        Sets the default resource group to be used when loading layouts
+
+    \param resourceGroup
+        String describing the default resource group identifier to be used.
+
+    \return
+        Nothing.
+    */
+    static void setDefaultResourceGroup(const String& resourceGroup)
+        { d_defaultResourceGroup = resourceGroup; }
+
 private:
     /*************************************************************************
         Implementation Methods
@@ -367,13 +375,14 @@ private:
 	/*************************************************************************
 		Implementation Data
 	*************************************************************************/
-	typedef std::map<String, Window*>			WindowRegistry;				//!< Type used to implement registry of Window objects
+	typedef std::map<String, Window*, String::FastLessCompare>			WindowRegistry;				//!< Type used to implement registry of Window objects
     typedef std::vector<Window*>    WindowVector;   //!< Type to use for a collection of Window pointers.
 
 	WindowRegistry			d_windowRegistry;			//!< The container that forms the Window registry
     WindowVector    d_deathrow;     //!< Collection of 'destroyed' windows.
 
     unsigned long   d_uid_counter;  //!< Counter used to generate unique window names.
+    static String d_defaultResourceGroup;   //!< holds default resource group
 
 public:
 	/*************************************************************************
