@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUICombobox.h
 	created:	13/4/2004
 	author:		Paul D Turner
 	
 	purpose:	Interface to base class for Combobox widget
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUICombobox_h_
 #define _CEGUICombobox_h_
 
@@ -49,6 +53,7 @@ class CEGUIEXPORT Combobox : public Window
 {
 public:
 	static const String EventNamespace;				//!< Namespace for global events
+    static const String WidgetTypeName;             //!< Window factory name
 
 	/*************************************************************************
 		Constants
@@ -76,6 +81,12 @@ public:
 	static const String EventDropListRemoved;				//!< Event triggered when the drop-down list is removed / hidden.
 	static const String EventListSelectionAccepted;		//!< Event triggered when the user accepts a selection from the drop-down list
 
+    /*************************************************************************
+        Child Widget name suffix constants
+    *************************************************************************/
+    static const String EditboxNameSuffix;          //!< Widget name suffix for the editbox component.
+    static const String DropListNameSuffix;   //!< Widget name suffix for the drop list component.
+    static const String ButtonNameSuffix;           //!< Widget name suffix for the button component.
 
 	/*!
 	\brief
@@ -111,6 +122,44 @@ public:
 	bool	isDropDownListVisible(void) const;
 
 
+    /*!
+    \brief
+        Return a pointer to the Editbox component widget for this Combobox.
+
+    \return
+        Pointer to an Editbox object.
+
+    \exception UnknownObjectException
+        Thrown if the Editbox component does not exist.
+    */
+    Editbox* getEditbox() const;
+
+    /*!
+    \brief
+        Return a pointer to the PushButton component widget for this Combobox.
+
+    \return
+        Pointer to a PushButton object.
+
+    \exception UnknownObjectException
+        Thrown if the PushButton component does not exist.
+    */
+    PushButton* getPushButton() const;
+
+    /*!
+    \brief
+        Return a pointer to the ComboDropList component widget for this
+        Combobox.
+
+    \return
+        Pointer to an ComboDropList object.
+
+    \exception UnknownObjectException
+        Thrown if the ComboDropList component does not exist.
+    */
+    ComboDropList* getDropList() const;
+
+
 	/*************************************************************************
 		Editbox Accessors
 	*************************************************************************/
@@ -224,50 +273,6 @@ public:
 	size_t	getMaxTextLength(void) const;
 
 
-	/*!
-	\brief
-		return the currently set colour to be used for rendering Editbox text in the
-		normal, unselected state.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	colour	getNormalTextColour(void) const;
-
-
-	/*!
-	\brief
-		return the currently set colour to be used for rendering the Editbox text when within the
-		selected region.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	colour	getSelectedTextColour(void) const;
-
-
-	/*!
-	\brief
-		return the currently set colour to be used for rendering the Editbox selection highlight
-		when the Editbox is active.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	colour	getNormalSelectBrushColour(void) const;
-
-
-	/*!
-	\brief
-		return the currently set colour to be used for rendering the Editbox selection highlight
-		when the Editbox is inactive.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	colour	getInactiveSelectBrushColour(void) const;
-
-
 	/*************************************************************************
 		List Accessors
 	*************************************************************************/
@@ -412,7 +417,7 @@ public:
 	\return
 		Nothing
 	*/
-	virtual void	initialise(void);
+	virtual void	initialiseComponents(void);
 
 
 	/*!
@@ -532,58 +537,6 @@ public:
 	void	setMaxTextLength(size_t max_len);
 
 
-	/*!
-	\brief
-		Set the colour to be used for rendering Editbox text in the normal, unselected state.
-
-	\param col
-		colour value describing the ARGB colour that is to be used.
-
-	\return
-		Nothing.
-	*/
-	void	setNormalTextColour(colour col);
-
-
-	/*!
-	\brief
-		Set the colour to be used for rendering the Editbox text when within the
-		selected region.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	void	setSelectedTextColour(colour col);
-
-
-	/*!
-	\brief
-		Set the colour to be used for rendering the Editbox selection highlight
-		when the Editbox is active.
-
-	\param col
-		colour value describing the ARGB colour that is to be used.
-
-	\return
-		Nothing.
-	*/
-	void	setNormalSelectBrushColour(colour col);
-
-
-	/*!
-	\brief
-		Set the colour to be used for rendering the Editbox selection highlight
-		when the Editbox is inactive.
-
-	\param col
-		colour value describing the ARGB colour that is to be used.
-
-	\return
-		Nothing.
-	*/
-	void	setInactiveSelectBrushColour(colour col);
-
-
 	/*!
 	\brief
 		Activate the edit box component of the Combobox.
@@ -785,46 +738,6 @@ protected:
 	/*************************************************************************
 		Implementation Methods
 	*************************************************************************/
-	/*!
-	\brief
-		Add combo box specific events
-	*/
-	void	addComboboxEvents(void);
-
-
-	/*!
-	\brief
-		Create, initialise, and return a pointer to an Editbox widget to be used as part
-		of this Combobox.
-
-	\return
-		Pointer to an Editbox derived class.
-	*/
-	virtual	Editbox*	createEditbox(const String& name) const		= 0;
-
-
-	/*!
-	\brief
-		Create, initialise, and return a pointer to a PushButton widget to be used as part
-		of this Combobox.
-
-	\return
-		Pointer to a PushButton derived class.
-	*/
-	virtual	PushButton*	createPushButton(const String& name) const	= 0;
-
-
-	/*!
-	\brief
-		Create, initialise, and return a pointer to a ComboDropList widget to be used as part
-		of this Combobox.
-
-	\return
-		Pointer to a ComboDropList derived class.
-	*/
-	virtual	ComboDropList*	createDropList(const String& name) const	= 0;
-
-
 	/*!
 	\brief
 		Handler function for button clicks.
@@ -855,7 +768,7 @@ protected:
 
 	/*!
 	\brief
-		Return whether this window was inherited from the given class name at some point in the inheritance heirarchy.
+		Return whether this window was inherited from the given class name at some point in the inheritance hierarchy.
 
 	\param class_name
 		The class name that is to be checked.
@@ -865,7 +778,7 @@ protected:
 	*/
 	virtual bool	testClassName_impl(const String& class_name) const
 	{
-		if (class_name==(const utf8*)"Combobox")	return true;
+		if (class_name=="Combobox")	return true;
 		return Window::testClassName_impl(class_name);
 	}
 
@@ -1026,9 +939,6 @@ protected:
 	/*************************************************************************
 		Implementation Data
 	*************************************************************************/
-	Editbox*		d_editbox;		//!< Editbox widget sub-component.
-	ComboDropList*	d_droplist;		//!< ComboDropList widget sub-component.	
-	PushButton*		d_button;		//!< PushButton widget sub-component.
 	bool			d_singleClickOperation;		//!< true if user can show and select from list in a single click.
 
 
@@ -1042,10 +952,6 @@ private:
 	static ComboboxProperties::EditSelectionStart			d_selStartProperty;
 	static ComboboxProperties::EditSelectionLength			d_selLengthProperty;
 	static ComboboxProperties::MaxEditTextLength			d_maxTextLengthProperty;
-	static ComboboxProperties::NormalEditTextColour			d_normalTextColourProperty;
-	static ComboboxProperties::SelectedEditTextColour		d_selectedTextColourProperty;
-	static ComboboxProperties::ActiveEditSelectionColour	d_activeSelectionColourProperty;
-	static ComboboxProperties::InactiveEditSelectionColour	d_inactiveSelectionColourProperty;
 	static ComboboxProperties::SortList						d_sortProperty;
 	static ComboboxProperties::ForceVertScrollbar			d_forceVertProperty;
 	static ComboboxProperties::ForceHorzScrollbar			d_forceHorzProperty;
