@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIEditbox.h
 	created:	13/4/2004
 	author:		Paul D Turner
 	
 	purpose:	Interface to base class for Editbox widget
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUIEditbox_h_
 #define _CEGUIEditbox_h_
 
@@ -43,6 +47,32 @@ namespace CEGUI
 // forward declare implementation data type
 struct RegexValidator;
 
+/*!
+\brief
+    Base class for the EditboxWindowRenderer class
+*/
+class CEGUIEXPORT EditboxWindowRenderer : public WindowRenderer
+{
+public:
+    /*!
+    \brief
+        Constructor
+    */
+    EditboxWindowRenderer(const String& name);
+
+    /*!
+    \brief
+        Return the text code point index that is rendered closest to screen position \a pt.
+
+    \param pt
+        Point object describing a position on the screen in pixels.
+
+    \return
+        Code point index into the text that is rendered closest to screen position \a pt.
+    */
+    virtual size_t getTextIndexFromPosition(const Point& pt) const = 0;
+};
+
 
 /*!
 \brief
@@ -52,17 +82,7 @@ class CEGUIEXPORT Editbox : public Window
 {
 public:
 	static const String EventNamespace;				//!< Namespace for global events
-
-
-	/*************************************************************************
-		Constants
-	*************************************************************************/
-	// default colours
-	static const argb_t	DefaultNormalTextColour;			//!< Colour applied to normal unselected text.
-	static const argb_t	DefaultSelectedTextColour;			//!< Colour applied to selected text.
-	static const argb_t	DefaultNormalSelectionColour;		//!< Colour applied to normal selection brush.
-	static const argb_t	DefaultInactiveSelectionColour;		//!< Colour applied to selection brush when widget is inactive.
-
+    static const String WidgetTypeName;             //!< Window factory name
 
 	/*************************************************************************
 		Event name constants
@@ -215,50 +235,6 @@ public:
 	size_t	getMaxTextLength(void) const		{return d_maxTextLen;}
 
 
-	/*!
-	\brief
-		return the currently set colour to be used for rendering Editbox text in the
-		normal, unselected state.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	colour	getNormalTextColour(void) const				{return d_normalTextColour;}
-
-
-	/*!
-	\brief
-		return the currently set colour to be used for rendering the Editbox text when within the
-		selected region.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	colour	getSelectedTextColour(void) const			{return d_selectTextColour;}
-
-
-	/*!
-	\brief
-		return the currently set colour to be used for rendering the Editbox selection highlight
-		when the Editbox is active.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	colour	getNormalSelectBrushColour(void) const		{return d_selectBrushColour;}
-
-
-	/*!
-	\brief
-		return the currently set colour to be used for rendering the Editbox selection highlight
-		when the Editbox is inactive.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	colour	getInactiveSelectBrushColour(void) const	{return d_inactiveSelectBrushColour;}
-
-
 	/*************************************************************************
 		Manipulators
 	*************************************************************************/
@@ -370,58 +346,6 @@ public:
 	void	setMaxTextLength(size_t max_len);
 
 
-	/*!
-	\brief
-		Set the colour to be used for rendering Editbox text in the normal, unselected state.
-
-	\param col
-		colour value describing the ARGB colour that is to be used.
-
-	\return
-		Nothing.
-	*/
-	void	setNormalTextColour(const colour& col);
-
-
-	/*!
-	\brief
-		Set the colour to be used for rendering the Editbox text when within the
-		selected region.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	void	setSelectedTextColour(const colour& col);
-
-
-	/*!
-	\brief
-		Set the colour to be used for rendering the Editbox selection highlight
-		when the Editbox is active.
-
-	\param col
-		colour value describing the ARGB colour that is to be used.
-
-	\return
-		Nothing.
-	*/
-	void	setNormalSelectBrushColour(const colour& col);
-
-
-	/*!
-	\brief
-		Set the colour to be used for rendering the Editbox selection highlight
-		when the Editbox is inactive.
-
-	\param col
-		colour value describing the ARGB colour that is to be used.
-
-	\return
-		Nothing.
-	*/
-	void	setInactiveSelectBrushColour(const colour& col);
-
-
 	/*************************************************************************
 		Construction / Destruction
 	*************************************************************************/
@@ -443,13 +367,6 @@ protected:
 	/*************************************************************************
 		Implementation functions
 	*************************************************************************/
-	/*!
-	\brief
-		Add edit box specific events
-	*/
-	void	addEditboxEvents(void);
-
-
 	/*!
 	\brief
 		Return the text code point index that is rendered closest to screen position \a pt.
@@ -460,7 +377,20 @@ protected:
 	\return
 		Code point index into the text that is rendered closest to screen position \a pt.
 	*/
-	virtual	size_t	getTextIndexFromPosition(const Point& pt) const		= 0;
+	size_t	getTextIndexFromPosition(const Point& pt) const;
+
+
+    /*!
+    \brief
+        Return the text code point index that is rendered closest to screen position \a pt.
+
+    \param pt
+        Point object describing a position on the screen in pixels.
+
+    \return
+        Code point index into the text that is rendered closest to screen position \a pt.
+    */
+    //virtual size_t  getTextIndexFromPosition_impl(const Point& pt) const = 0;
 
 
 	/*!
@@ -546,7 +476,7 @@ protected:
 
 	/*!
 	\brief
-		Return whether this window was inherited from the given class name at some point in the inheritance heirarchy.
+		Return whether this window was inherited from the given class name at some point in the inheritance hierarchy.
 
 	\param class_name
 		The class name that is to be checked.
@@ -556,10 +486,15 @@ protected:
 	*/
 	virtual bool	testClassName_impl(const String& class_name) const
 	{
-		if (class_name==(const utf8*)"Editbox")	return true;
+		if (class_name=="Editbox")	return true;
 		return Window::testClassName_impl(class_name);
 	}
-
+    
+    // validate window renderer
+    virtual bool    validateWindowRenderer(const String& name) const
+    {
+        return (name == "Editbox");
+    }
 
 	/*************************************************************************
 		New event handlers
@@ -674,13 +609,6 @@ protected:
 	bool	d_dragging;			//!< true when a selection is being dragged.
 	size_t	d_dragAnchorIdx;	//!< Selection index for drag selection anchor point.
 
-	// basic rendering colours
-	colour	d_normalTextColour;				//!< Text colour used normally.
-	colour	d_selectTextColour;				//!< Text colour used when text is highlighted
-	colour	d_selectBrushColour;			//!< Colour to apply to the selection brush.
-	colour	d_inactiveSelectBrushColour;	//!< Colour to apply to the selection brush when widget is inactive / read-only.
-
-
 private:
 	/*************************************************************************
 		Static Properties for this class
@@ -693,11 +621,6 @@ private:
 	static EditboxProperties::SelectionStart			d_selectionStartProperty;
 	static EditboxProperties::SelectionLength			d_selectionLengthProperty;
 	static EditboxProperties::MaxTextLength				d_maxTextLengthProperty;
-	static EditboxProperties::NormalTextColour			d_normalTextColourProperty;
-	static EditboxProperties::SelectedTextColour		d_selectedTextColourProperty;
-	static EditboxProperties::ActiveSelectionColour		d_activeSelectionColourProperty;
-	static EditboxProperties::InactiveSelectionColour	d_inactiveSelectionColourProperty;
-
 
 	/*************************************************************************
 		Private methods
