@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIFrameWindowProperties.h
 	created:	10/7/2004
 	author:		Paul D Turner
 	
 	purpose:	Interface to properties for FrameWIndow class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUIFrameWindowProperties_h_
 #define _CEGUIFrameWindowProperties_h_
 
@@ -239,7 +243,7 @@ public:
 	SizingBorderThickness() : Property(
 		"SizingBorderThickness",
 		"Property to get/set the setting for the sizing border thickness.  Value is a float specifying the border thickness in pixels.",
-		"8.000000")
+		"8")
 	{}
 
 	String	get(const PropertyReceiver* receiver) const;
@@ -247,54 +251,6 @@ public:
 };
 
 
-/*!
-\brief
-	Property to access the Font set for the windows title bar.
-
-	\par Usage:
-		- Name: TitlebarFont
-		- Format: "[text]".
-
-	\par Where:
-		- [text] is the name of the Font to assign for this windows title bar.  The Font specified must already be loaded.
-*/
-class TitlebarFont : public Property
-{
-public:
-	TitlebarFont() : Property(
-		"TitlebarFont",
-		"Property to get/set the Font set for the windows title bar.  Value is the name of the font to use (must be loaded already).",
-		"")
-	{}
-
-	String	get(const PropertyReceiver* receiver) const;
-	void	set(PropertyReceiver* receiver, const String& value);
-};
-
-/*!
-\brief
-	Property to colour used for rendering the caption text.
-
-	\par Usage:
-		- Name: CaptionColour
-		- Format: "aarrggbb".
-
-	\par Where:
-		- aarrggbb is the ARGB colour value to be used.
-*/
-class CaptionColour : public Property
-{
-public:
-	CaptionColour() : Property(
-		"CaptionColour",
-		"Property to get/set the colour used for rendering the caption text.  Value is \"aarrggbb\" (hex).",
-		"FF000000")
-	{}
-
-	String	get(const PropertyReceiver* receiver) const;
-	void	set(PropertyReceiver* receiver, const String& value);
-};
-
 /*!
 \brief
    Property to access the N-S (up-down) sizing cursor image
