@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIListHeaderSegment.h
 	created:	15/6/2004
 	author:		Paul D Turner
 	
 	purpose:	Interface to list header segment class.
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUIListHeaderSegment_h_
 #define _CEGUIListHeaderSegment_h_
 
@@ -48,6 +52,7 @@ class CEGUIEXPORT ListHeaderSegment : public Window
 {
 public:
 	static const String EventNamespace;				//!< Namespace for global events
+    static const String WidgetTypeName;             //!< Window factory name
 
 
 	/*************************************************************************
@@ -143,6 +148,45 @@ public:
 	bool	isClickable(void) const		{return d_allowClicks;}
 
 
+    /*!
+    \brief
+        Return whether the segment is currently in its hovering state.
+    */
+    bool    isSegmentHovering(void) const  {return d_segmentHover;}
+
+
+    /*!
+    \brief
+        Return whether the segment is currently in its pushed state.
+    */
+    bool    isSegmentPushed(void) const  {return d_segmentPushed;}
+
+
+    /*!
+    \brief
+        Return whether the splitter is currently in its hovering state.
+    */
+    bool    isSplitterHovering(void) const  {return d_splitterHover;}
+
+
+    /*!
+    \brief
+        Return whether the segment is currently being drag-moved.
+    */
+    bool    isBeingDragMoved(void) const  {return d_dragMoving;}
+
+
+    /*!
+    \brief
+        Return whether the segment is currently being drag-moved.
+    */
+    bool    isBeingDragSized(void) const  {return d_dragSizing;}
+
+
+    const Image* getSizingCursorImage() const;
+    const Image* getMovingCursorImage() const;
+
+
 	/*************************************************************************
 		Manipulator Methods
 	*************************************************************************/
@@ -203,6 +247,12 @@ public:
 	void setClickable(bool setting);
 
 
+    void setSizingCursorImage(const Image* image);
+    void setSizingCursorImage(const String& imageset, const String& image);
+    void setMovingCursorImage(const Image* image);
+    void setMovingCursorImage(const String& imageset, const String& image);
+
+
 	/*************************************************************************
 		Construction & Destruction
 	*************************************************************************/
@@ -224,13 +274,6 @@ protected:
 	/*************************************************************************
 		Implementation Methods
 	*************************************************************************/
-	/*!
-	\brief
-		Add list header segment specific events
-	*/
-	void	addHeaderSegmentEvents(void);
-
-
 	/*!
 	\brief
 		Update state for drag sizing.
@@ -295,7 +338,7 @@ protected:
 
 	/*!
 	\brief
-		Return whether this window was inherited from the given class name at some point in the inheritance heirarchy.
+		Return whether this window was inherited from the given class name at some point in the inheritance hierarchy.
 
 	\param class_name
 		The class name that is to be checked.
@@ -305,7 +348,7 @@ protected:
 	*/
 	virtual bool	testClassName_impl(const String& class_name) const
 	{
-		if (class_name==(const utf8*)"ListHeaderSegment")	return true;
+		if (class_name=="ListHeaderSegment")	return true;
 		return Window::testClassName_impl(class_name);
 	}
 
@@ -416,7 +459,6 @@ protected:
 	Point	d_dragPosition;		//!< position of dragged segment.
 	bool	d_allowClicks;		//!< true if the segment can be clicked.
 
-
 private:
 	/*************************************************************************
 		Static Properties for this class
@@ -425,6 +467,8 @@ private:
 	static ListHeaderSegmentProperties::Dragable		d_dragableProperty;
 	static ListHeaderSegmentProperties::Sizable			d_sizableProperty;
 	static ListHeaderSegmentProperties::SortDirection	d_sortDirectionProperty;
+    static ListHeaderSegmentProperties::SizingCursorImage   d_sizingCursorProperty;
+    static ListHeaderSegmentProperties::MovingCursorImage   d_movingCursorProperty;
 
 
 	/*************************************************************************
