@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIListbox.h
 	created:	13/4/2004
 	author:		Paul D Turner
 	
 	purpose:	Interface to base class for Listbox widget
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUIListbox_h_
 #define _CEGUIListbox_h_
 
@@ -42,6 +46,31 @@
 namespace CEGUI
 {
 
+/*!
+\brief
+    Base class for Listbox window renderer.
+*/
+class CEGUIEXPORT ListboxWindowRenderer : public WindowRenderer
+{
+public:
+    /*!
+    \brief
+        Constructor
+    */
+    ListboxWindowRenderer(const String& name);
+
+    /*!
+    \brief
+        Return a Rect object describing, in un-clipped pixels, the window relative area
+        that is to be used for rendering list items.
+
+    \return
+        Rect object describing the area of the Window to be used for rendering
+        list box items.
+    */
+    virtual Rect getListRenderArea(void) const = 0;
+};
+
 /*!
 \brief
 	Base class for standard Listbox widget.
@@ -50,7 +79,7 @@ class CEGUIEXPORT Listbox : public Window
 {
 public:
 	static const String EventNamespace;				//!< Namespace for global events
-
+    static const String WidgetTypeName;             //!< Window factory name
 
 	/*************************************************************************
 		Constants
@@ -63,6 +92,11 @@ public:
 	static const String EventVertScrollbarModeChanged;	//!< Event triggered when the vertical scroll bar 'force' setting changes.
 	static const String EventHorzScrollbarModeChanged;	//!< Event triggered when the horizontal scroll bar 'force' setting changes.
 
+    /*************************************************************************
+        Child Widget name suffix constants
+    *************************************************************************/
+    static const String VertScrollbarNameSuffix;   //!< Widget name suffix for the vertical scrollbar component.
+    static const String HorzScrollbarNameSuffix;   //!< Widget name suffix for the horizontal scrollbar component.
 
 	/*************************************************************************
 		Accessor Methods
@@ -245,7 +279,7 @@ public:
 	\return
 		Nothing
 	*/
-	virtual void	initialise(void);
+	virtual void	initialiseComponents(void);
 
 
 	/*!
@@ -460,6 +494,59 @@ public:
 	void	ensureItemIsVisible(const ListboxItem* item);
 
 
+    /*!
+    \brief
+        Return a Rect object describing, in un-clipped pixels, the window relative area
+        that is to be used for rendering list items.
+
+    \return
+        Rect object describing the area of the Window to be used for rendering
+        list box items.
+    */
+    virtual Rect    getListRenderArea(void) const;
+
+
+    /*!
+    \brief
+        Return a pointer to the vertical scrollbar component widget for this
+        Listbox.
+
+    \return
+        Pointer to a Scrollbar object.
+
+    \exception UnknownObjectException
+        Thrown if the vertical Scrollbar component does not exist.
+    */
+    Scrollbar* getVertScrollbar() const;
+
+    /*!
+    \brief
+        Return a pointer to the horizontal scrollbar component widget for this
+        Listbox.
+
+    \return
+        Pointer to a Scrollbar object.
+
+    \exception UnknownObjectException
+        Thrown if the horizontal Scrollbar component does not exist.
+    */
+    Scrollbar* getHorzScrollbar() const;
+
+
+    /*!
+    \brief
+        Return the sum of all item heights
+    */
+    float   getTotalItemsHeight(void) const;
+
+
+    /*!
+    \brief
+        Return the width of the widest item
+    */
+    float   getWidestItemWidth(void) const;
+
+
 	/*************************************************************************
 		Construction and Destruction
 	*************************************************************************/
@@ -490,58 +577,12 @@ protected:
 		Rect object describing the area of the Window to be used for rendering
 		list box items.
 	*/
-	virtual	Rect	getListRenderArea(void) const		= 0;
-
-
-	/*!
-	\brief
-		create and return a pointer to a Scrollbar widget for use as vertical scroll bar
-
-	\param name
-	   String holding the name to be given to the created widget component.
-
-	\return
-		Pointer to a Scrollbar to be used for scrolling the list vertically.
-	*/
-	virtual Scrollbar*	createVertScrollbar(const String& name) const		= 0;
- 
-
-	/*!
-	\brief
-		create and return a pointer to a Scrollbar widget for use as horizontal scroll bar
-
-	\param name
-	   String holding the name to be given to the created widget component.
-
-	\return
-		Pointer to a Scrollbar to be used for scrolling the list horizontally.
-	*/
-	virtual Scrollbar*	createHorzScrollbar(const String& name) const		= 0;
-
-
-	/*!
-	\brief
-		Perform caching of the widget control frame and other 'static' areas.  This
-		method should not render the actual items.  Note that the items are typically
-		rendered to layer 3, other layers can be used for rendering imagery behind and
-		infront of the items.
-
-	\return
-		Nothing.
-	*/
-	virtual	void cacheListboxBaseImagery() = 0;
+	//virtual	Rect	getListRenderArea_impl(void) const = 0;
 
 
 	/*************************************************************************
 		Implementation Functions
 	*************************************************************************/
-	/*!
-	\brief
-		Add list box specific events
-	*/
-	void	addListboxEvents(void);
-
-
 	/*!
 	\brief
 		display required integrated scroll bars according to current state of the list box and update their values.
@@ -556,20 +597,6 @@ protected:
 	void	selectRange(size_t start, size_t end);
 
 
-	/*!
-	\brief
-		Return the sum of all item heights
-	*/
-	float	getTotalItemsHeight(void) const;
-
-
-	/*!
-	\brief
-		Return the width of the widest item
-	*/
-	float	getWidestItemWidth(void) const;
-
-
 	/*!
 	\brief
 		Clear the selected state for all items (implementation)
@@ -607,7 +634,7 @@ protected:
 
 	/*!
 	\brief
-		Return whether this window was inherited from the given class name at some point in the inheritance heirarchy.
+		Return whether this window was inherited from the given class name at some point in the inheritance hierarchy.
 
 	\param class_name
 		The class name that is to be checked.
@@ -617,7 +644,7 @@ protected:
 	*/
 	virtual bool	testClassName_impl(const String& class_name) const
 	{
-		if (class_name==(const utf8*)"Listbox")	return true;
+		if (class_name=="Listbox")	return true;
 		return Window::testClassName_impl(class_name);
 	}
 
@@ -627,8 +654,12 @@ protected:
     */
     bool handle_scrollChange(const EventArgs& args);
 
-    // overridden from Window base class.
-    void populateRenderCache();
+
+    // validate window renderer
+    virtual bool validateWindowRenderer(const String& name) const
+    {
+        return (name == "Listbox");
+    }
 
 
 	/*************************************************************************
@@ -694,11 +725,10 @@ protected:
 	bool	d_forceVertScroll;		//!< true if vertical scrollbar should always be displayed
 	bool	d_forceHorzScroll;		//!< true if horizontal scrollbar should always be displayed
 	bool	d_itemTooltips;			//!< true if each item should have an individual tooltip
-	Scrollbar*	d_vertScrollbar;	//!< vertical scroll-bar widget
-	Scrollbar*	d_horzScrollbar;	//!< horizontal scroll-bar widget
 	LBItemList	d_listItems;		//!< list of items in the list box.
 	ListboxItem*	d_lastSelected;	//!< holds pointer to the last selected item (used in range selections)
 
+    friend class ListboxWindowRenderer;
 
 private:
 	/*************************************************************************
