@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIMultiColumnList.h
 	created:	13/4/2004
 	author:		Paul D Turner
 	
 	purpose:	Interface to base class for MultiColumnList widget
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUIMultiColumnList_h_
 #define _CEGUIMultiColumnList_h_
 
@@ -63,6 +67,30 @@ struct CEGUIEXPORT MCLGridRef
 	bool operator!=(const MCLGridRef& rhs) const;
 };
 
+/*!
+\brief
+    Base class for the multi column list window renderer.
+*/
+class CEGUIEXPORT MultiColumnListWindowRenderer : public WindowRenderer
+{
+public:
+    /*!
+    \brief
+        Constructor
+    */
+    MultiColumnListWindowRenderer(const String& name);
+
+    /*!
+    \brief
+        Return a Rect object describing, in un-clipped pixels, the window relative area
+        that is to be used for rendering list items.
+
+    \return
+        Rect object describing the area of the Window to be used for rendering
+        list box items.
+    */
+    virtual Rect    getListRenderArea(void) const = 0;
+};
 
 /*!
 \brief
@@ -72,7 +100,7 @@ class CEGUIEXPORT MultiColumnList : public Window
 {
 public:
 	static const String EventNamespace;				//!< Namespace for global events
-
+    static const String WidgetTypeName;             //!< Window factory name
 
 	/*************************************************************************
 		Constants
@@ -90,6 +118,12 @@ public:
 	static const String EventListColumnSized;				//!< Event fired when the width of a column in the list changes.
 	static const String EventListColumnMoved;				//!< Event fired when the column order changes.
 
+    /*************************************************************************
+        Child Widget name suffix constants
+    *************************************************************************/
+    static const String VertScrollbarNameSuffix;   //!< Widget name suffix for the vertical scrollbar component.
+    static const String HorzScrollbarNameSuffix;   //!< Widget name suffix for the horizontal scrollbar component.
+    static const String ListHeaderNameSuffix;      //!< Widget name suffix for the list header component.
 
 	/*************************************************************************
 		Enumerations
@@ -215,9 +249,9 @@ public:
 		Return the total width of all column headers.
 
 	\return
-		Sum total of all the column header widths in whichever metrics system is active.
+		Sum total of all the column header widths as a UDim.
 	*/
-	float	getTotalColumnHeadersWidth(void) const;
+	UDim getTotalColumnHeadersWidth(void) const;
 
 
 	/*!
@@ -228,11 +262,11 @@ public:
 		Zero based column index of the column whos width is to be returned.
 
 	\return
-		Width of the column header at the zero based column index specified by \a col_idx, in whichever is the active metrics system.
+		Width of the column header at the zero based column index specified by \a col_idx, as a UDim
 
 	\exception InvalidRequestException	thrown if \a column is out of range.
 	*/
-	float	getColumnHeaderWidth(uint col_idx) const;
+	UDim getColumnHeaderWidth(uint col_idx) const;
 
 
 	/*!
@@ -605,6 +639,75 @@ public:
 	uint	getRowWithID(uint row_id) const;
 
 
+    /*!
+    \brief
+        Return a Rect object describing, in un-clipped pixels, the window relative area
+        that is to be used for rendering list items.
+
+    \return
+        Rect object describing the area of the Window to be used for rendering
+        list box items.
+    */
+    Rect    getListRenderArea(void) const;
+
+
+    /*!
+    \brief
+        Return a pointer to the vertical scrollbar component widget for this
+        MultiColumnList.
+
+    \return
+        Pointer to a Scrollbar object.
+
+    \exception UnknownObjectException
+        Thrown if the vertical Scrollbar component does not exist.
+    */
+    Scrollbar* getVertScrollbar() const;
+
+    /*!
+    \brief
+        Return a pointer to the horizontal scrollbar component widget for this
+        MultiColumnList.
+
+    \return
+        Pointer to a Scrollbar object.
+
+    \exception UnknownObjectException
+        Thrown if the horizontal Scrollbar component does not exist.
+    */
+    Scrollbar* getHorzScrollbar() const;
+
+    /*!
+    \brief
+        Return a pointer to the list header component widget for this
+        MultiColumnList.
+
+    \return
+        Pointer to a ListHeader object.
+
+    \exception UnknownObjectException
+        Thrown if the list header component does not exist.
+    */
+    ListHeader* getListHeader() const;
+
+    /*!
+    \brief
+        Return the sum of all row heights in pixels.
+    */
+    float   getTotalRowsHeight(void) const;
+
+    /*!
+    \brief
+        Return the pixel width of the widest item in the given column
+    */
+    float   getWidestColumnItemWidth(uint col_idx) const;
+
+    /*!
+    \brief
+        Return, in pixels, the height of the highest item in the given row.
+    */
+    float   getHighestRowItemHeight(uint row_idx) const;
+
 	/*************************************************************************
 		Manipulator Methods
 	*************************************************************************/
@@ -618,7 +721,7 @@ public:
 	\return
 		Nothing
 	*/
-	virtual void	initialise(void);
+	virtual void	initialiseComponents(void);
 
 
 	/*!
@@ -641,12 +744,12 @@ public:
 		ID code to be assigned to the column header.
 
 	\param width
-		Initial width to be set for the column using the active metrics mode for this window.
+		UDim describing the initial width to be set for the column.
 
 	\return
 		Nothing.
 	*/
-	void	addColumn(const String& text, uint col_id, float width);
+	void	addColumn(const String& text, uint col_id, const UDim& width);
 
 
 	/*!
@@ -660,7 +763,7 @@ public:
 		ID code to be assigned to the column header.
 
 	\param width
-		Initial width to be set for the column using the active metrics mode for this window.
+		UDim describing the initial width to be set for the column.
 
 	\param position
 		Zero based index where the column is to be inserted.  If this is greater than the current
@@ -669,7 +772,7 @@ public:
 	\return
 		Nothing.
 	*/
-	void	insertColumn(const String& text, uint col_id, float width, uint position);
+	void	insertColumn(const String& text, uint col_id, const UDim& width, uint position);
 
 
 	/*!
@@ -1090,14 +1193,14 @@ public:
 		Zero based column index of the column whos width is to be set.
 
 	\param width
-		float value specifying the new width for the column using the active metrics system.
+		UDim value specifying the new width for the column.
 
 	\return
 		Nothing.
 
 	\exception InvalidRequestException	thrown if \a column is out of range.
 	*/
-	void	setColumnHeaderWidth(uint col_idx, float width);
+	void	setColumnHeaderWidth(uint col_idx, const UDim& width);
 
 
 	/*!
@@ -1203,74 +1306,12 @@ protected:
 		Rect object describing the area of the Window to be used for rendering
 		list box items.
 	*/
-	virtual	Rect	getListRenderArea(void) const		= 0;
-
-
-	/*!
-	\brief
-		create and return a pointer to a ListHeaer widget for use as the column headers.
-
-	\param name
-	   String holding the name to be assigned to the created component.
-
-	\return
-		Pointer to a ListHeader based object.
-	*/
-	virtual ListHeader*	createListHeader(const String& name) const		= 0;
- 
-
-	/*!
-	\brief
-		create and return a pointer to a Scrollbar widget for use as vertical scroll bar
-
-	\param name
-	   String holding the name to be assigned to the created component.
-
-	\return
-		Pointer to a Scrollbar to be used for scrolling the list vertically.
-	*/
-	virtual Scrollbar*	createVertScrollbar(const String& name) const		= 0;
- 
-
-	/*!
-	\brief
-		create and return a pointer to a Scrollbar widget for use as horizontal scroll bar
-
-	\param name
-	   String holding the name to be assigned to the created component.
-
-	\return
-		Pointer to a Scrollbar to be used for scrolling the list horizontally.
-	*/
-	virtual Scrollbar*	createHorzScrollbar(const String& name) const		= 0;
-
-
-	/*!
-	\brief
-		Perform rendering of the widget control frame and other 'static' areas.  This
-		method should not render the actual items.  Note that the items are typically
-		rendered to layer 3, other layers can be used for rendering imagery behind and
-		infront of the items.
-
-	\return
-		Nothing.
-	*/
-	virtual	void	cacheListboxBaseImagery()		= 0;
+	//virtual	Rect	getListRenderArea_impl(void) const		= 0;
 
 
 	/*************************************************************************
 		Implementation Functions
 	*************************************************************************/
-	/*!
-	\brief
-		Add multi column list box specific events
-	*/
-	void	addMultiColumnListboxEvents(void);
-
-    // overridden from base class.
-	virtual	void populateRenderCache();
-
-
 	/*!
 	\brief
 		display required integrated scroll bars according to current state of the list box and update their values.
@@ -1285,27 +1326,6 @@ protected:
 	bool	selectRange(const MCLGridRef& start, const MCLGridRef& end);
 
 
-	/*!
-	\brief
-		Return the sum of all row heights
-	*/
-	float	getTotalRowsHeight(void) const;
-
-
-	/*!
-	\brief
-		Return the width of the widest item in the given column
-	*/
-	float	getWidestColumnItemWidth(uint col_idx) const;
-
-
-	/*!
-	\brief
-		Return the height of the highest item in the given row.
-	*/
-	float	getHighestRowItemHeight(uint row_idx) const;
-
-
 	/*!
 	\brief
 		Clear the selected state for all items (implementation)
@@ -1376,7 +1396,7 @@ protected:
 
 	/*!
 	\brief
-		Return whether this window was inherited from the given class name at some point in the inheritance heirarchy.
+		Return whether this window was inherited from the given class name at some point in the inheritance hierarchy.
 
 	\param class_name
 		The class name that is to be checked.
@@ -1386,12 +1406,19 @@ protected:
 	*/
 	virtual bool	testClassName_impl(const String& class_name) const
 	{
-		if (class_name==(const utf8*)"MultiColumnList")	return true;
+		if (class_name=="MultiColumnList")	return true;
 		return Window::testClassName_impl(class_name);
 	}
+    
 
     // overrides function in base class.
-    int writePropertiesXML(OutStream& out_stream) const;
+    virtual bool validateWindowRenderer(const String& name) const
+    {
+        return (name == "MultiColumnList");
+    }
+
+    // overrides function in base class.
+    int writePropertiesXML(XMLSerializer& xml_stream) const;
 
 	/*************************************************************************
 		New event handlers for multi column list
@@ -1523,10 +1550,7 @@ protected:
 	/*************************************************************************
 		Implementation Data
 	*************************************************************************/
-	// component widgets and settings.
-	Scrollbar*	d_vertScrollbar;	//!< vertical scroll-bar widget
-	Scrollbar*	d_horzScrollbar;	//!< horizontal scroll-bar widget
-	ListHeader*	d_header;			//!< The ListHeader attached to this multi-column list.
+	// scrollbar settings.
 	bool	d_forceVertScroll;		//!< true if vertical scrollbar should always be displayed
 	bool	d_forceHorzScroll;		//!< true if horizontal scrollbar should always be displayed
 
@@ -1541,10 +1565,14 @@ protected:
 	bool	d_useNominatedCol;		//!< true if we use a nominated col to select.
 	ListboxItem*	d_lastSelected;	//!< holds pointer to the last selected item (used in range selections)
 
+    uint    d_columnCount;          //!< keeps track of the number of columns.
+
 	// storage of items in the list box.
 	typedef std::vector<ListRow>		ListItemGrid;
 	ListItemGrid	d_grid;			//!< Holds the list box data.
 
+    friend class MultiColumnListWindowRenderer;
+
 
 private:
 	/*************************************************************************
