@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIMultiLineEditbox.h
 	created:	30/6/2004
 	author:		Paul D Turner
 	
 	purpose:	Interface to the Multi-lien edit box base class.
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUIMultiLineEditbox_h_
 #define _CEGUIMultiLineEditbox_h_
 
@@ -43,6 +47,30 @@
 // Start of CEGUI namespace section
 namespace CEGUI
 {
+/*!
+\brief
+    Base class for multi-line edit box window renderer objects.
+*/
+class CEGUIEXPORT MultiLineEditboxWindowRenderer : public WindowRenderer
+{
+public:
+    /*!
+    \brief
+        Constructor
+    */
+    MultiLineEditboxWindowRenderer(const String& name);
+
+    /*!
+    \brief
+        Return a Rect object describing, in un-clipped pixels, the window relative area
+        that the text should be rendered in to.
+
+    \return
+        Rect object describing the area of the Window to be used for rendering text.
+    */
+    virtual Rect getTextRenderArea(void) const = 0;
+};
+
 /*!
 \brief
 	Base class for the multi-line edit box widget.
@@ -51,7 +79,7 @@ class CEGUIEXPORT MultiLineEditbox : public Window
 {
 public:
 	static const String EventNamespace;				//!< Namespace for global events
-
+    static const String WidgetTypeName;             //!< Window factory name
 
 	/*************************************************************************
 		Constants
@@ -66,12 +94,27 @@ public:
 	static const String EventVertScrollbarModeChanged;	//!< Event triggered when the vertical scroll bar 'force' setting changes.
 	static const String EventHorzScrollbarModeChanged;	//!< Event triggered when the horizontal scroll bar 'force' setting changes.
 
-	// default colours
-	static const argb_t	DefaultNormalTextColour;			//!< Colour applied to normal unselected text.
-	static const argb_t	DefaultSelectedTextColour;			//!< Colour applied to selected text.
-	static const argb_t	DefaultNormalSelectionColour;		//!< Colour applied to normal selection brush.
-	static const argb_t	DefaultInactiveSelectionColour;		//!< Colour applied to selection brush when widget is inactive.
+    /*************************************************************************
+        Child Widget name suffix constants
+    *************************************************************************/
+    static const String VertScrollbarNameSuffix;   //!< Widget name suffix for the vertical scrollbar component.
+    static const String HorzScrollbarNameSuffix;   //!< Widget name suffix for the horizontal scrollbar component.
 
+    /*************************************************************************
+        Implementation struct
+    *************************************************************************/
+    /*!
+    \brief
+        struct used to store information about a formatted line within the
+        paragraph.
+    */
+    struct LineInfo
+    {
+        size_t  d_startIdx;     //!< Starting index for this line.
+        size_t  d_length;       //!< Code point length of this line.
+        float   d_extent;       //!< Rendered extent of this line.
+    };
+    typedef std::vector<LineInfo>   LineList;   //!< Type for collection of LineInfos.
 
 	/*************************************************************************
 		Accessor Functions
@@ -150,50 +193,6 @@ public:
 	size_t	getMaxTextLength(void) const		{return d_maxTextLen;}
 
 
-	/*!
-	\brief
-		return the currently set colour to be used for rendering edit box text in the
-		normal, unselected state.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	colour	getNormalTextColour(void) const				{return d_normalTextColour;}
-
-
-	/*!
-	\brief
-		return the currently set colour to be used for rendering the edit box text when within the
-		selected region.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	colour	getSelectedTextColour(void) const			{return d_selectTextColour;}
-
-
-	/*!
-	\brief
-		return the currently set colour to be used for rendering the edit box selection highlight
-		when the edit box is active.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	colour	getNormalSelectBrushColour(void) const		{return d_selectBrushColour;}
-
-
-	/*!
-	\brief
-		return the currently set colour to be used for rendering the edit box selection highlight
-		when the edit box is inactive.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	colour	getInactiveSelectBrushColour(void) const	{return d_inactiveSelectBrushColour;}
-
-
 	/*!
 	\brief
 		Return whether the text in the edit box will be word-wrapped.
@@ -205,6 +204,54 @@ public:
 	bool	isWordWrapped(void) const;
 
 
+    /*!
+    \brief
+        Return a pointer to the vertical scrollbar component widget for this
+        MultiLineEditbox.
+
+    \return
+        Pointer to a Scrollbar object.
+
+    \exception UnknownObjectException
+        Thrown if the vertical Scrollbar component does not exist.
+    */
+    Scrollbar* getVertScrollbar() const;
+
+
+    /*!
+    \brief
+        Return a pointer to the horizontal scrollbar component widget for this
+        MultiLineEditbox.
+
+    \return
+        Pointer to a Scrollbar object.
+
+    \exception UnknownObjectException
+        Thrown if the horizontal Scrollbar component does not exist.
+    */
+    Scrollbar* getHorzScrollbar() const;
+
+
+    /*!
+    \brief
+        Return a Rect object describing, in un-clipped pixels, the window relative area
+        that the text should be rendered in to.
+
+    \return
+        Rect object describing the area of the Window to be used for rendering text.
+    */
+    Rect    getTextRenderArea(void) const;
+
+    // get d_lines
+    const LineList& getFormattedLines(void) const   {return d_lines;}
+
+    /*!
+    \brief
+        Return the line number a given index falls on with the current formatting.  Will return last line
+        if index is out of range.
+    */
+    size_t  getLineNumberFromIndex(size_t index) const;
+
 	/*************************************************************************
 		Manipulators
 	*************************************************************************/
@@ -218,7 +265,7 @@ public:
 	\return
 		Nothing
 	*/
-	virtual void	initialise(void);
+	virtual void	initialiseComponents(void);
 
 
 	/*!
@@ -280,58 +327,6 @@ public:
 	void	setMaxTextLength(size_t max_len);
 
 
-	/*!
-	\brief
-		Set the colour to be used for rendering edit box text in the normal, unselected state.
-
-	\param col
-		colour value describing the ARGB colour that is to be used.
-
-	\return
-		Nothing.
-	*/
-	void	setNormalTextColour(const colour& col);
-
-
-	/*!
-	\brief
-		Set the colour to be used for rendering the edit box text when within the
-		selected region.
-
-	\return
-		colour value describing the ARGB colour that is currently set.
-	*/
-	void	setSelectedTextColour(const colour& col);
-
-
-	/*!
-	\brief
-		Set the colour to be used for rendering the edit box selection highlight
-		when the edit box is active.
-
-	\param col
-		colour value describing the ARGB colour that is to be used.
-
-	\return
-		Nothing.
-	*/
-	void	setNormalSelectBrushColour(const colour& col);
-
-
-	/*!
-	\brief
-		Set the colour to be used for rendering the edit box selection highlight
-		when the edit box is inactive.
-
-	\param col
-		colour value describing the ARGB colour that is to be used.
-
-	\return
-		Nothing.
-	*/
-	void	setInactiveSelectBrushColour(const colour& col);
-
-
 	/*!
 	\brief
 		Scroll the view so that the current carat position is visible.
@@ -352,6 +347,9 @@ public:
 	*/
 	void	setWordWrapping(bool setting);
 
+    // selection brush image property support
+    void setSelectionBrushImage(const Image* image);
+    const Image* getSelectionBrushImage() const;
 
 	/*************************************************************************
 		Construction and Destruction
@@ -382,75 +380,12 @@ protected:
 	\return
 		Rect object describing the area of the Window to be used for rendering text.
 	*/
-	virtual	Rect	getTextRenderArea(void) const		= 0;
-
-
-	/*!
-	\brief
-		create and return a pointer to a Scrollbar widget for use as vertical scroll bar
-
-	\param name
-	   String holding the name to be assigned to the component.
-
-	\return
-		Pointer to a Scrollbar to be used for scrolling vertically.
-	*/
-	virtual Scrollbar*	createVertScrollbar(const String& name) const		= 0;
- 
-
-	/*!
-	\brief
-		create and return a pointer to a Scrollbar widget for use as horizontal scroll bar
-
-	\param name
-	   String holding the name to be assigned to the component.
-
-	\return
-		Pointer to a Scrollbar to be used for scrolling horizontally.
-	*/
-	virtual Scrollbar*	createHorzScrollbar(const String& name) const		= 0;
-
-
-	/*!
-	\brief
-		Perform rendering of the widget control frame and other 'static' areas.  This
-		method should not render the actual text.  Note that the text will be rendered
-		to layer 4 and the selection brush to layer 3, other layers can be used for
-		rendering imagery behind and infront of the text & selection..
-
-	\return
-		Nothing.
-	*/
-	virtual	void	cacheEditboxBaseImagery()		= 0;
-
-
-	/*!
-	\brief
-		Render the carat.
-
-	\return
-		Nothing
-	*/
-	virtual void	cacheCaratImagery(const Rect& textArea)	= 0;
+	//virtual	Rect	getTextRenderArea_impl(void) const		= 0;
 
 
 	/*************************************************************************
 		Implementation Methods
 	*************************************************************************/
-	/*!
-	\brief
-		Add multi-line edit box specific events
-	*/
-	void	addMultiLineEditboxEvents(void);
-
-
-	/*!
-	\brief
-		Render text lines.
-	*/
-	void	cacheTextLines(const Rect& dest_area);
-
-
 	/*!
 	\brief
 		Format the text into lines as needed by the current formatting options.
@@ -471,10 +406,7 @@ protected:
 	size_t	getNextTokenLength(const String& text, size_t start_idx) const;
 
 
-	virtual	void populateRenderCache();
-
-
-	/*!
+    /*!
 	\brief
 		display required integrated scroll bars according to current state of the edit box and update their values.
 	*/
@@ -494,14 +426,6 @@ protected:
 	size_t	getTextIndexFromPosition(const Point& pt) const;
 
 
-	/*!
-	\brief
-		Return the line number a given index falls on with the current formatting.  Will return last line
-		if index is out of range.
-	*/
-	size_t	getLineNumberFromIndex(size_t index) const;
-
-
 	/*!
 	\brief
 		Clear the current selection setting
@@ -610,9 +534,23 @@ protected:
 	void	handleNewLine(uint sysKeys);
 
 
+    /*!
+    \brief
+        Processing to move caret one page up
+    */
+    void    handlePageUp(uint sysKeys);
+
+
+    /*!
+    \brief
+        Processing to move caret one page down
+    */
+    void    handlePageDown(uint sysKeys);
+
+
 	/*!
 	\brief
-		Return whether this window was inherited from the given class name at some point in the inheritance heirarchy.
+		Return whether this window was inherited from the given class name at some point in the inheritance hierarchy.
 
 	\param class_name
 		The class name that is to be checked.
@@ -622,8 +560,8 @@ protected:
 	*/
 	virtual bool	testClassName_impl(const String& class_name) const
 	{
-		if ((class_name==(const utf8*)"MultiLineEditBox") ||
-			(class_name==(const utf8*)"MultiLineEditbox"))
+		if ((class_name=="MultiLineEditBox") ||
+			(class_name=="MultiLineEditbox"))
 		{
 			return true;
 		}
@@ -637,6 +575,11 @@ protected:
     */
     bool handle_scrollChange(const EventArgs& args);
 
+    // validate window renderer
+    virtual bool validateWindowRenderer(const String& name) const
+    {
+        return (name == EventNamespace);
+    }
 
 	/*************************************************************************
 		New event handlers
@@ -713,22 +656,6 @@ protected:
 	virtual	void	onMouseWheel(MouseEventArgs& e);
 
 
-	/*************************************************************************
-		Implementation struct
-	*************************************************************************/
-	/*!
-	\brief
-		struct used to store information about a formatted line within the
-		paragraph.
-	*/
-	struct LineInfo
-	{
-		size_t	d_startIdx;		//!< Starting index for this line.
-		size_t	d_length;		//!< Code point length of this line.
-		float	d_extent;		//!< Rendered extent of this line.
-	};
-
-
 	/*************************************************************************
 		Implementation data
 	*************************************************************************/
@@ -740,27 +667,18 @@ protected:
 	bool	d_dragging;			//!< true when a selection is being dragged.
 	size_t	d_dragAnchorIdx;	//!< Selection index for drag selection anchor point.
 
-	typedef	std::vector<LineInfo>	LineList;	//!< Type for collection of LineInfos.
 	static String d_lineBreakChars;	//!< Holds what we consider to be line break characters.
 	bool		d_wordWrap;			//!< true when formatting uses word-wrapping.
 	LineList	d_lines;			//!< Holds the lines for the current formatting.
 	float		d_widestExtent;		//!< Holds the extent of the widest line as calculated in the last formatting pass.
 
-	// component widgets
-	Scrollbar*	d_vertScrollbar;	//!< Points to the vertical scroll bar widget.
-	Scrollbar*	d_horzScrollbar;	//!< Points to the horizontal scroll bar widget.
+	// component widget settings
 	bool	d_forceVertScroll;		//!< true if vertical scrollbar should always be displayed
 	bool	d_forceHorzScroll;		//!< true if horizontal scrollbar should always be displayed
 
 	// images
 	const Image*	d_selectionBrush;	//!< Image to use as the selection brush (should be set by derived class).
 
-	// basic rendering colours
-	colour	d_normalTextColour;				//!< Text colour used normally.
-	colour	d_selectTextColour;				//!< Text colour used when text is highlighted
-	colour	d_selectBrushColour;			//!< Colour to apply to the selection brush.
-	colour	d_inactiveSelectBrushColour;	//!< Colour to apply to the selection brush when widget is inactive / read-only.
-
 
 private:
 	/*************************************************************************
@@ -772,10 +690,7 @@ private:
 	static MultiLineEditboxProperties::SelectionStart			d_selectionStartProperty;
 	static MultiLineEditboxProperties::SelectionLength			d_selectionLengthProperty;
 	static MultiLineEditboxProperties::MaxTextLength			d_maxTextLengthProperty;
-	static MultiLineEditboxProperties::NormalTextColour			d_normalTextColourProperty;
-	static MultiLineEditboxProperties::SelectedTextColour		d_selectedTextColourProperty;
-	static MultiLineEditboxProperties::ActiveSelectionColour	d_activeSelectionColourProperty;
-	static MultiLineEditboxProperties::InactiveSelectionColour	d_inactiveSelectionColourProperty;
+    static MultiLineEditboxProperties::SelectionBrushImage      d_selectionBrushProperty;
 
 
 	/*************************************************************************
