@@ -1,35 +1,38 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUITabControl.h
 	created:	08/08/2004
 	author:		Steve Streeting
 	
 	purpose:	Interface to base class for TabControl widget
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #ifndef _CEGUITabControl_h_
 #define _CEGUITabControl_h_
 
 #include "CEGUIBase.h"
 #include "CEGUIWindow.h"
 #include "elements/CEGUITabControlProperties.h"
-#include "elements/CEGUITabPane.h"
 #include <vector>
 
 
@@ -46,6 +49,30 @@ namespace CEGUI
     // Forward declaration
     class TabButton;
 
+/*!
+\brief
+    Base class for TabControl window renderer objects.
+*/
+class CEGUIEXPORT TabControlWindowRenderer : public WindowRenderer
+{
+public:
+    /*!
+    \brief
+        Constructor
+    */
+    TabControlWindowRenderer(const String& name);
+
+    /*!
+    \brief
+        create and return a pointer to a TabButton widget for use as a clickable tab header
+    \param name
+        Button name
+    \return
+        Pointer to a TabButton to be used for changing tabs.
+    */
+    virtual TabButton*  createTabButton(const String& name) const       = 0;
+};
+
 /*!
 \brief
 	Base class for standard Tab Control widget.
@@ -54,7 +81,13 @@ class CEGUIEXPORT TabControl : public Window
 {
 public:
 	static const String EventNamespace;				//!< Namespace for global events
+    static const String WidgetTypeName;             //!< Window factory name
 
+	enum TabPanePosition
+	{
+		Top,
+		Bottom
+	};
 
 	/*************************************************************************
 		Constants
@@ -62,6 +95,15 @@ public:
 	// event names
 	static const String EventSelectionChanged;			//!< Event triggered when there is a change to the currently selected tab.
 
+    /*************************************************************************
+        Child Widget name suffix constants
+    *************************************************************************/
+    static const String ContentPaneNameSuffix; //!< Widget name suffix for the tab content pane component.
+    static const String TabButtonNameSuffix;   //!< Widget name suffix for the tab button components.
+    static const String TabButtonPaneNameSuffix; //!< Widget name suffix for the tab button pane component.
+    static const String ButtonScrollLeftSuffix;//!< Widget name suffix for the scroll tabs to right pane component.
+    static const String ButtonScrollRightSuffix; //!< Widget name suffix for the scroll tabs to left pane component.
+
 
 	/*************************************************************************
 		Accessor Methods
@@ -73,19 +115,37 @@ public:
 	\return
 		the number of tabs currently present.
 	*/
-	uint	getTabCount(void) const;
+	size_t	getTabCount(void) const;
 
+	/*!
+	\brief
+		Return the positioning of the tab pane.
+	\return
+		The positioning of the tab window within the tab control.
+	*/
+	TabPanePosition getTabPanePosition(void) const
+	{ return d_tabPanePos; }
+
+	/*!
+	\brief
+		Change the positioning of the tab button pane.
+	\param pos
+		The new positioning of the tab pane
+	*/
+	void	setTabPanePosition(TabPanePosition pos);
 
     /*!
     \brief
         Set the selected tab by the name of the root window within it.
-    \exception	InvalidRequestException	thrown if \a index is out of range.
+		Also ensures that the tab is made visible (tab pane is scrolled if required).
+    \exception	InvalidRequestException	thrown if there's no tab named \a name.
     */
     void    setSelectedTab(const String &name);
 
     /*!
     \brief
         Set the selected tab by the ID of the root window within it.
+		Also ensures that the tab is made visible (tab pane is scrolled if required).
     \exception	InvalidRequestException	thrown if \a index is out of range.
     */
     void    setSelectedTab(uint ID);
@@ -93,9 +153,31 @@ public:
     /*!
     \brief
         Set the selected tab by the index position in the tab control.
+		Also ensures that the tab is made visible (tab pane is scrolled if required).
     \exception	InvalidRequestException	thrown if \a index is out of range.
     */
-    void    setSelectedTabAtIndex(uint index);
+    void    setSelectedTabAtIndex(size_t index);
+
+    /*!
+    \brief
+        Ensure that the tab by the name of the root window within it is visible.
+    \exception	InvalidRequestException	thrown if there's no tab named \a name.
+    */
+    void    makeTabVisible(const String &name);
+
+    /*!
+    \brief
+        Ensure that the tab by the ID of the root window within it is visible.
+    \exception	InvalidRequestException	thrown if \a index is out of range.
+    */
+    void    makeTabVisible(uint ID);
+
+    /*!
+    \brief
+        Ensure that the tab by the index position in the tab control is visible.
+    \exception	InvalidRequestException	thrown if \a index is out of range.
+    */
+    void    makeTabVisibleAtIndex(size_t index);
 
     /*!
 	\brief
@@ -109,7 +191,7 @@ public:
 
 	\exception	InvalidRequestException	thrown if \a index is out of range.
 	*/
-    Window*	getTabContentsAtIndex(uint index) const;
+    Window*	getTabContentsAtIndex(size_t index) const;
 
     /*!
     \brief
@@ -160,39 +242,19 @@ public:
 	\return
 		index of the currently selected tab.
 	*/
-    uint	getSelectedTabIndex() const;
+    size_t	getSelectedTabIndex() const;
 	
     /*!
     \brief
-        Return the relative height of the tabs
+        Return the height of the tabs
     */
-    float getRelativeTabHeight(void) const { return d_rel_tabHeight; }
+    const UDim& getTabHeight(void) const { return d_tabHeight; }
+
     /*!
     \brief
-        Return the absolute height of the tabs
+        Return the amount of padding to add either side of the text in the tab
     */
-    float getAbsoluteTabHeight(void) const { return d_abs_tabHeight; }
-    /*!
-    \brief
-        Return the height of the tabs in the current metrics mode
-    */
-    float getTabHeight(void) const;
-    /*!
-    \brief
-        Return the amount of padding to add either side of the text in the tab,
-        according to the current metrics mode.
-    */
-    float getTabTextPadding(void) const;
-    /*!
-    \brief
-        Return the relative amount of padding to add either side of the text in the tab
-    */
-    float getRelativeTabTextPadding(void) const { return d_rel_tabPadding; }
-    /*!
-    \brief
-        Return the absolute amount of padding to add either side of the text in the tab
-    */
-    float getAbsoluteTabTextPadding(void) const { return d_abs_tabPadding; }
+    const UDim& getTabTextPadding(void) const { return d_tabPadding; }
 
 
     /*************************************************************************
@@ -208,40 +270,20 @@ public:
 	\return
 		Nothing
 	*/
-	virtual void	initialise(void);
+	virtual void	initialiseComponents(void);
 
     /*!
     \brief
-        Set the relative height of the tabs
+        Set the height of the tabs
     */
-    void setRelativeTabHeight(float height);
-    /*!
-    \brief
-        Set the absolute height of the tabs
-    */
-    void setAbsoluteTabHeight(float height);
-    /*!
-    \brief
-        Set the height of the tabs in the current metrics mode
-    */
-    void setTabHeight(float height);
+    void setTabHeight(const UDim& height);
 
     /*!
     \brief
-        Set the amount of padding to add either side of the text in the tab,
-        according to the current metrics mode.
+        Set the amount of padding to add either side of the text in the tab
     */
-    void setTabTextPadding(float);
-    /*!
-    \brief
-        Set the relative amount of padding to add either side of the text in the tab
-    */
-    void setRelativeTabTextPadding(float);
-    /*!
-    \brief
-        Set the absolute amount of padding to add either side of the text in the tab
-    */
-    void setAbsoluteTabTextPadding(float);
+    void setTabTextPadding(const UDim& padding);
+
     /*!
     \brief 
         Add a new tab to the tab control. 
@@ -301,12 +343,6 @@ protected:
     */
     virtual	void	drawSelf(float z) { /* do nothing; rendering handled by children */ }
 
-    /*!
-	\brief
-		Add tab control specific events
-	*/
-	void	addTabControlEvents(void);
-
     /*!
     \brief
         Add a TabButton for the specified child Window.
@@ -338,9 +374,18 @@ protected:
     virtual void selectTab_impl(Window* wnd);
 
 
+    /*!
+    \brief
+        Internal implementation of make tab visible.
+    \param wnd
+        Pointer to a Window which is the root of the tab content to make visible
+    */
+    virtual void makeTabVisible_impl(Window* wnd);
+
+
 	/*!
 	\brief
-		Return whether this window was inherited from the given class name at some point in the inheritance heirarchy.
+		Return whether this window was inherited from the given class name at some point in the inheritance hierarchy.
 
 	\param class_name
 		The class name that is to be checked.
@@ -350,12 +395,54 @@ protected:
 	*/
 	virtual bool	testClassName_impl(const String& class_name) const
 	{
-		if (class_name==(const utf8*)"TabControl")	return true;
+		if (class_name=="TabControl")	return true;
 		return Window::testClassName_impl(class_name);
 	}
 
+    /*!
+    \brief
+        Return a pointer to the tab button pane (Window)for
+        this TabControl.
+
+    \return
+        Pointer to a Window object.
+
+    \exception UnknownObjectException
+        Thrown if the component does not exist.
+    */
+    Window* getTabButtonPane() const;
+
+    /*!
+    \brief
+        Return a pointer to the content component widget for
+        this TabControl.
+
+    \return
+        Pointer to a Window object.
+
+    \exception UnknownObjectException
+        Thrown if the component does not exist.
+    */
+    Window* getTabPane() const;
+
 	void performChildWindowLayout();
-    int writeChildWindowsXML(OutStream& out_stream) const;
+    int writeChildWindowsXML(XMLSerializer& xml_stream) const;
+
+    // validate window renderer
+    virtual bool validateWindowRenderer(const String& name) const
+    {
+        return (name == "TabControl");
+    }
+
+    /*!
+    \brief
+        create and return a pointer to a TabButton widget for use as a clickable tab header
+    \param name
+        Button name
+    \return
+        Pointer to a TabButton to be used for changing tabs.
+    */
+    TabButton*  createTabButton(const String& name) const;
 
 	/*************************************************************************
 		New event handlers
@@ -380,42 +467,16 @@ protected:
 	/*************************************************************************
 		Implementation Data
 	*************************************************************************/
-    Window*     d_tabButtonPane;    //!< The area containing the tab buttons
-    TabPane*    d_tabContentPane;   //!< The content area window
-    float       d_abs_tabHeight;    //!< The height of the tabs in pixels
-    float       d_rel_tabHeight;    //!< The height of the tabs relative to parent
-    float       d_abs_tabPadding;    //!< The padding of the tabs in pixels
-    float       d_rel_tabPadding;    //!< The padding of the tabs relative to parent
-    uint        d_nextTabIndex;     //!< The index to give the next tab 
-    typedef std::map<uint, TabButton*> TabButtonIndexMap; 
-    TabButtonIndexMap d_tabButtonIndexMap;  //!< Sorting for tabs
+    UDim        d_tabHeight;        //!< The height of the tabs in pixels
+    UDim        d_tabPadding;       //!< The padding of the tabs relative to parent
+    typedef std::vector<TabButton*> TabButtonVector;
+    TabButtonVector d_tabButtonVector;  //!< Sorting for tabs
+    float       d_firstTabOffset;   //!< The offset in pixels of the first tab
+    TabPanePosition d_tabPanePos;   //!< The position of the tab pane
+    float       d_btGrabPos;        //!< The position on the button tab where user grabbed
     /*************************************************************************
     Abstract Implementation Functions (must be provided by derived class)
     *************************************************************************/
-    /*!
-    \brief
-    create and return a pointer to a TabPane widget for use as content area
-
-    \param name
-    String holding the name to be given to the new component.
-
-    \return
-    Pointer to a TabPane to be used for holding the main content.
-    */
-    virtual TabPane*	createTabContentPane(const String& name) const		= 0;
-
-    /*!
-    \brief
-        create and return a pointer to a widget for use as tab button area
-
-    \param name
-    String holding the name to be given to the new component.
-
-    \return
-        Pointer to a Static to be used for holding the tab buttons.
-    */
-    virtual Window*	createTabButtonPane(const String& name) const;
-
     /*!
     \brief
         create and return a pointer to a TabButton widget for use as a clickable tab header
@@ -424,33 +485,26 @@ protected:
     \return
         Pointer to a TabButton to be used for changing tabs.
     */
-    virtual TabButton*	createTabButton(const String& name) const		= 0;
+    //virtual TabButton*	createTabButton_impl(const String& name) const		= 0;
 
     /*!
     \brief
         Calculate the correct position and size of a tab button, based on the
         index it is due to be placed at.
-    \param btn
-        Pointer to a button to calculate the size and position for. This button
-        will be updated in place with these metrics.
-    \param targetIndex
-        The index at which the tab is/will be placed. Tabs must exist for all the
-        indexes before this.
+    \param index
+        The index of the tab button
     */
-    void calculateTabButtonSizePosition(TabButton* btn, uint targetIndex);
+    void calculateTabButtonSizePosition(size_t index);
 
 protected:
 	/*************************************************************************
 		Static Properties for this class
 	*************************************************************************/
-	static TabControlProperties::TabHeight			d_tabHeightProperty;
-    static TabControlProperties::AbsoluteTabHeight	d_absoluteTabHeightProperty;
-    static TabControlProperties::RelativeTabHeight	d_relativeTabHeightProperty;
+    static TabControlProperties::TabHeight       d_tabHeightProperty;
+    static TabControlProperties::TabTextPadding  d_tabTextPaddingProperty;
+    static TabControlProperties::TabPanePosition d_tabPanePosition;
 
-    static TabControlProperties::TabTextPadding			d_tabTextPaddingProperty;
-    static TabControlProperties::AbsoluteTabTextPadding	d_absoluteTabTextPaddingProperty;
-    static TabControlProperties::RelativeTabTextPadding	d_relativeTabTextPaddingProperty;
-	/*************************************************************************
+    /*************************************************************************
 		Private methods
 	*************************************************************************/
 	void	addTabControlProperties(void);
@@ -463,10 +517,12 @@ protected:
     *************************************************************************/
     bool handleContentWindowTextChanged(const EventArgs& args);
     bool handleTabButtonClicked(const EventArgs& args);
+    bool handleScrollPane(const EventArgs& e);
+    bool handleDraggedPane(const EventArgs& e);
+    bool handleWheeledPane(const EventArgs& e);
 };
 
 
-
 } // End of  CEGUI namespace section
 
 
