@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIImageset.cpp
 	created:	21/2/2004
 	author:		Paul D Turner
 	
 	purpose:	Implements the Imageset class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "CEGUIImageset.h"
 #include "CEGUIExceptions.h"
 #include "CEGUITexture.h"
@@ -32,6 +36,8 @@
 #include "CEGUILogger.h"
 #include "CEGUIDataContainer.h"
 #include "CEGUIXMLParser.h"
+#include "CEGUIXMLSerializer.h" 
+#include "CEGUIPropertyHelper.h" 
 #include <iostream>
 #include <cmath>
 
@@ -40,10 +46,11 @@ namespace CEGUI
 {
 
 /*************************************************************************
-	Definition of constant data for Imageset (and sub-classes)
+	Definition of static /const data for Imageset (and sub-classes)
 *************************************************************************/
 // Declared in Imageset
 const char	Imageset::ImagesetSchemaName[]			= "Imageset.xsd";
+String Imageset::d_defaultResourceGroup;
 
 
 /*************************************************************************
@@ -53,11 +60,10 @@ Imageset::Imageset(const String& name, Texture* texture) :
 	d_name(name),
 	d_texture(texture)
 {
-	if (d_texture == NULL)
+	if (!d_texture)
 	{
-		throw NullObjectException((utf8*)"Imageset::Imageset - Texture object supplied for Imageset creation must not be NULL");
+		throw NullObjectException("Imageset::Imageset - Texture object supplied for Imageset creation must be valid.");
 	}
-
 	// defaults for scaling options
 	d_autoScale = false;
 	setNativeResolution(Size(DefaultNativeHorzRes, DefaultNativeVertRes));
@@ -73,7 +79,7 @@ Imageset::Imageset(const String& filename, const String& resourceGroup)
 	d_autoScale = false;
 	setNativeResolution(Size(DefaultNativeHorzRes, DefaultNativeVertRes));
 
-	d_texture = NULL;
+	d_texture = 0;
 	load(filename, resourceGroup);
 }
 
@@ -83,7 +89,12 @@ Imageset::Imageset(const String& name, const String& filename, const String& res
 {
     // try to load the image file using the renderer
     d_texture =
-        System::getSingleton().getRenderer()->createTexture(filename, resourceGroup);
+        System::getSingleton().getRenderer()->createTexture(filename,
+        resourceGroup.empty() ? d_defaultResourceGroup : resourceGroup);
+
+    // store texture filename
+    d_textureFilename = filename;
+    // TODO: Should we store the resource group too?
 
     // initialse the auto-scaling for this Imageset
     d_autoScale = true;
@@ -93,7 +104,7 @@ Imageset::Imageset(const String& name, const String& filename, const String& res
     // define the default image for this Imageset
     defineImage(
         "full_image",
-        Rect(0, 0, d_texture->getWidth(), d_texture->getHeight()),
+        Rect(0, 0, d_texture->getOriginalWidth(), d_texture->getOriginalHeight()),
         Point(0, 0)
     );
 }
@@ -113,9 +124,9 @@ Imageset::~Imageset(void)
 *************************************************************************/
 void Imageset::setTexture(Texture* texture)
 {
-	if (d_texture == NULL)
+	if (!d_texture)
 	{
-		throw NullObjectException((utf8*)"Imageset::setTexture - Texture object supplied for Imageset creation must not be NULL");
+		throw NullObjectException("Imageset::setTexture - Texture object supplied for Imageset creation must be valid.");
 	}
 
 	d_texture = texture;
@@ -130,9 +141,9 @@ void Imageset::load(const String& filename, const String& resourceGroup)
 	// unload old data and texture.
 	unload();
 
-	if (filename.empty() || (filename == (utf8*)""))
+	if (filename.empty())
 	{
-		throw InvalidRequestException((utf8*)"Imageset::load - Filename supplied for Imageset loading must be valid");
+		throw InvalidRequestException("Imageset::load - Filename supplied for Imageset loading must be valid");
 	}
 
     // create handler object
@@ -141,7 +152,9 @@ void Imageset::load(const String& filename, const String& resourceGroup)
 	// do parse (which uses handler to create actual data)
 	try
 	{
-        System::getSingleton().getXMLParser()->parseXMLFile(handler, filename, ImagesetSchemaName, resourceGroup);
+        System::getSingleton().getXMLParser()->parseXMLFile(
+                handler, filename, ImagesetSchemaName,
+                resourceGroup.empty() ? d_defaultResourceGroup : resourceGroup);
 	}
 	catch(...)
 	{
@@ -203,8 +216,10 @@ void Imageset::draw(const Rect& source_rect, const Rect& dest_rect, float z, con
 	// check if rect was totally clipped
 	if (final_rect.getWidth() != 0)
 	{
-		float x_scale = 1.0f / (float)d_texture->getWidth();
-		float y_scale = 1.0f / (float)d_texture->getHeight();
+        // Fix bug #45
+        // Obtain correct scale vlaue from the texture
+        const float x_scale = d_texture->getXScale();
+        const float y_scale = d_texture->getYScale();
 
 		float tex_per_pix_x = source_rect.getWidth() / dest_rect.getWidth();
 		float tex_per_pix_y = source_rect.getHeight() / dest_rect.getHeight();
@@ -235,7 +250,7 @@ void Imageset::unload(void)
 
 	// cleanup texture
 	System::getSingleton().getRenderer()->destroyTexture(d_texture);
-	d_texture = NULL;
+	d_texture = 0;
 }
 
 
@@ -308,34 +323,33 @@ void Imageset::notifyScreenResolution(const Size& size)
 
 }
 
-void Imageset::writeXMLToStream(OutStream& out_stream) const
+void Imageset::writeXMLToStream(XMLSerializer& xml_stream) const
 {
-    // output opening tag
-    out_stream << "<Imageset Name=\"" << d_name << "\" ";
-    out_stream << "Filename=\"" << d_textureFilename << "\" ";
+    // output Imageset tag
+    xml_stream.openTag("Imageset")
+        .attribute("Name", d_name)
+        .attribute("Imagefile", d_textureFilename);
 
     if (d_nativeHorzRes != DefaultNativeHorzRes)
-        out_stream << "NativeHorzRes=\"" << static_cast<uint>(d_nativeHorzRes) << "\" ";
-
+        xml_stream.attribute("NativeHorzRes", 
+          PropertyHelper::uintToString(static_cast<uint>(d_nativeHorzRes)));
     if (d_nativeVertRes != DefaultNativeVertRes)
-        out_stream << "NativeVertRes=\"" << static_cast<uint>(d_nativeVertRes) << "\" ";
+        xml_stream.attribute("NativeVertRes", 
+          PropertyHelper::uintToString(static_cast<uint>(d_nativeVertRes)));
 
     if (d_autoScale)
-        out_stream << "AutoScaled=\"True\" ";
-
-    out_stream << ">" << std::endl;
-
+        xml_stream.attribute("AutoScaled", "true");
+    
     // output images
     ImageIterator image = getIterator();
-
     while (!image.isAtEnd())
     {
-        image.getCurrentValue().writeXMLToStream(out_stream);
+        image.getCurrentValue().writeXMLToStream(xml_stream);
         ++image;
     }
 
     // output closing tag
-    out_stream << "</Imageset>" << std::endl;
+    xml_stream.closeTag();
 }
 
 
