@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIImagesetManager.cpp
 	created:	21/2/2004
 	author:		Paul D Turner
 	
 	purpose:	Implements the ImagesetManager class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "CEGUIImagesetManager.h"
 #include "CEGUIExceptions.h"
 #include "CEGUILogger.h"
@@ -35,7 +39,7 @@ namespace CEGUI
 	Static Data Definitions
 *************************************************************************/
 // singleton instance pointer
-template<> ImagesetManager* Singleton<ImagesetManager>::ms_Singleton	= NULL;
+template<> ImagesetManager* Singleton<ImagesetManager>::ms_Singleton	= 0;
 
 
 /*************************************************************************
@@ -43,7 +47,7 @@ template<> ImagesetManager* Singleton<ImagesetManager>::ms_Singleton	= NULL;
 *************************************************************************/
 ImagesetManager::ImagesetManager(void)
 {
-	Logger::getSingleton().logEvent((utf8*)"CEGUI::ImagesetManager singleton created");
+	Logger::getSingleton().logEvent("CEGUI::ImagesetManager singleton created");
 }
 
 
@@ -52,11 +56,11 @@ ImagesetManager::ImagesetManager(void)
 *************************************************************************/
 ImagesetManager::~ImagesetManager(void)
 {
-	Logger::getSingleton().logEvent((utf8*)"---- Begining cleanup of Imageset system ----");
+	Logger::getSingleton().logEvent("---- Begining cleanup of Imageset system ----");
 
 	destroyAllImagesets();
 
-	Logger::getSingleton().logEvent((utf8*)"CEGUI::ImagesetManager singleton destroyed");
+	Logger::getSingleton().logEvent("CEGUI::ImagesetManager singleton destroyed");
 }
 
 
@@ -66,7 +70,7 @@ ImagesetManager::~ImagesetManager(void)
 *************************************************************************/
 Imageset* ImagesetManager::createImageset(const String& name, Texture* texture)
 {
-	Logger::getSingleton().logEvent((utf8*)"Attempting to create Imageset '" + name +"' with texture only.");
+	Logger::getSingleton().logEvent("Attempting to create Imageset '" + name +"' with texture only.");
 
 	if (isImagesetPresent(name))
 	{
@@ -85,7 +89,7 @@ Imageset* ImagesetManager::createImageset(const String& name, Texture* texture)
 *************************************************************************/
 Imageset* ImagesetManager::createImageset(const String& filename, const String& resourceGroup)
 {
-	Logger::getSingleton().logEvent((utf8*)"Attempting to create an Imageset from the information specified in file '" + filename + "'.");
+	Logger::getSingleton().logEvent("Attempting to create an Imageset from the information specified in file '" + filename + "'.");
 
 	Imageset* temp = new Imageset(filename, resourceGroup);
 
@@ -109,7 +113,7 @@ Imageset* ImagesetManager::createImageset(const String& filename, const String&
 *************************************************************************/
 Imageset* ImagesetManager::createImagesetFromImageFile(const String& name, const String& filename, const String& resourceGroup)
 {
-    Logger::getSingleton().logEvent((utf8*)"Attempting to create Imageset '" + name + "' using image file '" + filename + "'.");
+    Logger::getSingleton().logEvent("Attempting to create Imageset '" + name + "' using image file '" + filename + "'.");
 
     if (isImagesetPresent(name))
     {
@@ -137,7 +141,7 @@ void ImagesetManager::destroyImageset(const String& name)
 		delete pos->second;
 		d_imagesets.erase(pos);
 
-		Logger::getSingleton().logEvent((utf8*)"Imageset '" + tmpName +"' has been destroyed.", Informative);
+		Logger::getSingleton().logEvent("Imageset '" + tmpName +"' has been destroyed.", Informative);
 	}
 
 }
@@ -148,7 +152,7 @@ void ImagesetManager::destroyImageset(const String& name)
 *************************************************************************/
 void ImagesetManager::destroyImageset(Imageset* imageset)
 {
-	if (imageset != NULL)
+	if (imageset)
 	{
 		destroyImageset(imageset->getName());
 	}
@@ -200,18 +204,6 @@ void ImagesetManager::notifyScreenResolution(const Size& size)
 }
 
 
-ImagesetManager& ImagesetManager::getSingleton(void)
-{
-	return Singleton<ImagesetManager>::getSingleton();
-}
-
-
-ImagesetManager* ImagesetManager::getSingletonPtr(void)
-{
-	return Singleton<ImagesetManager>::getSingletonPtr();
-}
-
-
 /*************************************************************************
 	Return a ImagesetManager::ImagesetIterator object to iterate over
 	the available Imageset objects.
@@ -225,12 +217,9 @@ ImagesetManager::ImagesetIterator ImagesetManager::getIterator(void) const
 void ImagesetManager::writeImagesetToStream(const String& imageset, OutStream& out_stream) const
 {
     const Imageset* iset = getImageset(imageset);
-
-    // output xml header
-    out_stream << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << std::endl;
-
-    // output imageset data
-    iset->writeXMLToStream(out_stream);
+    // Create an XMLSerializer which make use of 4 space and UTF-8 encoding 
+    XMLSerializer xml(out_stream);
+    iset->writeXMLToStream(xml);
 }
 
 
