@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIMouseCursor.cpp
 	created:	21/2/2004
 	author:		Paul D Turner
 	
 	purpose:	Implements MouseCursor class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "CEGUIMouseCursor.h"
 #include "CEGUIExceptions.h"
 #include "CEGUILogger.h"
@@ -41,13 +45,13 @@ const String MouseCursor::EventNamespace("MouseCursor");
 	Static Data Definitions
 *************************************************************************/
 // singleton instance pointer
-template<> MouseCursor* Singleton<MouseCursor>::ms_Singleton	= NULL;
+template<> MouseCursor* Singleton<MouseCursor>::ms_Singleton	= 0;
 
 
 /*************************************************************************
 	Event name constants
 *************************************************************************/
-const String MouseCursor::EventImageChanged( (utf8*)"ImageChanged" );
+const String MouseCursor::EventImageChanged( "ImageChanged" );
 
 
 /*************************************************************************
@@ -60,7 +64,7 @@ MouseCursor::MouseCursor(void)
 	// mouse defaults to middle of the constrained area
 	d_position.d_x = screenArea.getWidth() / 2;
 	d_position.d_y = screenArea.getHeight() / 2;
-	d_position.d_z = 1.0f;
+	d_position.d_z = 0.0f;
 
 	// default constraint is to whole screen
 	setConstraintArea(&screenArea);
@@ -69,12 +73,9 @@ MouseCursor::MouseCursor(void)
 	d_visible = true;
 
 	// no default image though
-	d_cursorImage = NULL;
+	d_cursorImage = 0;
 
-	// add events
-	addMouseCursorEvents();
-
-	Logger::getSingleton().logEvent((utf8*)"CEGUI::MouseCursor singleton created.");
+	Logger::getSingleton().logEvent("CEGUI::MouseCursor singleton created.");
 }
 
 
@@ -83,7 +84,7 @@ MouseCursor::MouseCursor(void)
 *************************************************************************/
 MouseCursor::~MouseCursor(void)
 {
-	Logger::getSingleton().logEvent((utf8*)"CEGUI::MouseCursor singleton destroyed.");
+	Logger::getSingleton().logEvent("CEGUI::MouseCursor singleton destroyed.");
 }
 
 
@@ -113,7 +114,7 @@ void MouseCursor::setImage(const String& imageset, const String& image_name)
 *************************************************************************/
 void MouseCursor::draw(void) const
 {
-	if (d_visible && (d_cursorImage != NULL))
+	if (d_visible && (d_cursorImage != 0))
 	{
 		d_cursorImage->draw( d_position, System::getSingleton().getRenderer()->getRect() );
 	}
@@ -171,7 +172,7 @@ void MouseCursor::setConstraintArea(const Rect* area)
 {
 	Rect renderer_area = System::getSingleton().getRenderer()->getRect();
 
-	if (area == NULL)
+	if (!area)
 	{
 		d_constraints.d_min.d_x = cegui_reldim(renderer_area.d_left / renderer_area.getWidth());
 		d_constraints.d_min.d_y = cegui_reldim(renderer_area.d_top / renderer_area.getHeight());
@@ -241,16 +242,6 @@ Point MouseCursor::getDisplayIndependantPosition(void) const
 }
 
 
-/*************************************************************************
-	Add MouseCursor events
-*************************************************************************/
-void MouseCursor::addMouseCursorEvents(void)
-{
-	// mouse cursor events
-	addEvent(EventImageChanged);
-}
-
-
 //////////////////////////////////////////////////////////////////////////
 /*************************************************************************
 
