@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUISchemeManager.cpp
 	created:	21/2/2004
 	author:		Paul D Turner
 	
 	purpose:	Implements SchemeManager class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "CEGUISchemeManager.h"
 #include "CEGUIExceptions.h"
 #include "CEGUILogger.h"
@@ -35,7 +39,7 @@ namespace CEGUI
 	Static Data Definitions
 *************************************************************************/
 // singleton instance pointer
-template<> SchemeManager* Singleton<SchemeManager>::ms_Singleton	= NULL;
+template<> SchemeManager* Singleton<SchemeManager>::ms_Singleton	= 0;
 
 
 
@@ -44,7 +48,7 @@ template<> SchemeManager* Singleton<SchemeManager>::ms_Singleton	= NULL;
 *************************************************************************/
 SchemeManager::SchemeManager(void)
 {
-	Logger::getSingleton().logEvent((utf8*)"CEGUI::SchemeManager singleton created.");
+	Logger::getSingleton().logEvent("CEGUI::SchemeManager singleton created.");
 }
 
 
@@ -53,11 +57,11 @@ SchemeManager::SchemeManager(void)
 *************************************************************************/
 SchemeManager::~SchemeManager(void)
 {
-	Logger::getSingleton().logEvent((utf8*)"---- Begining cleanup of GUI Scheme system ----");
+	Logger::getSingleton().logEvent("---- Begining cleanup of GUI Scheme system ----");
 
 	unloadAllSchemes();
 
-	Logger::getSingleton().logEvent((utf8*)"CEGUI::SchemeManager singleton destroyed.");
+	Logger::getSingleton().logEvent("CEGUI::SchemeManager singleton destroyed.");
 }
 
 
@@ -66,7 +70,7 @@ SchemeManager::~SchemeManager(void)
 *************************************************************************/
 Scheme* SchemeManager::loadScheme(const String& scheme_filename, const String& resourceGroup)
 {
-	Logger::getSingleton().logEvent((utf8*)"Attempting to load Scheme from file '" + scheme_filename + "'.");
+	Logger::getSingleton().logEvent("Attempting to load Scheme from file '" + scheme_filename + "'.");
 
 	Scheme* tmp = new Scheme(scheme_filename, resourceGroup);
 	String name = tmp->getName();
@@ -89,7 +93,7 @@ void SchemeManager::unloadScheme(const String& scheme_name)
 		delete pos->second;
 		d_schemes.erase(pos);
 
-		Logger::getSingleton().logEvent((utf8*)"Scheme '" + tmpName + "' has been unloaded.");
+		Logger::getSingleton().logEvent("Scheme '" + tmpName + "' has been unloaded.");
 	}
 	else
 	{
@@ -115,18 +119,6 @@ Scheme* SchemeManager::getScheme(const String& name) const
 }
 
 
-SchemeManager& SchemeManager::getSingleton(void)
-{
-	return Singleton<SchemeManager>::getSingleton();
-}
-
-
-SchemeManager* SchemeManager::getSingletonPtr(void)
-{
-	return Singleton<SchemeManager>::getSingletonPtr();
-}
-
-
 /*************************************************************************
 	Return a SchemeManager::SchemeIterator object to iterate over the
 	available schemes.
