@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIWindowFactoryManager.cpp
 	created:	22/2/2004
 	author:		Paul D Turner
 	
 	purpose:	Implements the WindowFactoryManager
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "CEGUIWindowFactoryManager.h"
 #include "CEGUIWindowFactory.h"
 #include "CEGUIExceptions.h"
@@ -35,7 +39,7 @@ namespace CEGUI
 	Static Data Definitions
 *************************************************************************/
 // singleton instance pointer
-template<> WindowFactoryManager* Singleton<WindowFactoryManager>::ms_Singleton	= NULL;
+template<> WindowFactoryManager* Singleton<WindowFactoryManager>::ms_Singleton	= 0;
 
 
 /*************************************************************************
@@ -44,21 +48,21 @@ template<> WindowFactoryManager* Singleton<WindowFactoryManager>::ms_Singleton	=
 void WindowFactoryManager::addFactory(WindowFactory* factory)
 {
 	// throw exception if passed factory is null.
-	if (factory == NULL)
+	if (!factory)
 	{
-		throw NullObjectException((utf8*)"WindowFactoryManager::addFactory - The provided WindowFactory pointer was NULL");
+		throw NullObjectException("WindowFactoryManager::addFactory - The provided WindowFactory pointer was invalid.");
 	}
 
 	// throw exception if type name for factory is already in use
 	if (d_factoryRegistry.find(factory->getTypeName()) != d_factoryRegistry.end())
 	{
-		throw AlreadyExistsException((utf8*)"WindowFactoryManager::addFactory - A WindowFactory for type '" + factory->getTypeName() + (utf8*)"' is already registered.");
+		throw AlreadyExistsException("WindowFactoryManager::addFactory - A WindowFactory for type '" + factory->getTypeName() + "' is already registered.");
 	}
 
 	// add the factory to the registry
 	d_factoryRegistry[factory->getTypeName()] = factory;
 
-	Logger::getSingleton().logEvent((utf8*)"WindowFactory for '" + factory->getTypeName() +"' windows added.");
+	Logger::getSingleton().logEvent("WindowFactory for '" + factory->getTypeName() +"' windows added.");
 }
 
 
@@ -69,7 +73,7 @@ void WindowFactoryManager::removeFactory(const String& name)
 {
 	d_factoryRegistry.erase(name);
 
-	Logger::getSingleton().logEvent((utf8*)"WindowFactory for '" + name +"' windows removed.");
+	Logger::getSingleton().logEvent("WindowFactory for '" + name +"' windows removed.");
 }
 
 
@@ -78,7 +82,7 @@ void WindowFactoryManager::removeFactory(const String& name)
 *************************************************************************/
 void WindowFactoryManager::removeFactory(WindowFactory* factory)
 {
-	if (factory != NULL)
+	if (factory)
 	{
 		removeFactory(factory->getTypeName());
 	}
@@ -144,18 +148,6 @@ bool WindowFactoryManager::isFactoryPresent(const String& name) const
 }
 
 
-WindowFactoryManager& WindowFactoryManager::getSingleton(void)
-{
-	return Singleton<WindowFactoryManager>::getSingleton();
-}
-
-
-WindowFactoryManager* WindowFactoryManager::getSingletonPtr(void)
-{
-	return Singleton<WindowFactoryManager>::getSingletonPtr();
-}
-
-
 /*************************************************************************
 	Return a WindowFactoryManager::WindowFactoryIterator object to
 	iterate over the available WindowFactory types.
@@ -184,7 +176,7 @@ void WindowFactoryManager::addWindowTypeAlias(const String& aliasName, const Str
 	// throw if target type does not exist
 	if (!isFactoryPresent(targetType))
 	{
-		throw UnknownObjectException((utf8*)"WindowFactoryManager::addWindowTypeAlias - alias '" + aliasName + "' could not be created because the target type '" + targetType + "' is unknown within the system.");
+		throw UnknownObjectException("WindowFactoryManager::addWindowTypeAlias - alias '" + aliasName + "' could not be created because the target type '" + targetType + "' is unknown within the system.");
 	}
 
 	TypeAliasRegistry::iterator pos = d_aliasRegistry.find(aliasName);
@@ -199,7 +191,7 @@ void WindowFactoryManager::addWindowTypeAlias(const String& aliasName, const Str
 		pos->second.d_targetStack.push_back(targetType);
 	}
 
-	Logger::getSingleton().logEvent((utf8*)"Window type alias named '" + aliasName + "' added for window type '" + targetType +"'.");
+	Logger::getSingleton().logEvent("Window type alias named '" + aliasName + "' added for window type '" + targetType +"'.");
 }
 
 
@@ -223,7 +215,7 @@ void WindowFactoryManager::removeWindowTypeAlias(const String& aliasName, const
 			// erase the target mapping
 			pos->second.d_targetStack.erase(aliasPos);
 
-			Logger::getSingleton().logEvent((utf8*)"Window type alias named '" + aliasName + "' removed for window type '" + targetType +"'.");
+			Logger::getSingleton().logEvent("Window type alias named '" + aliasName + "' removed for window type '" + targetType +"'.");
 
 			// if the list of targets for this alias is now empty
 			if (pos->second.d_targetStack.empty())
@@ -231,7 +223,7 @@ void WindowFactoryManager::removeWindowTypeAlias(const String& aliasName, const
 				// erase the alias name also
 				d_aliasRegistry.erase(aliasName);
 
-				Logger::getSingleton().logEvent((utf8*)"Window type alias named '" + aliasName + "' has no more targets and has been removed.", Informative);
+				Logger::getSingleton().logEvent("Window type alias named '" + aliasName + "' has no more targets and has been removed.", Informative);
 			}
 
 		}
@@ -240,12 +232,13 @@ void WindowFactoryManager::removeWindowTypeAlias(const String& aliasName, const
 
 }
 
-void WindowFactoryManager::addFalagardWindowMapping(const String& newType, const String& targetType, const String& lookName)
+void WindowFactoryManager::addFalagardWindowMapping(const String& newType, const String& targetType, const String& lookName, const String& renderer)
 {
     WindowFactoryManager::FalagardWindowMapping mapping;
     mapping.d_windowType = newType;
     mapping.d_baseType   = targetType;
     mapping.d_lookName   = lookName;
+    mapping.d_rendererType = renderer;
 
     // see if the type we're creating already exists
     if (d_falagardRegistry.find(newType) != d_falagardRegistry.end())
@@ -254,7 +247,7 @@ void WindowFactoryManager::addFalagardWindowMapping(const String& newType, const
         Logger::getSingleton().logEvent("Falagard mapping for type '" + newType + "' already exists - current mapping will be replaced.");
     }
 
-    Logger::getSingleton().logEvent("Creating falagard mapping for type '" + newType + "' using base type '" + targetType + "' and LookN'Feel '" + lookName + "'.");
+    Logger::getSingleton().logEvent("Creating falagard mapping for type '" + newType + "' using base type '" + targetType + "', window renderer '" + renderer + "' and Look'N'Feel '" + lookName + "'.");
 
     d_falagardRegistry[newType] = mapping;
 }
@@ -296,6 +289,22 @@ const String& WindowFactoryManager::getMappedLookForType(const String& type) con
     }
 }
 
+const String& WindowFactoryManager::getMappedRendererForType(const String& type) const
+{
+    FalagardMapRegistry::const_iterator iter =
+        d_falagardRegistry.find(getDereferencedAliasType(type));
+
+    if (iter != d_falagardRegistry.end())
+    {
+        return (*iter).second.d_rendererType;
+    }
+    // type does not exist as a mapped type (or an alias for one)
+    else
+    {
+        throw InvalidRequestException("WindowFactoryManager::getMappedLookForType - Window factory type '" + type + "' is not a falagard mapped type (or an alias for one).");
+    }
+}
+
 String WindowFactoryManager::getDereferencedAliasType(const String& type) const
 {
     TypeAliasRegistry::const_iterator alias = d_aliasRegistry.find(type);
@@ -309,6 +318,22 @@ String WindowFactoryManager::getDereferencedAliasType(const String& type) const
     return type;
 }
 
+const WindowFactoryManager::FalagardWindowMapping& WindowFactoryManager::getFalagardMappingForType(const String& type) const
+{
+    FalagardMapRegistry::const_iterator iter =
+        d_falagardRegistry.find(getDereferencedAliasType(type));
+
+    if (iter != d_falagardRegistry.end())
+    {
+        return (*iter).second;
+    }
+    // type does not exist as a mapped type (or an alias for one)
+    else
+    {
+        throw InvalidRequestException("WindowFactoryManager::getFalagardMappingForType - Window factory type '" + type + "' is not a falagard mapped type (or an alias for one).");
+    }
+}
+
 
 //////////////////////////////////////////////////////////////////////////
 /*************************************************************************
