@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIWindowManager.cpp
 	created:	21/2/2004
 	author:		Paul D Turner
 	
 	purpose:	Implements the WindowManager class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "CEGUIWindowManager.h"
 #include "CEGUIWindowFactoryManager.h"
 #include "CEGUIWindowFactory.h"
@@ -40,8 +44,9 @@ namespace CEGUI
 	Static Data Definitions
 *************************************************************************/
 // singleton instance pointer
-template<> WindowManager* Singleton<WindowManager>::ms_Singleton	= NULL;
-
+template<> WindowManager* Singleton<WindowManager>::ms_Singleton	= 0;
+// default resource group
+String WindowManager::d_defaultResourceGroup;
 
 /*************************************************************************
 	Definition of constant data for WindowManager
@@ -57,7 +62,7 @@ const String WindowManager::GeneratedWindowNameBase("__cewin_uid_");
 WindowManager::WindowManager(void) :
     d_uid_counter(0)
 {
-    Logger::getSingleton().logEvent((utf8*)"CEGUI::WindowManager singleton created");
+    Logger::getSingleton().logEvent("CEGUI::WindowManager singleton created");
 }
 
 
@@ -69,7 +74,7 @@ WindowManager::~WindowManager(void)
 	destroyAllWindows();
     cleanDeadPool();
 
-	Logger::getSingleton().logEvent((utf8*)"CEGUI::WindowManager singleton destroyed");
+	Logger::getSingleton().logEvent("CEGUI::WindowManager singleton destroyed");
 }
 
 
@@ -94,14 +99,14 @@ Window* WindowManager::createWindow(const String& type, const String& name)
     // see if we need to assign a look to this window
     if (wfMgr.isFalagardMappedType(type))
     {
+        const WindowFactoryManager::FalagardWindowMapping& fwm = wfMgr.getFalagardMappingForType(type);
         // this was a mapped type, so assign a look to the window so it can finalise
         // its initialisation
-        newWindow->setLookNFeel(type, wfMgr.getMappedLookForType(type));
+        newWindow->d_falagardType = type;
+        newWindow->setWindowRenderer(fwm.d_rendererType);
+        newWindow->setLookNFeel(fwm.d_lookName);
     }
 
-    // perform initialisation step
-    newWindow->initialise();
-
 	d_windowRegistry[finalName] = newWindow;
 
 	return newWindow;
@@ -113,7 +118,7 @@ Window* WindowManager::createWindow(const String& type, const String& name)
 *************************************************************************/
 void WindowManager::destroyWindow(Window* window)
 {
-	if (window != NULL)
+	if (window)
 	{
 		// this is done because the name is used for the log after the window is destroyed,
 		// if we just did getName() we would get a const ref to the Window's internal name
@@ -150,7 +155,7 @@ void WindowManager::destroyWindow(const String& window)
         // notify system object of the window destruction
         System::getSingleton().notifyWindowDestroyed(wnd);
 
-		Logger::getSingleton().logEvent((utf8*)"Window '" + window + "' has been added to dead pool.", Informative);
+		Logger::getSingleton().logEvent("Window '" + window + "' has been added to dead pool.", Informative);
 	}
 
 }
@@ -202,13 +207,13 @@ void WindowManager::destroyAllWindows(void)
 *************************************************************************/
 Window* WindowManager::loadWindowLayout(const String& filename, const String& name_prefix, const String& resourceGroup, PropertyCallback* callback, void* userdata)
 {
-	if (filename.empty() || (filename == (utf8*)""))
+	if (filename.empty())
 	{
-		throw InvalidRequestException((utf8*)"WindowManager::loadWindowLayout - Filename supplied for gui-layout loading must be valid.");
+		throw InvalidRequestException("WindowManager::loadWindowLayout - Filename supplied for gui-layout loading must be valid.");
 	}
 
 	// log the fact we are about to load a layout
-	Logger::getSingleton().logEvent((utf8*)"---- Beginning loading of GUI layout from '" + filename + "' ----", Informative);
+	Logger::getSingleton().logEvent("---- Beginning loading of GUI layout from '" + filename + "' ----", Informative);
 
     // create handler object
     GUILayout_xmlHandler handler(name_prefix, callback, userdata);
@@ -216,7 +221,8 @@ Window* WindowManager::loadWindowLayout(const String& filename, const String& na
 	// do parse (which uses handler to create actual data)
 	try
 	{
-        System::getSingleton().getXMLParser()->parseXMLFile(handler, filename, GUILayoutSchemaName, resourceGroup);
+        System::getSingleton().getXMLParser()->parseXMLFile(handler,
+            filename, GUILayoutSchemaName, resourceGroup.empty() ? d_defaultResourceGroup : resourceGroup);
 	}
 	catch(...)
 	{
@@ -225,23 +231,11 @@ Window* WindowManager::loadWindowLayout(const String& filename, const String& na
 	}
 
     // log the completion of loading
-    Logger::getSingleton().logEvent((utf8*)"---- Successfully completed loading of GUI layout from '" + filename + "' ----", Standard);
+    Logger::getSingleton().logEvent("---- Successfully completed loading of GUI layout from '" + filename + "' ----", Standard);
 
 	return handler.getLayoutRootWindow();
 }
 
-
-WindowManager& WindowManager::getSingleton(void)
-{
-	return Singleton<WindowManager>::getSingleton();
-}
-
-
-WindowManager* WindowManager::getSingletonPtr(void)
-{
-	return Singleton<WindowManager>::getSingletonPtr();
-}
-
 bool WindowManager::isDeadPoolEmpty(void) const
 {
     return d_deathrow.empty();
@@ -267,21 +261,19 @@ void WindowManager::cleanDeadPool(void)
 
 void WindowManager::writeWindowLayoutToStream(const Window& window, OutStream& out_stream, bool writeParent) const
 {
-    // output xml header
-    out_stream << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << std::endl;
+
+    XMLSerializer xml(out_stream);
     // output GUILayout start element
-    out_stream << "<GUILayout";
+    xml.openTag("GUILayout");
     // see if we need the parent attribute to be written
     if ((window.getParent() != 0) && writeParent)
     {
-        out_stream << " Parent=\"" << window.getParent()->getName() << "\" ";
+        xml.attribute("Parent", window.getParent()->getName());
     }
-    // close opening tag
-    out_stream << ">" << std::endl;
     // write windows
-    window.writeXMLToStream(out_stream);
+    window.writeXMLToStream(xml);
     // write closing GUILayout element
-    out_stream << "</GUILayout>" << std::endl;
+    xml.closeTag();
 }
 
 void WindowManager::writeWindowLayoutToStream(const String& window, OutStream& out_stream, bool writeParent) const
@@ -322,8 +314,21 @@ void WindowManager::renameWindow(Window* window, const String& new_name)
         {
             // erase old window name from registry
             d_windowRegistry.erase(pos);
-            // rename the window
-            window->rename(new_name);
+
+            try
+            {
+                // attempt to rename the window
+                window->rename(new_name);
+            }
+            // rename fails if target name already exists
+            catch (AlreadyExistsException&)
+            {
+                // re-add window to registry under it's old name
+                d_windowRegistry[window->getName()] = window;
+                // rethrow exception.
+                throw;
+            }
+
             // add window to registry under new name
             d_windowRegistry[new_name] = window;
         }
