@@ -1,42 +1,48 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIComboDropList.cpp
 	created:	13/6/2004
 	author:		Paul D Turner
 	
 	purpose:	Implements the Combobox Drop-List widget base class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "elements/CEGUIComboDropList.h"
 #include "elements/CEGUIScrollbar.h"
-
+#include "CEGUICoordConverter.h"
 
 // Start of CEGUI namespace section
 namespace CEGUI
 {
 const String ComboDropList::EventNamespace("ComboDropList");
+const String ComboDropList::WidgetTypeName("CEGUI/ComboDropList");
+
 
 /*************************************************************************
 	Constants
 *************************************************************************/
 // Event names
-const String ComboDropList::EventListSelectionAccepted( (utf8*)"ListSelectionAccepted" );
+const String ComboDropList::EventListSelectionAccepted( "ListSelectionAccepted" );
 
 
 /*************************************************************************
@@ -48,7 +54,6 @@ ComboDropList::ComboDropList(const String& type, const String& name) :
 	d_autoArm = false;
 	d_armed = false;
 
-	addComboDropListEvents();
 	hide();
 
     // pass captured inputs to children to enable scrollbars
@@ -67,22 +72,13 @@ ComboDropList::~ComboDropList(void)
 /*************************************************************************
 	Initialise the Window based object ready for use.	
 *************************************************************************/
-void ComboDropList::initialise(void)
+void ComboDropList::initialiseComponents(void)
 {
-	Listbox::initialise();
+	Listbox::initialiseComponents();
 
 	// set-up scroll bars so they return capture to us.
-	d_vertScrollbar->setRestoreCapture(true);
-	d_horzScrollbar->setRestoreCapture(true);
-}
-
-
-/*************************************************************************
-	Add drop-list specific events
-*************************************************************************/
-void ComboDropList::addComboDropListEvents(void)
-{
-	addEvent(EventListSelectionAccepted);
+	getVertScrollbar()->setRestoreCapture(true);
+	getHorzScrollbar()->setRestoreCapture(true);
 }
 
 
@@ -105,7 +101,7 @@ void ComboDropList::onMouseMove(MouseEventArgs& e)
 	// if mouse is within our area (but not our children)
 	if (isHit(e.position))
 	{
-		if (getChildAtPosition(e.position) == NULL)
+		if (!getChildAtPosition(e.position))
 		{
 			// handle auto-arm
 			if (d_autoArm)
@@ -118,18 +114,13 @@ void ComboDropList::onMouseMove(MouseEventArgs& e)
 				//
 				// Convert mouse position to absolute window pixels
 				//
-				Point localPos(screenToWindow(e.position));
-
-				if (getMetricsMode() == Relative)
-				{
-					localPos = relativeToAbsolute(localPos);
-				}
+				Point localPos(CoordConverter::screenToWindow(*this, e.position));
 
 				// check for an item under the mouse
 				ListboxItem* selItem = getItemAtPoint(localPos);
 
 				// if an item is under mouse, select it
-				if (selItem != NULL)
+				if (selItem)
 				{
 					setItemSelectState(selItem, true);
 				}
@@ -191,7 +182,7 @@ void ComboDropList::onMouseButtonUp(MouseEventArgs& e)
 
 	if (e.button == LeftButton)
 	{
-		if (d_armed && (getChildAtPosition(e.position) == NULL))
+		if (d_armed && (getChildAtPosition(e.position) == 0))
 		{
 			releaseInput();
 
