@@ -1,34 +1,38 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUICombobox.cpp
 	created:	13/4/2004
 	author:		Paul D Turner
 	
 	purpose:	Implementation of Combobox base class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "elements/CEGUICombobox.h"
 #include "elements/CEGUIEditbox.h"
 #include "elements/CEGUIPushButton.h"
 #include "elements/CEGUIComboDropList.h"
 #include "elements/CEGUIListboxItem.h"
-
+#include "CEGUIWindowManager.h"
 
 /*************************************************************************
 	General information.
@@ -42,6 +46,8 @@
 namespace CEGUI
 {
 const String Combobox::EventNamespace("Combobox");
+const String Combobox::WidgetTypeName("CEGUI/Combobox");
+
 
 /*************************************************************************
 	Definitions for Properties for this class
@@ -52,10 +58,6 @@ ComboboxProperties::CaratIndex					Combobox::d_caratIndexProperty;
 ComboboxProperties::EditSelectionStart			Combobox::d_selStartProperty;
 ComboboxProperties::EditSelectionLength			Combobox::d_selLengthProperty;
 ComboboxProperties::MaxEditTextLength			Combobox::d_maxTextLengthProperty;
-ComboboxProperties::NormalEditTextColour		Combobox::d_normalTextColourProperty;
-ComboboxProperties::SelectedEditTextColour		Combobox::d_selectedTextColourProperty;
-ComboboxProperties::ActiveEditSelectionColour	Combobox::d_activeSelectionColourProperty;
-ComboboxProperties::InactiveEditSelectionColour	Combobox::d_inactiveSelectionColourProperty;
 ComboboxProperties::SortList					Combobox::d_sortProperty;
 ComboboxProperties::ForceVertScrollbar			Combobox::d_forceVertProperty;
 ComboboxProperties::ForceHorzScrollbar			Combobox::d_forceHorzProperty;
@@ -66,27 +68,34 @@ ComboboxProperties::SingleClickMode				Combobox::d_singleClickOperationProperty;
 	Constants
 *************************************************************************/
 // event names from edit box
-const String Combobox::EventReadOnlyModeChanged( (utf8*)"ReadOnlyChanged" );
-const String Combobox::EventValidationStringChanged( (utf8*)"ValidationStringChanged" );
-const String Combobox::EventMaximumTextLengthChanged( (utf8*)"MaximumTextLengthChanged" );
-const String Combobox::EventTextInvalidated( (utf8*)"TextInvalidatedEvent" );
-const String Combobox::EventInvalidEntryAttempted( (utf8*)"InvalidEntryAttempted" );
-const String Combobox::EventCaratMoved( (utf8*)"CaratMoved" );
-const String Combobox::EventTextSelectionChanged( (utf8*)"TextSelectionChanged" );
-const String Combobox::EventEditboxFull( (utf8*)"EditboxFullEvent" );
-const String Combobox::EventTextAccepted( (utf8*)"TextAcceptedEvent" );
+const String Combobox::EventReadOnlyModeChanged( "ReadOnlyChanged" );
+const String Combobox::EventValidationStringChanged( "ValidationStringChanged" );
+const String Combobox::EventMaximumTextLengthChanged( "MaximumTextLengthChanged" );
+const String Combobox::EventTextInvalidated( "TextInvalidatedEvent" );
+const String Combobox::EventInvalidEntryAttempted( "InvalidEntryAttempted" );
+const String Combobox::EventCaratMoved( "CaratMoved" );
+const String Combobox::EventTextSelectionChanged( "TextSelectionChanged" );
+const String Combobox::EventEditboxFull( "EditboxFullEvent" );
+const String Combobox::EventTextAccepted( "TextAcceptedEvent" );
 
 // event names from list box
-const String Combobox::EventListContentsChanged( (utf8*)"ListContentsChanged" );
-const String Combobox::EventListSelectionChanged( (utf8*)"ListSelectionChanged" );
-const String Combobox::EventSortModeChanged( (utf8*)"SortModeChanged" );
-const String Combobox::EventVertScrollbarModeChanged( (utf8*)"VertScrollbarModeChanged" );
-const String Combobox::EventHorzScrollbarModeChanged( (utf8*)"HorzScrollbarModeChanged" );
+const String Combobox::EventListContentsChanged( "ListContentsChanged" );
+const String Combobox::EventListSelectionChanged( "ListSelectionChanged" );
+const String Combobox::EventSortModeChanged( "SortModeChanged" );
+const String Combobox::EventVertScrollbarModeChanged( "VertScrollbarModeChanged" );
+const String Combobox::EventHorzScrollbarModeChanged( "HorzScrollbarModeChanged" );
 
 // events we produce / generate ourselves
-const String Combobox::EventDropListDisplayed( (utf8*)"DropListDisplayed" );
-const String Combobox::EventDropListRemoved( (utf8*)"DropListRemoved" );
-const String Combobox::EventListSelectionAccepted( (utf8*)"ListSelectionAccepted" );
+const String Combobox::EventDropListDisplayed( "DropListDisplayed" );
+const String Combobox::EventDropListRemoved( "DropListRemoved" );
+const String Combobox::EventListSelectionAccepted( "ListSelectionAccepted" );
+
+/*************************************************************************
+    Child Widget name suffix constants
+*************************************************************************/
+const String Combobox::EditboxNameSuffix( "__auto_editbox__" );
+const String Combobox::DropListNameSuffix( "__auto_droplist__" );
+const String Combobox::ButtonNameSuffix( "__auto_button__" );
 
 	
 /*************************************************************************
@@ -97,7 +106,6 @@ Combobox::Combobox(const String& type, const String& name) :
 {
 	d_singleClickOperation = false;
 
-	addComboboxEvents();
 	addComboboxProperties();
 }
 
@@ -113,40 +121,36 @@ Combobox::~Combobox(void)
 /*************************************************************************
 	Initialise the Window based object ready for use.
 *************************************************************************/
-void Combobox::initialise(void)
+void Combobox::initialiseComponents(void)
 {
-	d_editbox	= createEditbox(getName() + "__auto_editbox__");
-	d_droplist	= createDropList(getName() + "__auto_droplist__");
-	d_button	= createPushButton(getName() + "__auto_button__");
-    d_droplist->setFont(getFont());
-    d_editbox->setFont(getFont());
-
-	addChildWindow(d_editbox);
-	addChildWindow(d_droplist);
-	addChildWindow(d_button);
+	Editbox* editbox        = getEditbox();
+	ComboDropList* droplist = getDropList();
+	PushButton* button      = getPushButton();
+    droplist->setFont(getFont());
+    editbox->setFont(getFont());
 
 	// internal event wiring
-	d_button->subscribeEvent(PushButton::EventMouseButtonDown, Event::Subscriber(&CEGUI::Combobox::button_PressHandler, this));
-	d_droplist->subscribeEvent(ComboDropList::EventListSelectionAccepted, Event::Subscriber(&CEGUI::Combobox::droplist_SelectionAcceptedHandler, this));
-	d_droplist->subscribeEvent(Window::EventHidden, Event::Subscriber(&CEGUI::Combobox::droplist_HiddenHandler, this));
-	d_editbox->subscribeEvent(Window::EventMouseButtonDown, Event::Subscriber(&CEGUI::Combobox::editbox_MouseDownHandler, this));
+	button->subscribeEvent(PushButton::EventMouseButtonDown, Event::Subscriber(&CEGUI::Combobox::button_PressHandler, this));
+	droplist->subscribeEvent(ComboDropList::EventListSelectionAccepted, Event::Subscriber(&CEGUI::Combobox::droplist_SelectionAcceptedHandler, this));
+	droplist->subscribeEvent(Window::EventHidden, Event::Subscriber(&CEGUI::Combobox::droplist_HiddenHandler, this));
+	editbox->subscribeEvent(Window::EventMouseButtonDown, Event::Subscriber(&CEGUI::Combobox::editbox_MouseDownHandler, this));
 
 	// event forwarding setup
-	d_editbox->subscribeEvent(Editbox::EventReadOnlyModeChanged, Event::Subscriber(&CEGUI::Combobox::editbox_ReadOnlyChangedHandler, this));
-	d_editbox->subscribeEvent(Editbox::EventValidationStringChanged, Event::Subscriber(&CEGUI::Combobox::editbox_ValidationStringChangedHandler, this));
-	d_editbox->subscribeEvent(Editbox::EventMaximumTextLengthChanged, Event::Subscriber(&CEGUI::Combobox::editbox_MaximumTextLengthChangedHandler, this));
-	d_editbox->subscribeEvent(Editbox::EventTextInvalidated, Event::Subscriber(&CEGUI::Combobox::editbox_TextInvalidatedEventHandler, this));
-	d_editbox->subscribeEvent(Editbox::EventInvalidEntryAttempted, Event::Subscriber(&CEGUI::Combobox::editbox_InvalidEntryAttemptedHandler, this));
-	d_editbox->subscribeEvent(Editbox::EventCaratMoved, Event::Subscriber(&CEGUI::Combobox::editbox_CaratMovedHandler, this));
-	d_editbox->subscribeEvent(Editbox::EventTextSelectionChanged, Event::Subscriber(&CEGUI::Combobox::editbox_TextSelectionChangedHandler, this));
-	d_editbox->subscribeEvent(Editbox::EventEditboxFull, Event::Subscriber(&CEGUI::Combobox::editbox_EditboxFullEventHandler, this));
-	d_editbox->subscribeEvent(Editbox::EventTextAccepted, Event::Subscriber(&CEGUI::Combobox::editbox_TextAcceptedEventHandler, this));
-	d_editbox->subscribeEvent(Editbox::EventTextChanged, Event::Subscriber(&CEGUI::Combobox::editbox_TextChangedEventHandler, this));
-	d_droplist->subscribeEvent(Listbox::EventListContentsChanged, Event::Subscriber(&CEGUI::Combobox::listbox_ListContentsChangedHandler, this));
-	d_droplist->subscribeEvent(Listbox::EventSelectionChanged, Event::Subscriber(&CEGUI::Combobox::listbox_ListSelectionChangedHandler, this));
-	d_droplist->subscribeEvent(Listbox::EventSortModeChanged, Event::Subscriber(&CEGUI::Combobox::listbox_SortModeChangedHandler, this));
-	d_droplist->subscribeEvent(Listbox::EventVertScrollbarModeChanged, Event::Subscriber(&CEGUI::Combobox::listbox_VertScrollModeChangedHandler, this));
-	d_droplist->subscribeEvent(Listbox::EventHorzScrollbarModeChanged, Event::Subscriber(&CEGUI::Combobox::listbox_HorzScrollModeChangedHandler, this));
+	editbox->subscribeEvent(Editbox::EventReadOnlyModeChanged, Event::Subscriber(&CEGUI::Combobox::editbox_ReadOnlyChangedHandler, this));
+	editbox->subscribeEvent(Editbox::EventValidationStringChanged, Event::Subscriber(&CEGUI::Combobox::editbox_ValidationStringChangedHandler, this));
+	editbox->subscribeEvent(Editbox::EventMaximumTextLengthChanged, Event::Subscriber(&CEGUI::Combobox::editbox_MaximumTextLengthChangedHandler, this));
+	editbox->subscribeEvent(Editbox::EventTextInvalidated, Event::Subscriber(&CEGUI::Combobox::editbox_TextInvalidatedEventHandler, this));
+	editbox->subscribeEvent(Editbox::EventInvalidEntryAttempted, Event::Subscriber(&CEGUI::Combobox::editbox_InvalidEntryAttemptedHandler, this));
+	editbox->subscribeEvent(Editbox::EventCaratMoved, Event::Subscriber(&CEGUI::Combobox::editbox_CaratMovedHandler, this));
+	editbox->subscribeEvent(Editbox::EventTextSelectionChanged, Event::Subscriber(&CEGUI::Combobox::editbox_TextSelectionChangedHandler, this));
+	editbox->subscribeEvent(Editbox::EventEditboxFull, Event::Subscriber(&CEGUI::Combobox::editbox_EditboxFullEventHandler, this));
+	editbox->subscribeEvent(Editbox::EventTextAccepted, Event::Subscriber(&CEGUI::Combobox::editbox_TextAcceptedEventHandler, this));
+	editbox->subscribeEvent(Editbox::EventTextChanged, Event::Subscriber(&CEGUI::Combobox::editbox_TextChangedEventHandler, this));
+	droplist->subscribeEvent(Listbox::EventListContentsChanged, Event::Subscriber(&CEGUI::Combobox::listbox_ListContentsChangedHandler, this));
+	droplist->subscribeEvent(Listbox::EventSelectionChanged, Event::Subscriber(&CEGUI::Combobox::listbox_ListSelectionChangedHandler, this));
+	droplist->subscribeEvent(Listbox::EventSortModeChanged, Event::Subscriber(&CEGUI::Combobox::listbox_SortModeChangedHandler, this));
+	droplist->subscribeEvent(Listbox::EventVertScrollbarModeChanged, Event::Subscriber(&CEGUI::Combobox::listbox_VertScrollModeChangedHandler, this));
+	droplist->subscribeEvent(Listbox::EventHorzScrollbarModeChanged, Event::Subscriber(&CEGUI::Combobox::listbox_HorzScrollModeChangedHandler, this));
 
 	// put components in their initial positions
 	performChildWindowLayout();
@@ -159,9 +163,10 @@ void Combobox::initialise(void)
 void Combobox::showDropList(void)
 {
 	// Display the box
-	d_droplist->show();
-	d_droplist->activate();
-	d_droplist->captureInput();
+    ComboDropList* droplist = getDropList();
+	droplist->show();
+	droplist->activate();
+	droplist->captureInput();
 
 	// Fire off event
 	WindowEventArgs args(this);
@@ -176,7 +181,7 @@ void Combobox::hideDropList(void)
 {
 	// the natural order of things when this happens will ensure the list is
 	// hidden and events are fired.
-	d_droplist->releaseInput();
+	getDropList()->releaseInput();
 }
 
 
@@ -185,7 +190,7 @@ void Combobox::hideDropList(void)
 *************************************************************************/
 bool Combobox::hasInputFocus(void) const
 {
-	return d_editbox->hasInputFocus();
+	return getEditbox()->hasInputFocus();
 }
 
 
@@ -194,7 +199,7 @@ bool Combobox::hasInputFocus(void) const
 *************************************************************************/
 bool Combobox::isReadOnly(void) const
 {
-	return d_editbox->isReadOnly();
+	return getEditbox()->isReadOnly();
 }
 
 
@@ -204,7 +209,7 @@ bool Combobox::isReadOnly(void) const
 *************************************************************************/
 bool Combobox::isTextValid(void) const
 {
-	return d_editbox->isTextValid();
+	return getEditbox()->isTextValid();
 }
 
 
@@ -213,7 +218,7 @@ bool Combobox::isTextValid(void) const
 *************************************************************************/
 const String& Combobox::getValidationString(void) const
 {
-	return d_editbox->getValidationString();
+	return getEditbox()->getValidationString();
 }
 
 
@@ -222,7 +227,7 @@ const String& Combobox::getValidationString(void) const
 *************************************************************************/
 size_t Combobox::getCaratIndex(void) const
 {
-	return d_editbox->getCaratIndex();
+	return getEditbox()->getCaratIndex();
 }
 
 
@@ -231,7 +236,7 @@ size_t Combobox::getCaratIndex(void) const
 *************************************************************************/
 size_t Combobox::getSelectionStartIndex(void) const
 {
-	return d_editbox->getSelectionStartIndex();
+	return getEditbox()->getSelectionStartIndex();
 }
 
 
@@ -240,7 +245,7 @@ size_t Combobox::getSelectionStartIndex(void) const
 *************************************************************************/
 size_t Combobox::getSelectionEndIndex(void) const
 {
-	return d_editbox->getSelectionEndIndex();
+	return getEditbox()->getSelectionEndIndex();
 }
 
 
@@ -249,7 +254,7 @@ size_t Combobox::getSelectionEndIndex(void) const
 *************************************************************************/
 size_t Combobox::getSelectionLength(void) const
 {
-	return d_editbox->getSelectionLength();
+	return getEditbox()->getSelectionLength();
 }
 
 
@@ -258,47 +263,7 @@ size_t Combobox::getSelectionLength(void) const
 *************************************************************************/
 size_t Combobox::getMaxTextLength(void) const
 {
-	return d_editbox->getMaxTextLength();
-}
-
-
-/*************************************************************************
-	return the currently set colour to be used for rendering Editbox text
-	in the normal, unselected state.
-*************************************************************************/
-colour Combobox::getNormalTextColour(void) const	
-{
-	return d_editbox->getNormalTextColour();
-}
-
-
-/*************************************************************************
-	return the currently set colour to be used for rendering the Editbox
-	text when within the selected region.
-*************************************************************************/
-colour Combobox::getSelectedTextColour(void) const
-{
-	return d_editbox->getSelectedTextColour();
-}
-
-
-/*************************************************************************
-	return the currently set colour to be used for rendering the Editbox
-	selection highlight when the Editbox is active.
-*************************************************************************/
-colour Combobox::getNormalSelectBrushColour(void) const
-{
-	return d_editbox->getNormalSelectBrushColour();
-}
-
-
-/*************************************************************************
-	return the currently set colour to be used for rendering the Editbox
-	selection highlight when the Editbox is inactive.
-*************************************************************************/
-colour Combobox::getInactiveSelectBrushColour(void) const
-{
-	return d_editbox->getInactiveSelectBrushColour();
+	return getEditbox()->getMaxTextLength();
 }
 
 
@@ -307,7 +272,7 @@ colour Combobox::getInactiveSelectBrushColour(void) const
 *************************************************************************/
 void Combobox::setReadOnly(bool setting)
 {
-	d_editbox->setReadOnly(setting);
+	getEditbox()->setReadOnly(setting);
 }
 
 
@@ -316,7 +281,7 @@ void Combobox::setReadOnly(bool setting)
 *************************************************************************/
 void Combobox::setValidationString(const String& validation_string)
 {
-	d_editbox->setValidationString(validation_string);
+	getEditbox()->setValidationString(validation_string);
 }
 
 
@@ -325,7 +290,7 @@ void Combobox::setValidationString(const String& validation_string)
 *************************************************************************/
 void Combobox::setCaratIndex(size_t carat_pos)
 {
-	d_editbox->setCaratIndex(carat_pos);
+	getEditbox()->setCaratIndex(carat_pos);
 }
 
 
@@ -334,7 +299,7 @@ void Combobox::setCaratIndex(size_t carat_pos)
 *************************************************************************/
 void Combobox::setSelection(size_t start_pos, size_t end_pos)
 {
-	d_editbox->setSelection(start_pos, end_pos);
+	getEditbox()->setSelection(start_pos, end_pos);
 }
 
 
@@ -343,47 +308,7 @@ void Combobox::setSelection(size_t start_pos, size_t end_pos)
 *************************************************************************/
 void Combobox::setMaxTextLength(size_t max_len)
 {
-	d_editbox->setMaxTextLength(max_len);
-}
-
-
-/*************************************************************************
-	Set the colour to be used for rendering Editbox text in the normal,
-	unselected state.
-*************************************************************************/
-void Combobox::setNormalTextColour(colour col)
-{
-	d_editbox->setNormalTextColour(col);
-}
-
-
-/*************************************************************************
-	Set the colour to be used for rendering the Editbox text when within
-	the selected region.	
-*************************************************************************/
-void Combobox::setSelectedTextColour(colour col)
-{
-	d_editbox->setSelectedTextColour(col);
-}
-
-
-/*************************************************************************
-	Set the colour to be used for rendering the Editbox selection
-	highlight when the Editbox is active.
-*************************************************************************/
-void Combobox::setNormalSelectBrushColour(colour col)
-{
-	d_editbox->setNormalSelectBrushColour(col);
-}
-
-
-/*************************************************************************
-	Set the colour to be used for rendering the Editbox selection
-	highlight when the Editbox is inactive.
-*************************************************************************/
-void Combobox::setInactiveSelectBrushColour(colour col)
-{
-	d_editbox->setInactiveSelectBrushColour(col);
+	getEditbox()->setMaxTextLength(max_len);
 }
 
 
@@ -392,7 +317,7 @@ void Combobox::setInactiveSelectBrushColour(colour col)
 *************************************************************************/
 size_t Combobox::getItemCount(void) const
 {
-	return d_droplist->getItemCount();
+	return getDropList()->getItemCount();
 }
 
 
@@ -401,7 +326,7 @@ size_t Combobox::getItemCount(void) const
 *************************************************************************/
 ListboxItem* Combobox::getSelectedItem(void) const
 {
-	return d_droplist->getFirstSelectedItem();
+	return getDropList()->getFirstSelectedItem();
 }
 
 
@@ -410,7 +335,7 @@ ListboxItem* Combobox::getSelectedItem(void) const
 *************************************************************************/
 ListboxItem* Combobox::getListboxItemFromIndex(size_t index) const
 {
-	return d_droplist->getListboxItemFromIndex(index);
+	return getDropList()->getListboxItemFromIndex(index);
 }
 
 
@@ -419,7 +344,7 @@ ListboxItem* Combobox::getListboxItemFromIndex(size_t index) const
 *************************************************************************/
 size_t Combobox::getItemIndex(const ListboxItem* item) const
 {
-	return d_droplist->getItemIndex(item);
+	return getDropList()->getItemIndex(item);
 }
 
 
@@ -428,7 +353,7 @@ size_t Combobox::getItemIndex(const ListboxItem* item) const
 *************************************************************************/
 bool Combobox::isSortEnabled(void) const
 {
-	return d_droplist->isSortEnabled();
+	return getDropList()->isSortEnabled();
 }
 
 
@@ -437,7 +362,7 @@ bool Combobox::isSortEnabled(void) const
 *************************************************************************/
 bool Combobox::isItemSelected(size_t index) const
 {
-	return d_droplist->isItemSelected(index);
+	return getDropList()->isItemSelected(index);
 }
 
 
@@ -446,7 +371,7 @@ bool Combobox::isItemSelected(size_t index) const
 *************************************************************************/
 ListboxItem* Combobox::findItemWithText(const String& text, const ListboxItem* start_item)
 {
-	return d_droplist->findItemWithText(text, start_item);
+	return getDropList()->findItemWithText(text, start_item);
 }
 
 
@@ -455,7 +380,7 @@ ListboxItem* Combobox::findItemWithText(const String& text, const ListboxItem* s
 *************************************************************************/
 bool Combobox::isListboxItemInList(const ListboxItem* item) const
 {
-	return d_droplist->isListboxItemInList(item);
+	return getDropList()->isListboxItemInList(item);
 }
 
 
@@ -464,7 +389,7 @@ bool Combobox::isListboxItemInList(const ListboxItem* item) const
 *************************************************************************/
 void Combobox::resetList(void)
 {
-	d_droplist->resetList();
+	getDropList()->resetList();
 }
 
 
@@ -473,7 +398,7 @@ void Combobox::resetList(void)
 *************************************************************************/
 void Combobox::addItem(ListboxItem* item)
 {
-	d_droplist->addItem(item);
+	getDropList()->addItem(item);
 }
 
 
@@ -483,7 +408,7 @@ void Combobox::addItem(ListboxItem* item)
 *************************************************************************/
 void Combobox::insertItem(ListboxItem* item, const ListboxItem* position)
 {
-	d_droplist->insertItem(item, position);
+	getDropList()->insertItem(item, position);
 }
 
 
@@ -492,7 +417,7 @@ void Combobox::insertItem(ListboxItem* item, const ListboxItem* position)
 *************************************************************************/
 void Combobox::removeItem(const ListboxItem* item)
 {
-	d_droplist->removeItem(item);
+	getDropList()->removeItem(item);
 }
 
 
@@ -501,7 +426,7 @@ void Combobox::removeItem(const ListboxItem* item)
 *************************************************************************/
 void Combobox::clearAllSelections(void)
 {
-	d_droplist->clearAllSelections();
+	getDropList()->clearAllSelections();
 }
 
 
@@ -510,7 +435,7 @@ void Combobox::clearAllSelections(void)
 *************************************************************************/
 void Combobox::setSortingEnabled(bool setting)
 {
-	d_droplist->setSortingEnabled(setting);
+	getDropList()->setSortingEnabled(setting);
 }
 
 
@@ -519,7 +444,7 @@ void Combobox::setSortingEnabled(bool setting)
 *************************************************************************/
 void Combobox::setShowVertScrollbar(bool setting)
 {
-	d_droplist->setShowVertScrollbar(setting);
+	getDropList()->setShowVertScrollbar(setting);
 }
 
 
@@ -528,7 +453,7 @@ void Combobox::setShowVertScrollbar(bool setting)
 *************************************************************************/
 void Combobox::setShowHorzScrollbar(bool setting)
 {
-	d_droplist->setShowHorzScrollbar(setting);
+	getDropList()->setShowHorzScrollbar(setting);
 }
 
 
@@ -537,7 +462,7 @@ void Combobox::setShowHorzScrollbar(bool setting)
 *************************************************************************/
 void Combobox::setItemSelectState(ListboxItem* item, bool state)
 {
-	d_droplist->setItemSelectState(item, state);
+	getDropList()->setItemSelectState(item, state);
 }
 
 
@@ -546,7 +471,7 @@ void Combobox::setItemSelectState(ListboxItem* item, bool state)
 *************************************************************************/
 void Combobox::setItemSelectState(size_t item_index, bool state)
 {
-	d_droplist->setItemSelectState(item_index, state);
+	getDropList()->setItemSelectState(item_index, state);
 }
 
 
@@ -556,24 +481,7 @@ void Combobox::setItemSelectState(size_t item_index, bool state)
 *************************************************************************/
 void Combobox::handleUpdatedListItemData(void)
 {
-	d_droplist->handleUpdatedItemData();
-}
-
-
-/*************************************************************************
-	Add combo box specific events
-*************************************************************************/
-void Combobox::addComboboxEvents(void)
-{
-	addEvent(EventReadOnlyModeChanged);			addEvent(EventValidationStringChanged);
-	addEvent(EventMaximumTextLengthChanged);	addEvent(EventTextInvalidated);
-	addEvent(EventInvalidEntryAttempted);		addEvent(EventCaratMoved);
-	addEvent(EventTextSelectionChanged);		addEvent(EventEditboxFull);
-	addEvent(EventTextAccepted);				addEvent(EventListContentsChanged);
-	addEvent(EventListSelectionChanged);		addEvent(EventSortModeChanged);
-	addEvent(EventVertScrollbarModeChanged);	addEvent(EventHorzScrollbarModeChanged);
-	addEvent(EventDropListDisplayed);			addEvent(EventDropListRemoved);
-	addEvent(EventListSelectionAccepted);
+    getDropList()->handleUpdatedItemData();
 }
 
 
@@ -736,8 +644,8 @@ void Combobox::onListSelectionAccepted(WindowEventArgs& e)
 void Combobox::onFontChanged(WindowEventArgs& e)
 {
     // Propagate to children
-    d_editbox->setFont(getFont());
-    d_droplist->setFont(getFont());
+    getEditbox()->setFont(getFont());
+    getDropList()->setFont(getFont());
 
     // Call base class handler
     Window::onFontChanged(e);
@@ -749,11 +657,13 @@ void Combobox::onFontChanged(WindowEventArgs& e)
 *************************************************************************/
 void Combobox::onTextChanged(WindowEventArgs& e)
 {
+    Editbox* editbox = getEditbox();
+
 	// update ourselves only if needed (prevents perpetual event loop & stack overflow)
-	if (d_editbox->getText() != d_text)
+	if (editbox->getText() != d_text)
 	{
 		// done before doing base class processing so event subscribers see 'updated' version of this.
-		d_editbox->setText(d_text);
+		editbox->setText(d_text);
 		e.handled = true;
 
 		Window::onTextChanged(e);
@@ -768,20 +678,22 @@ void Combobox::onTextChanged(WindowEventArgs& e)
 *************************************************************************/
 bool Combobox::button_PressHandler(const EventArgs& e)
 {
+    ComboDropList* droplist = getDropList();
+
 	showDropList();
 
 	// if there is an item with the same text as the edit box, pre-select it
-	ListboxItem* item = d_droplist->findItemWithText(d_editbox->getText(), NULL);
+	ListboxItem* item = droplist->findItemWithText(getEditbox()->getText(), 0);
 
-	if (item != NULL)
+	if (item)
 	{
-		d_droplist->setItemSelectState(item, true);
-		d_droplist->ensureItemIsVisible(item);
+		droplist->setItemSelectState(item, true);
+		droplist->ensureItemIsVisible(item);
 	}
 	// no matching item, so select nothing
 	else
 	{
-		d_droplist->clearAllSelections();
+		droplist->clearAllSelections();
 	}
 
 	return true;
@@ -796,26 +708,27 @@ bool Combobox::droplist_SelectionAcceptedHandler(const EventArgs& e)
 	// copy the text from the selected item into the edit box
 	ListboxItem* item = ((ComboDropList*)((WindowEventArgs&)e).window)->getFirstSelectedItem();
 
-	if (item != NULL)
+	if (item)
 	{
+        Editbox* editbox = getEditbox();
 		// Put the text from the list item into the edit box
-		d_editbox->setText(item->getText());
+		editbox->setText(item->getText());
 
 		// select text if it's editable, and move carat to end
 		if (!isReadOnly())
 		{
-			d_editbox->setSelection(0, item->getText().length());
-			d_editbox->setCaratIndex(item->getText().length());
+			editbox->setSelection(0, item->getText().length());
+			editbox->setCaratIndex(item->getText().length());
 		}
 
-		d_editbox->setCaratIndex(0);
+		editbox->setCaratIndex(0);
 
 		// fire off an event of our own
 		WindowEventArgs args(this);
 		onListSelectionAccepted(args);
 
 		// finally, activate the edit box
-		d_editbox->activate();
+		editbox->activate();
 	}
 
 	return true;
@@ -842,23 +755,26 @@ bool Combobox::editbox_MouseDownHandler(const EventArgs& e)
 	// only interested in left button
 	if (((const MouseEventArgs&)e).button == LeftButton)
 	{
+        Editbox* editbox = getEditbox();
+
 		// if edit box is read-only, show list
-		if (d_editbox->isReadOnly())
+		if (editbox->isReadOnly())
 		{
+            ComboDropList* droplist = getDropList();
 			showDropList();
 
 			// if there is an item with the same text as the edit box, pre-select it
-			ListboxItem* item = d_droplist->findItemWithText(d_editbox->getText(), NULL);
+			ListboxItem* item = droplist->findItemWithText(editbox->getText(), 0);
 
-			if (item != NULL)
+			if (item)
 			{
-				d_droplist->setItemSelectState(item, true);
-				d_droplist->ensureItemIsVisible(item);
+				droplist->setItemSelectState(item, true);
+				droplist->ensureItemIsVisible(item);
 			}
 			// no matching item, so select nothing
 			else
 			{
-				d_droplist->clearAllSelections();
+				droplist->clearAllSelections();
 			}
 
 			return true;
@@ -874,7 +790,7 @@ bool Combobox::editbox_MouseDownHandler(const EventArgs& e)
 *************************************************************************/
 bool Combobox::isVertScrollbarAlwaysShown(void) const
 {
-	return d_droplist->isVertScrollbarAlwaysShown();
+	return getDropList()->isVertScrollbarAlwaysShown();
 }
 
 
@@ -883,7 +799,7 @@ bool Combobox::isVertScrollbarAlwaysShown(void) const
 *************************************************************************/
 bool Combobox::isHorzScrollbarAlwaysShown(void) const
 {
-	return d_droplist->isHorzScrollbarAlwaysShown();
+	return getDropList()->isHorzScrollbarAlwaysShown();
 }
 
 
@@ -900,10 +816,6 @@ void Combobox::addComboboxProperties(void)
 	addProperty(&d_maxTextLengthProperty);
 	addProperty(&d_selStartProperty);
 	addProperty(&d_selLengthProperty);
-	addProperty(&d_normalTextColourProperty);
-	addProperty(&d_selectedTextColourProperty);
-	addProperty(&d_activeSelectionColourProperty);
-	addProperty(&d_inactiveSelectionColourProperty);
 	addProperty(&d_caratIndexProperty);
 	addProperty(&d_singleClickOperationProperty);
 }
@@ -914,9 +826,11 @@ void Combobox::addComboboxProperties(void)
 *************************************************************************/
 void Combobox::activateEditbox(void)
 {
-	if (!d_editbox->isActive())
+    Editbox* editbox = getEditbox();
+
+	if (!editbox->isActive())
 	{
-		d_editbox->activate();
+		editbox->activate();
 	}
 }
 
@@ -950,7 +864,7 @@ bool Combobox::getSingleClickEnabled(void) const
 *************************************************************************/
 bool Combobox::isDropDownListVisible(void) const
 {
-	return d_droplist->isVisible();
+	return getDropList()->isVisible();
 }
 
 
@@ -960,7 +874,36 @@ bool Combobox::isDropDownListVisible(void) const
 void Combobox::setSingleClickEnabled(bool setting)
 {
 	d_singleClickOperation = setting;
-	d_droplist->setAutoArmEnabled(setting);
+	getDropList()->setAutoArmEnabled(setting);
+}
+
+
+/************************************************************************
+    Return a pointer to the Editbox component widget for this Combobox.
+************************************************************************/
+Editbox* Combobox::getEditbox() const
+{
+    return static_cast<Editbox*>(WindowManager::getSingleton().getWindow(
+                                 getName() + EditboxNameSuffix));
+}
+
+/************************************************************************
+    Return a pointer to the PushButton component widget for this Combobox.
+************************************************************************/
+PushButton* Combobox::getPushButton() const
+{
+    return static_cast<PushButton*>(WindowManager::getSingleton().getWindow(
+                                    getName() + ButtonNameSuffix));
+}
+
+/************************************************************************
+    Return a pointer to the ComboDropList component widget for this
+    Combobox.
+************************************************************************/
+ComboDropList* Combobox::getDropList() const
+{
+    return static_cast<ComboDropList*>(WindowManager::getSingleton().getWindow(
+                                       getName() + DropListNameSuffix));
 }
 
 
