@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIFrameWindow.cpp
 	created:	13/4/2004
 	author:		Paul D Turner
 	
 	purpose:	Implementation of FrameWindow base class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "elements/CEGUIFrameWindow.h"
 #include "elements/CEGUITitlebar.h"
 #include "elements/CEGUIPushButton.h"
@@ -31,11 +35,14 @@
 #include "CEGUIExceptions.h"
 #include "CEGUIImagesetManager.h"
 #include "CEGUIImageset.h"
+#include "CEGUICoordConverter.h"
 
 // Start of CEGUI namespace section
 namespace CEGUI
 {
 const String FrameWindow::EventNamespace("FrameWindow");
+const String FrameWindow::WidgetTypeName("CEGUI/FrameWindow");
+
 
 /*************************************************************************
 	Definitions for Properties
@@ -48,8 +55,6 @@ FrameWindowProperties::RollUpState				FrameWindow::d_rollUpStateProperty;
 FrameWindowProperties::RollUpEnabled			FrameWindow::d_rollUpEnabledProperty;
 FrameWindowProperties::DragMovingEnabled		FrameWindow::d_dragMovingEnabledProperty;
 FrameWindowProperties::SizingBorderThickness	FrameWindow::d_sizingBorderThicknessProperty;
-FrameWindowProperties::TitlebarFont				FrameWindow::d_titlebarFontProperty;
-FrameWindowProperties::CaptionColour			FrameWindow::d_captionColourProperty;
 FrameWindowProperties::NSSizingCursorImage      FrameWindow::d_nsSizingCursorProperty;
 FrameWindowProperties::EWSizingCursorImage      FrameWindow::d_ewSizingCursorProperty;
 FrameWindowProperties::NWSESizingCursorImage    FrameWindow::d_nwseSizingCursorProperty;
@@ -60,12 +65,18 @@ FrameWindowProperties::NESWSizingCursorImage    FrameWindow::d_neswSizingCursorP
 	Constants
 *************************************************************************/
 // additional event names for this window
-const String FrameWindow::EventRollupToggled( (utf8*)"RollupToggled" );
-const String FrameWindow::EventCloseClicked( (utf8*)"CloseClicked" );
+const String FrameWindow::EventRollupToggled( "RollupToggled" );
+const String FrameWindow::EventCloseClicked( "CloseClicked" );
 
 // other bits
 const float FrameWindow::DefaultSizingBorderSize	= 8.0f;
 
+/*************************************************************************
+    Child Widget name suffix constants
+*************************************************************************/
+const String FrameWindow::TitlebarNameSuffix( "__auto_titlebar__" );
+const String FrameWindow::CloseButtonNameSuffix( "__auto_closebutton__" );
+
 
 /*************************************************************************
 	Constructor
@@ -82,9 +93,8 @@ FrameWindow::FrameWindow(const String& type, const String& name) :
 
 	d_borderSize		= DefaultSizingBorderSize;
 
-	d_nsSizingCursor = d_ewSizingCursor = d_neswSizingCursor = d_nwseSizingCursor = NULL;
+	d_nsSizingCursor = d_ewSizingCursor = d_neswSizingCursor = d_nwseSizingCursor = 0;
 
-	addFrameWindowEvents();
 	addFrameWindowProperties();
 }
 
@@ -100,28 +110,38 @@ FrameWindow::~FrameWindow(void)
 /*************************************************************************
 	Initialises the Window based object ready for use.
 *************************************************************************/
-void FrameWindow::initialise(void)
+void FrameWindow::initialiseComponents(void)
 {
-	// create child windows
-	d_titlebar		= createTitlebar(getName() + "__auto_titlebar__");
-	d_closeButton	= createCloseButton(getName() + "__auto_closebutton__");
+    // get component windows
+    Titlebar* titlebar = getTitlebar();
+    PushButton* closeButton = getCloseButton();
 
-	// add child controls
-	if (d_titlebar != NULL)
-	{
-		d_titlebar->setDraggingEnabled(d_dragMovable);
-		addChildWindow(d_titlebar);
-	}
+    // configure titlebar
+    titlebar->setDraggingEnabled(d_dragMovable);
+    titlebar->setText(d_text);
 
-	if (d_closeButton != NULL)
-	{
-		addChildWindow(d_closeButton);
+    // bind handler to close button 'Click' event
+    closeButton->subscribeEvent(PushButton::EventClicked, Event::Subscriber(&CEGUI::FrameWindow::closeClickHandler, this));
 
-		// bind handler to close button 'Click' event
-		d_closeButton->subscribeEvent(PushButton::EventClicked, Event::Subscriber(&CEGUI::FrameWindow::closeClickHandler, this));
-	}
+    performChildWindowLayout();
+}
 
-	performChildWindowLayout();
+
+/*************************************************************************
+    Return whether the title bar for this window is enabled.
+*************************************************************************/
+bool FrameWindow::isTitleBarEnabled(void) const
+{
+    return !getTitlebar()->isDisabled();
+}
+
+
+/*************************************************************************
+    Return whether this close button for this window is enabled.
+*************************************************************************/
+bool FrameWindow::isCloseButtonEnabled(void) const
+{
+    return !getCloseButton()->isDisabled();
 }
 
 
@@ -149,14 +169,9 @@ void FrameWindow::setFrameEnabled(bool setting)
 *************************************************************************/
 void FrameWindow::setTitleBarEnabled(bool setting)
 {
-    try
-    {
-        Window* titlebar = WindowManager::getSingleton().getWindow(getName() + "__auto_titlebar__");
-        titlebar->setEnabled(setting);
-        titlebar->setVisible(setting);
-    }
-    catch (UnknownObjectException)
-    {}
+    Window* titlebar = getTitlebar();
+    titlebar->setEnabled(setting);
+    titlebar->setVisible(setting);
 }
 
 
@@ -165,14 +180,9 @@ void FrameWindow::setTitleBarEnabled(bool setting)
 *************************************************************************/
 void FrameWindow::setCloseButtonEnabled(bool setting)
 {
-    try
-    {
-        Window* closebtn = WindowManager::getSingleton().getWindow(getName() + "__auto_closebutton__");
-        closebtn->setEnabled(setting);
-        closebtn->setVisible(setting);
-    }
-    catch (UnknownObjectException)
-    {}
+    Window* closebtn = getCloseButton();
+    closebtn->setEnabled(setting);
+    closebtn->setVisible(setting);
 }
 
 
@@ -208,55 +218,15 @@ void FrameWindow::toggleRollup(void)
 }
 
 
-/*************************************************************************
-	Set the font to use for the title bar text	
-*************************************************************************/
-void FrameWindow::setTitlebarFont(const String& name)
-{
-    try
-    {
-        WindowManager::getSingleton().getWindow(getName() + "__auto_titlebar__")->setFont(name);
-    }
-    catch (UnknownObjectException)
-    {}
-}
-
-
-/*************************************************************************
-	Set the font to use for the title bar text	
-*************************************************************************/
-void FrameWindow::setTitlebarFont(Font* font)
-{
-    try
-    {
-        WindowManager::getSingleton().getWindow(getName() + "__auto_titlebar__")->setFont(font);
-    }
-    catch (UnknownObjectException)
-    {}
-}
-
-
 /*************************************************************************
 	Move the window by the pixel offsets specified in 'offset'.	
 *************************************************************************/
 void FrameWindow::offsetPixelPosition(const Vector2& offset)
 {
-    UVector2 uOffset;
+    UVector2 uOffset(cegui_absdim(PixelAligned(offset.d_x)),
+                     cegui_absdim(PixelAligned(offset.d_y)));
 
-    if (getMetricsMode() == Relative)
-    {
-        Size sz = getParentSize();
-
-        uOffset.d_x = cegui_reldim((sz.d_width != 0) ? offset.d_x / sz.d_width : 0);
-        uOffset.d_y = cegui_reldim((sz.d_height != 0) ? offset.d_y / sz.d_height : 0);
-    }
-    else
-    {
-        uOffset.d_x = cegui_absdim(PixelAligned(offset.d_x));
-        uOffset.d_y = cegui_absdim(PixelAligned(offset.d_y));
-    }
-
-    setWindowPosition(d_area.getPosition() + uOffset);
+    setPosition(d_area.getPosition() + uOffset);
 }
 
 
@@ -336,10 +306,7 @@ FrameWindow::SizingLocation FrameWindow::getSizingBorderAtPoint(const Point& pt)
 *************************************************************************/
 void FrameWindow::moveLeftEdge(float delta)
 {
-    float orgWidth = getAbsoluteWidth();
-    float adjustment;
-    float* minDim;
-    float* maxDim;
+    float orgWidth = d_pixelSize.d_width;
     URect area(d_area);
 
     // ensure that we only size to the set constraints.
@@ -356,36 +323,24 @@ void FrameWindow::moveLeftEdge(float delta)
     else if (newWidth < minWidth)
         delta = orgWidth - minWidth;
 
-    // we use the active metrics mode to decide which component of the window edge to modify
-    if (getMetricsMode() == Relative)
-    {
-        float base = getParentWidth();
-        adjustment = (base != 0) ? delta / base : 0;
-        minDim = &area.d_min.d_x.d_scale;
-        maxDim = &area.d_max.d_x.d_scale;
-    }
-    else
-    {
-        adjustment = PixelAligned(delta);
-        minDim = &area.d_min.d_x.d_offset;
-        maxDim = &area.d_max.d_x.d_offset;
-    }
+    // ensure adjustment will be whole pixel
+    float adjustment = PixelAligned(delta);
 
     if (d_horzAlign == HA_RIGHT)
     {
-        *maxDim -= adjustment;
+        area.d_max.d_x.d_offset -= adjustment;
     }
     else if (d_horzAlign == HA_CENTRE)
     {
-        *maxDim -= adjustment * 0.5f;
-        *minDim += adjustment * 0.5f;
+        area.d_max.d_x.d_offset -= adjustment * 0.5f;
+        area.d_min.d_x.d_offset += adjustment * 0.5f;
     }
     else
     {
-        *minDim += adjustment;
+        area.d_min.d_x.d_offset += adjustment;
     }
 
-    setWindowArea_impl(area.d_min, area.getSize(), d_horzAlign == HA_LEFT);
+    setArea_impl(area.d_min, area.getSize(), d_horzAlign == HA_LEFT);
 }
 /*************************************************************************
 	move the window's right edge by 'delta'.  The rest of the window
@@ -394,10 +349,7 @@ void FrameWindow::moveLeftEdge(float delta)
 void FrameWindow::moveRightEdge(float delta)
 {
     // store this so we can work out how much size actually changed
-    float orgWidth = getAbsoluteWidth();
-    float adjustment;
-    float* minDim;
-    float* maxDim;
+    float orgWidth = d_pixelSize.d_width;
     URect area(d_area);
 
     // ensure that we only size to the set constraints.
@@ -413,39 +365,27 @@ void FrameWindow::moveRightEdge(float delta)
         delta = maxWidth - orgWidth;
     else if (newWidth < minWidth)
         delta = minWidth - orgWidth;
-    
-    // we use the active metrics mode to decide which component of the window edge to modify
-    if (getMetricsMode() == Relative)
-    {
-        float base = getParentWidth();
-        adjustment = (base != 0) ? delta / base : 0;
-        minDim = &area.d_min.d_x.d_scale;
-        maxDim = &area.d_max.d_x.d_scale;
-    }
-    else
-    {
-        adjustment = PixelAligned(delta);
-        minDim = &area.d_min.d_x.d_offset;
-        maxDim = &area.d_max.d_x.d_offset;
-    }
 
-    *maxDim += adjustment;
+    // ensure adjustment will be whole pixel
+    float adjustment = PixelAligned(delta);
+
+    area.d_max.d_x.d_offset += adjustment;
 
     if (d_horzAlign == HA_RIGHT)
     {
-        *maxDim += adjustment;
-        *minDim += adjustment;
+        area.d_max.d_x.d_offset += adjustment;
+        area.d_min.d_x.d_offset += adjustment;
     }
     else if (d_horzAlign == HA_CENTRE)
     {
-        *maxDim += adjustment * 0.5f;
-        *minDim += adjustment * 0.5f;
+        area.d_max.d_x.d_offset += adjustment * 0.5f;
+        area.d_min.d_x.d_offset += adjustment * 0.5f;
     }
 
-    setWindowArea_impl(area.d_min, area.getSize(), d_horzAlign == HA_RIGHT);
+    setArea_impl(area.d_min, area.getSize(), d_horzAlign == HA_RIGHT);
 
     // move the dragging point so mouse remains 'attached' to edge of window
-    d_dragPoint.d_x += getAbsoluteWidth() - orgWidth;
+    d_dragPoint.d_x += d_pixelSize.d_width - orgWidth;
 }
 
 /*************************************************************************
@@ -454,10 +394,7 @@ void FrameWindow::moveRightEdge(float delta)
 *************************************************************************/
 void FrameWindow::moveTopEdge(float delta)
 {
-    float orgHeight = getAbsoluteHeight();
-    float adjustment;
-    float* minDim;
-    float* maxDim;
+    float orgHeight = d_pixelSize.d_height;
     URect area(d_area);
 
     // ensure that we only size to the set constraints.
@@ -474,36 +411,24 @@ void FrameWindow::moveTopEdge(float delta)
     else if (newHeight < minHeight)
         delta = orgHeight - minHeight;
 
-    // we use the active metrics mode to decide which component of the window edge to modify
-    if (getMetricsMode() == Relative)
-    {
-        float base = getParentHeight();
-        adjustment = (base != 0) ? delta / base : 0;
-        minDim = &area.d_min.d_y.d_scale;
-        maxDim = &area.d_max.d_y.d_scale;
-    }
-    else
-    {
-        adjustment = PixelAligned(delta);
-        minDim = &area.d_min.d_y.d_offset;
-        maxDim = &area.d_max.d_y.d_offset;
-    }
+    // ensure adjustment will be whole pixel
+    float adjustment = PixelAligned(delta);
 
     if (d_vertAlign == VA_BOTTOM)
     {
-        *maxDim -= adjustment;
+        area.d_max.d_y.d_offset -= adjustment;
     }
     else if (d_vertAlign == VA_CENTRE)
     {
-        *maxDim -= adjustment * 0.5f;
-        *minDim += adjustment * 0.5f;
+        area.d_max.d_y.d_offset -= adjustment * 0.5f;
+        area.d_min.d_y.d_offset += adjustment * 0.5f;
     }
     else
     {
-        *minDim += adjustment;
+        area.d_min.d_y.d_offset += adjustment;
     }
 
-    setWindowArea_impl(area.d_min, area.getSize(), d_vertAlign == VA_TOP);
+    setArea_impl(area.d_min, area.getSize(), d_vertAlign == VA_TOP);
 }
 
 
@@ -514,10 +439,7 @@ void FrameWindow::moveTopEdge(float delta)
 void FrameWindow::moveBottomEdge(float delta)
 {
     // store this so we can work out how much size actually changed
-    float orgHeight = getAbsoluteHeight();
-    float adjustment;
-    float* minDim;
-    float* maxDim;
+    float orgHeight = d_pixelSize.d_height;
     URect area(d_area);
 
     // ensure that we only size to the set constraints.
@@ -533,49 +455,27 @@ void FrameWindow::moveBottomEdge(float delta)
         delta = maxHeight - orgHeight;
     else if (newHeight < minHeight)
         delta = minHeight - orgHeight;
-    
-    // we use the active metrics mode to decide which component of the window edge to modify
-    if (getMetricsMode() == Relative)
-    {
-        float base = getParentHeight();
-        adjustment = (base != 0) ? delta / base : 0;
-        minDim = &area.d_min.d_y.d_scale;
-        maxDim = &area.d_max.d_y.d_scale;
-    }
-    else
-    {
-        adjustment = PixelAligned(delta);
-        minDim = &area.d_min.d_y.d_offset;
-        maxDim = &area.d_max.d_y.d_offset;
-    }
 
-    *maxDim += adjustment;
+    // ensure adjustment will be whole pixel
+    float adjustment = PixelAligned(delta);
+
+    area.d_max.d_y.d_offset += adjustment;
 
     if (d_vertAlign == VA_BOTTOM)
     {
-        *maxDim += adjustment;
-        *minDim += adjustment;
+        area.d_max.d_y.d_offset += adjustment;
+        area.d_min.d_y.d_offset += adjustment;
     }
     else if (d_vertAlign == VA_CENTRE)
     {
-        *maxDim += adjustment * 0.5f;
-        *minDim += adjustment * 0.5f;
+        area.d_max.d_y.d_offset += adjustment * 0.5f;
+        area.d_min.d_y.d_offset += adjustment * 0.5f;
     }
 
-    setWindowArea_impl(area.d_min, area.getSize(), d_vertAlign == VA_BOTTOM);
+    setArea_impl(area.d_min, area.getSize(), d_vertAlign == VA_BOTTOM);
 
     // move the dragging point so mouse remains 'attached' to edge of window
-    d_dragPoint.d_y += getAbsoluteHeight() - orgHeight;
-}
-
-
-/*************************************************************************
-	Add frame window specific events	
-*************************************************************************/
-void FrameWindow::addFrameWindowEvents(void)
-{
-	addEvent(EventRollupToggled);
-	addEvent(EventCloseClicked);
+    d_dragPoint.d_y += d_pixelSize.d_height - orgHeight;
 }
 
 
@@ -634,6 +534,7 @@ void FrameWindow::setCursorForPoint(const Point& pt) const
 void FrameWindow::onRollupToggled(WindowEventArgs& e)
 {
     requestRedraw();
+    notifyClippingChanged();
 
 	fireEvent(EventRollupToggled, e, EventNamespace);
 }
@@ -664,12 +565,7 @@ void FrameWindow::onMouseMove(MouseEventArgs& e)
 
 	if (isSizingEnabled())
 	{
-		Point localMousePos(screenToWindow(e.position));
-
-		if (getMetricsMode() == Relative)
-		{
-			localMousePos = relativeToAbsolute(localMousePos);
-		}
+		Point localMousePos(CoordConverter::screenToWindow(*this, e.position));
 
 		if (d_beingSized)
 		{
@@ -725,12 +621,7 @@ void FrameWindow::onMouseButtonDown(MouseEventArgs& e)
 		if (isSizingEnabled())
 		{
 			// get position of mouse as co-ordinates local to this window.
-			Point localPos(screenToWindow(e.position));
-
-			if (getMetricsMode() == Relative)
-			{
-				localPos = relativeToAbsolute(localPos);
-			}
+			Point localPos(CoordConverter::screenToWindow(*this, e.position));
 
 			// if the mouse is on the sizing border
 			if (getSizingBorderAtPoint(localPos) != SizingNone)
@@ -741,13 +632,13 @@ void FrameWindow::onMouseButtonDown(MouseEventArgs& e)
 					// setup the 'dragging' state variables
 					d_beingSized = true;
 					d_dragPoint = localPos;
+					e.handled = true;
 				}
 
 			}
 
 		}
 
-		e.handled = true;
 	}
 
 }
@@ -761,7 +652,7 @@ void FrameWindow::onMouseButtonUp(MouseEventArgs& e)
 	// default processing (this is now essential as it controls event firing).
 	Window::onMouseButtonUp(e);
 
-	if (e.button == LeftButton)
+	if (e.button == LeftButton && isCapturedByThis())
 	{
 		// release our capture on the input data
 		releaseInput();
@@ -791,9 +682,10 @@ void FrameWindow::onCaptureLost(WindowEventArgs& e)
 *************************************************************************/
 void FrameWindow::onTextChanged(WindowEventArgs& e)
 {
+    Window::onTextChanged(e);
     // pass this onto titlebar component.
-    WindowManager::getSingleton().getWindow(getName() + "__auto_titlebar__")->setText(d_text);
-    // redo child layout
+    getTitlebar()->setText(d_text);
+    // maybe the user is using a fontdim for titlebar dimensions ;)
     performChildWindowLayout();
 }
 
@@ -804,7 +696,7 @@ void FrameWindow::onTextChanged(WindowEventArgs& e)
 void FrameWindow::onActivated(ActivationEventArgs& e)
 {
 	Window::onActivated(e);
-	d_titlebar->requestRedraw();
+	getTitlebar()->requestRedraw();
 }
 
 
@@ -814,7 +706,7 @@ void FrameWindow::onActivated(ActivationEventArgs& e)
 void FrameWindow::onDeactivated(ActivationEventArgs& e)
 {
 	Window::onDeactivated(e);
-	d_titlebar->requestRedraw();
+	getTitlebar()->requestRedraw();
 }
 
 
@@ -827,33 +719,12 @@ void FrameWindow::setDragMovingEnabled(bool setting)
 	{
 		d_dragMovable = setting;
 
-        try
-        {
-            static_cast<Titlebar*>(WindowManager::getSingleton().getWindow(getName() + "__auto_titlebar__"))->setDraggingEnabled(setting);
-        }
-        catch (UnknownObjectException)
-        {}
+        getTitlebar()->setDraggingEnabled(setting);
     }
 
 }
 
 
-/*************************************************************************
-	Return the font being used for the title bar text
-*************************************************************************/
-const Font* FrameWindow::getTitlebarFont(void) const
-{
-    try
-    {
-        return WindowManager::getSingleton().getWindow(getName() + "__auto_titlebar__")->getFont();
-    }
-    catch (UnknownObjectException)
-    {
-        return 0;
-    }
-}
-
-
 /*************************************************************************
 	Add properties for this class
 *************************************************************************/
@@ -867,8 +738,6 @@ void FrameWindow::addFrameWindowProperties(void)
 	addProperty(&d_rollUpStateProperty);
 	addProperty(&d_dragMovingEnabledProperty);
 	addProperty(&d_sizingBorderThicknessProperty);
-	addProperty(&d_titlebarFontProperty);
-	addProperty(&d_captionColourProperty);
     addProperty(&d_nsSizingCursorProperty);
     addProperty(&d_ewSizingCursorProperty);
     addProperty(&d_nwseSizingCursorProperty);
@@ -877,81 +746,118 @@ void FrameWindow::addFrameWindowProperties(void)
 
 
 /*************************************************************************
-	Return the current colour used for rendering the caption text
+    return the image used for the north-south sizing cursor.
 *************************************************************************/
-colour FrameWindow::getCaptionColour(void) const
-{
-    return static_cast<Titlebar*>(WindowManager::getSingleton().getWindow(getName() + "__auto_titlebar__"))->getCaptionColour();
-}
-
-
-/*************************************************************************
-	Sets the colour to be used for rendering the caption text.
-*************************************************************************/
-void FrameWindow::setCaptionColour(colour col)
-{
-    static_cast<Titlebar*>(WindowManager::getSingleton().getWindow(getName() + "__auto_titlebar__"))->setCaptionColour(col);
-}
-
-
 const Image* FrameWindow::getNSSizingCursorImage() const
 {
     return d_nsSizingCursor;
 }
 
+/*************************************************************************
+    return the image used for the east-west sizing cursor.
+*************************************************************************/
 const Image* FrameWindow::getEWSizingCursorImage() const
 {
     return d_ewSizingCursor;
 }
 
+/*************************************************************************
+    return the image used for the northwest-southeast sizing cursor.
+*************************************************************************/
 const Image* FrameWindow::getNWSESizingCursorImage() const
 {
     return d_nwseSizingCursor;
 }
 
+/*************************************************************************
+    return the image used for the northeast-southwest sizing cursor.
+*************************************************************************/
 const Image* FrameWindow::getNESWSizingCursorImage() const
 {
     return d_neswSizingCursor;
 }
 
+/*************************************************************************
+    set the image used for the north-south sizing cursor.
+*************************************************************************/
 void FrameWindow::setNSSizingCursorImage(const Image* image)
 {
     d_nsSizingCursor = image;
 }
 
+/*************************************************************************
+    set the image used for the east-west sizing cursor.
+*************************************************************************/
 void FrameWindow::setEWSizingCursorImage(const Image* image)
 {
     d_ewSizingCursor = image;
 }
 
+/*************************************************************************
+    set the image used for the northwest-southeast sizing cursor.
+*************************************************************************/
 void FrameWindow::setNWSESizingCursorImage(const Image* image)
 {
     d_nwseSizingCursor = image;
 }
 
+/*************************************************************************
+    set the image used for the northeast-southwest sizing cursor.
+*************************************************************************/
 void FrameWindow::setNESWSizingCursorImage(const Image* image)
 {
     d_neswSizingCursor = image;
 }
 
+/*************************************************************************
+    set the image used for the north-south sizing cursor.
+*************************************************************************/
 void FrameWindow::setNSSizingCursorImage(const String& imageset, const String& image)
 {
     d_nsSizingCursor = &ImagesetManager::getSingleton().getImageset(imageset)->getImage(image);
 }
 
+/*************************************************************************
+    set the image used for the east-west sizing cursor.
+*************************************************************************/
 void FrameWindow::setEWSizingCursorImage(const String& imageset, const String& image)
 {
     d_ewSizingCursor = &ImagesetManager::getSingleton().getImageset(imageset)->getImage(image);
 }
 
+/*************************************************************************
+    set the image used for the northwest-southeast sizing cursor.
+*************************************************************************/
 void FrameWindow::setNWSESizingCursorImage(const String& imageset, const String& image)
 {
     d_nwseSizingCursor = &ImagesetManager::getSingleton().getImageset(imageset)->getImage(image);
 }
 
+/*************************************************************************
+    set the image used for the northeast-southwest sizing cursor.
+*************************************************************************/
 void FrameWindow::setNESWSizingCursorImage(const String& imageset, const String& image)
 {
     d_neswSizingCursor = &ImagesetManager::getSingleton().getImageset(imageset)->getImage(image);
 }
 
+/*************************************************************************
+    Return a pointer to the Titlebar component widget for this FrameWindow.
+*************************************************************************/
+Titlebar* FrameWindow::getTitlebar() const
+{
+    return static_cast<Titlebar*>(WindowManager::getSingleton().getWindow(
+                                  getName() + TitlebarNameSuffix));
+}
+
+/*************************************************************************
+    Return a pointer to the close button component widget for this
+    FrameWindow.
+*************************************************************************/
+PushButton* FrameWindow::getCloseButton() const
+{
+    return static_cast<PushButton*>(WindowManager::getSingleton().getWindow(
+                                    getName() + CloseButtonNameSuffix));
+}
+
 } // End of  CEGUI namespace section
