@@ -1,31 +1,35 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIListHeader.cpp
 	created:	13/4/2004
 	author:		Paul D Turner
 	
 	purpose:	Implementation of ListHeader widget base class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "elements/CEGUIListHeader.h"
 #include "CEGUIExceptions.h"
-
+#include "CEGUICoordConverter.h"
 #include <sstream>
 
 
@@ -33,6 +37,15 @@
 namespace CEGUI
 {
 const String ListHeader::EventNamespace("ListHeader");
+const String ListHeader::WidgetTypeName("CEGUI/ListHeader");
+
+/*************************************************************************
+    ListHeaderWindowRenderer constructor
+*************************************************************************/
+ListHeaderWindowRenderer::ListHeaderWindowRenderer(const String& name) :
+    WindowRenderer(name, "ListHeader")
+{
+}
 
 /*************************************************************************
 	Properties for this class
@@ -48,30 +61,36 @@ ListHeaderProperties::SortDirection			ListHeader::d_sortDirectionProperty;
 	Constants
 *************************************************************************/
 // Event names
-const String ListHeader::EventSortColumnChanged( (utf8*)"SortColumnChanged" );
-const String ListHeader::EventSortDirectionChanged( (utf8*)"SortDirectionChanged" );
-const String ListHeader::EventSegmentSized( (utf8*)"SegmentSized" );
-const String ListHeader::EventSegmentClicked( (utf8*)"SegmentClicked" );
-const String ListHeader::EventSplitterDoubleClicked( (utf8*)"SplitterDoubleClicked" );
-const String ListHeader::EventSegmentSequenceChanged( (utf8*)"SegmentSequenceChanged" );
-const String ListHeader::EventSegmentAdded( (utf8*)"SegmentAdded" );
-const String ListHeader::EventSegmentRemoved( (utf8*)"SegmentRemoved" );
-const String ListHeader::EventSortSettingChanged( (utf8*)"SortSettingChanged" );
-const String ListHeader::EventDragMoveSettingChanged( (utf8*)"DragMoveSettingChanged" );
-const String ListHeader::EventDragSizeSettingChanged( (utf8*)"DragSizeSettingChanged" );
-const String ListHeader::EventSegmentRenderOffsetChanged( (utf8*)"SegmentOffsetChanged" );
+const String ListHeader::EventSortColumnChanged( "SortColumnChanged" );
+const String ListHeader::EventSortDirectionChanged( "SortDirectionChanged" );
+const String ListHeader::EventSegmentSized( "SegmentSized" );
+const String ListHeader::EventSegmentClicked( "SegmentClicked" );
+const String ListHeader::EventSplitterDoubleClicked( "SplitterDoubleClicked" );
+const String ListHeader::EventSegmentSequenceChanged( "SegmentSequenceChanged" );
+const String ListHeader::EventSegmentAdded( "SegmentAdded" );
+const String ListHeader::EventSegmentRemoved( "SegmentRemoved" );
+const String ListHeader::EventSortSettingChanged( "SortSettingChanged" );
+const String ListHeader::EventDragMoveSettingChanged( "DragMoveSettingChanged" );
+const String ListHeader::EventDragSizeSettingChanged( "DragSizeSettingChanged" );
+const String ListHeader::EventSegmentRenderOffsetChanged( "SegmentOffsetChanged" );
 
 // values
 const float	ListHeader::ScrollSpeed	= 8.0f;
 const float	ListHeader::MinimumSegmentPixelWidth	= 20.0f;
 
+/*************************************************************************
+    Child Widget name suffix constants
+*************************************************************************/
+const char ListHeader::SegmentNameSuffix[] = "__auto_seg_";
+
+
 
 /*************************************************************************
 	Constructor for the list header base class.
 *************************************************************************/
 ListHeader::ListHeader(const String& type, const String& name) :
 	Window(type, name),
-	d_sortSegment(NULL),
+	d_sortSegment(0),
 	d_sizingEnabled(true),
 	d_sortingEnabled(true),
 	d_movingEnabled(true),
@@ -79,7 +98,6 @@ ListHeader::ListHeader(const String& type, const String& name) :
 	d_segmentOffset(0.0f),
 	d_sortDir(ListHeaderSegment::None)
 {
-	addListHeaderEvents();
 	addHeaderProperties();
 }
 
@@ -108,7 +126,7 @@ ListHeaderSegment& ListHeader::getSegmentFromColumn(uint column) const
 {
 	if (column >= getColumnCount())
 	{
-		throw InvalidRequestException((utf8*)"ListHeader::getSegmentFromColumn - requested column index is out of range for this ListHeader.");
+		throw InvalidRequestException("ListHeader::getSegmentFromColumn - requested column index is out of range for this ListHeader.");
 	}
 	else
 	{
@@ -133,7 +151,7 @@ ListHeaderSegment& ListHeader::getSegmentFromID(uint id) const
 	}
 
 	// No such segment found, throw exception
-	throw InvalidRequestException((utf8*)"ListHeader::getSegmentFromID - no segment with the requested ID is attached to this ListHeader.");
+	throw InvalidRequestException("ListHeader::getSegmentFromID - no segment with the requested ID is attached to this ListHeader.");
 }
 
 
@@ -142,9 +160,9 @@ ListHeaderSegment& ListHeader::getSegmentFromID(uint id) const
 *************************************************************************/
 ListHeaderSegment& ListHeader::getSortSegment(void) const
 {
-	if (d_sortSegment == NULL)
+	if (!d_sortSegment)
 	{
-		throw	InvalidRequestException((utf8*)"ListHeader::getSortSegment - Sort segment was NULL!  (No segments are attached to the ListHeader?)");
+		throw	InvalidRequestException("ListHeader::getSortSegment - Sort segment was invalid!  (No segments are attached to the ListHeader?)");
 	}
 	else
 	{
@@ -169,7 +187,7 @@ uint ListHeader::getColumnFromSegment(const ListHeaderSegment& segment) const
 	}
 
 	// No such segment found, throw exception
-	throw InvalidRequestException((utf8*)"ListHeader::getColumnFromSegment - the given ListHeaderSegment is not attached to this ListHeader.");
+	throw InvalidRequestException("ListHeader::getColumnFromSegment - the given ListHeaderSegment is not attached to this ListHeader.");
 }
 
 
@@ -188,7 +206,7 @@ uint ListHeader::getColumnFromID(uint id) const
 	}
 
 	// No such segment found, throw exception
-	throw InvalidRequestException((utf8*)"ListHeader::getColumnFromID - no column with the requested ID is available on this ListHeader.");
+	throw InvalidRequestException("ListHeader::getColumnFromID - no column with the requested ID is available on this ListHeader.");
 }
 
 
@@ -217,7 +235,7 @@ uint ListHeader::getColumnWithText(const String& text) const
 	}
 
 	// No such segment found, throw exception
-	throw InvalidRequestException((utf8*)"ListHeader::getColumnWithText - no column with the text '" + text + "' is attached to this ListHeader.");
+	throw InvalidRequestException("ListHeader::getColumnWithText - no column with the text '" + text + "' is attached to this ListHeader.");
 }
 
 
@@ -235,11 +253,11 @@ float ListHeader::getPixelOffsetToSegment(const ListHeaderSegment& segment) cons
 			return offset;
 		}
 
-		offset += d_segments[i]->getAbsoluteWidth();
+		offset += d_segments[i]->getPixelSize().d_width;
 	}
 
 	// No such segment found, throw exception
-	throw InvalidRequestException((utf8*)"ListHeader::getPixelOffsetToSegment - the given ListHeaderSegment is not attached to this ListHeader.");
+	throw InvalidRequestException("ListHeader::getPixelOffsetToSegment - the given ListHeaderSegment is not attached to this ListHeader.");
 }
 
 
@@ -250,7 +268,7 @@ float ListHeader::getPixelOffsetToColumn(uint column) const
 {
 	if (column >= getColumnCount())
 	{
-		throw InvalidRequestException((utf8*)"ListHeader::getPixelOffsetToColumn - requested column index is out of range for this ListHeader.");
+		throw InvalidRequestException("ListHeader::getPixelOffsetToColumn - requested column index is out of range for this ListHeader.");
 	}
 	else
 	{
@@ -258,7 +276,7 @@ float ListHeader::getPixelOffsetToColumn(uint column) const
 
 		for (uint i = 0; i < column; ++i)
 		{
-			offset += d_segments[i]->getAbsoluteWidth();
+			offset += d_segments[i]->getPixelSize().d_width;
 		}
 
 		return offset;
@@ -276,7 +294,7 @@ float ListHeader::getTotalSegmentsPixelExtent(void) const
 
 	for (uint i = 0; i < getColumnCount(); ++i)
 	{
-		extent += d_segments[i]->getAbsoluteWidth();
+		extent += d_segments[i]->getPixelSize().d_width;
 	}
 
 	return extent;
@@ -284,17 +302,17 @@ float ListHeader::getTotalSegmentsPixelExtent(void) const
 
 
 /*************************************************************************
-	Return the pixel width of the segment at the specified column index.
+	Return the width of the segment at the specified column index.
 *************************************************************************/
-float ListHeader::getColumnPixelWidth(uint column) const
+UDim ListHeader::getColumnWidth(uint column) const
 {
 	if (column >= getColumnCount())
 	{
-		throw InvalidRequestException((utf8*)"ListHeader::getColumnPixelWidth - requested column index is out of range for this ListHeader.");
+		throw InvalidRequestException("ListHeader::getColumnWidth - requested column index is out of range for this ListHeader.");
 	}
 	else
 	{
-		return d_segments[column]->getAbsoluteWidth();
+		return d_segments[column]->getWidth();
 	}
 
 }
@@ -369,7 +387,7 @@ void ListHeader::setSortDirection(ListHeaderSegment::SortDirection direction)
 		d_sortDir = direction;
 
 		// set direction of current sort segment
-		if (d_sortSegment != NULL)
+		if (d_sortSegment)
 		{
 			d_sortSegment->setSortDirection(direction);
 		}
@@ -398,7 +416,7 @@ void ListHeader::setSortColumn(uint column)
 {
 	if (column >= getColumnCount())
 	{
-		throw InvalidRequestException((utf8*)"ListHeader::setSortColumn - specified column index is out of range for this ListHeader.");
+		throw InvalidRequestException("ListHeader::setSortColumn - specified column index is out of range for this ListHeader.");
 	}
 	else
 	{
@@ -406,7 +424,7 @@ void ListHeader::setSortColumn(uint column)
 		if (d_sortSegment != d_segments[column])
 		{
 			// set sort direction on 'old' sort segment to none.
-			if (d_sortSegment != NULL)
+			if (d_sortSegment)
 			{
 				d_sortSegment->setSortDirection(ListHeaderSegment::None);
 			}
@@ -483,7 +501,7 @@ void ListHeader::setColumnDraggingEnabled(bool setting)
 /*************************************************************************
 	Add a new column segment to the header.
 *************************************************************************/
-void ListHeader::addColumn(const String& text, uint id, float width)
+void ListHeader::addColumn(const String& text, uint id, const UDim& width)
 {
 	// add just inserts at end.
 	insertColumn(text, id, width, getColumnCount());
@@ -493,7 +511,7 @@ void ListHeader::addColumn(const String& text, uint id, float width)
 /*************************************************************************
 	Insert a new column segment into the header
 *************************************************************************/
-void ListHeader::insertColumn(const String& text, uint id, float width, uint position)
+void ListHeader::insertColumn(const String& text, uint id, const UDim& width, uint position)
 {
 	// if position is too big, insert at end.
 	if (position > getColumnCount())
@@ -514,7 +532,7 @@ void ListHeader::insertColumn(const String& text, uint id, float width, uint pos
 	onSegmentAdded(args);
 
 	// if sort segment is invalid, make it valid now we have a segment attached
-	if (d_sortSegment == NULL)
+	if (!d_sortSegment)
 	{
 		setSortColumn(position);
 	}
@@ -528,7 +546,7 @@ void ListHeader::removeColumn(uint column)
 {
 	if (column >= getColumnCount())
 	{
-		throw InvalidRequestException((utf8*)"ListHeader::removeColumn - specified column index is out of range for this ListHeader.");
+		throw InvalidRequestException("ListHeader::removeColumn - specified column index is out of range for this ListHeader.");
 	}
 	else
 	{
@@ -550,7 +568,7 @@ void ListHeader::removeColumn(uint column)
 			// no columns, set sort segment to NULL
 			else
 			{
-				d_sortSegment = NULL;
+				d_sortSegment = 0;
 			}
 
 		}
@@ -578,7 +596,7 @@ void ListHeader::moveColumn(uint column, uint position)
 {
 	if (column >= getColumnCount())
 	{
-		throw InvalidRequestException((utf8*)"ListHeader::moveColumn - specified column index is out of range for this ListHeader.");
+		throw InvalidRequestException("ListHeader::moveColumn - specified column index is out of range for this ListHeader.");
 	}
 	else
 	{
@@ -609,7 +627,7 @@ void ListHeader::moveColumn(uint column, uint position)
 /*************************************************************************
 	Insert a new column segment into the header
 *************************************************************************/
-void ListHeader::insertColumn(const String& text, uint id, float width, const ListHeaderSegment& position)
+void ListHeader::insertColumn(const String& text, uint id, const UDim& width, const ListHeaderSegment& position)
 {
 	insertColumn(text, id, width, getColumnFromSegment(position));
 }
@@ -671,17 +689,17 @@ void ListHeader::setSegmentOffset(float offset)
 
 
 /*************************************************************************
-	Set the pixel width of the specified column.	
+	Set the width of the specified column.
 *************************************************************************/
-void ListHeader::setColumnPixelWidth(uint column, float width)
+void ListHeader::setColumnWidth(uint column, const UDim& width)
 {
 	if (column >= getColumnCount())
 	{
-		throw InvalidRequestException((utf8*)"ListHeader::setColumnPixelWidth - specified column index is out of range for this ListHeader.");
+		throw InvalidRequestException("ListHeader::setColumnWidth - specified column index is out of range for this ListHeader.");
 	}
 	else
 	{
-		d_segments[column]->setWidth(Absolute, width);
+		d_segments[column]->setWidth(width);
 
 		layoutSegments();
 
@@ -697,22 +715,24 @@ void ListHeader::setColumnPixelWidth(uint column, float width)
 	Create initialise and return a ListHeaderSegment object, with all
 	events subscribed and ready to use.
 *************************************************************************/
-ListHeaderSegment* ListHeader::createInitialisedSegment(const String& text, uint id, float width)
+ListHeaderSegment* ListHeader::createInitialisedSegment(const String& text, uint id, const UDim& width)
 {
 	// Build unique name
 	std::stringstream name;
-	name << getName().c_str() << "__auto_seg_" << d_uniqueIDNumber;
+	name << getName().c_str() << SegmentNameSuffix << d_uniqueIDNumber;
 
 	// create segment.
 	ListHeaderSegment* newseg = createNewSegment(name.str());
 	d_uniqueIDNumber++;
 
 	// setup segment;
-	newseg->setMetricsMode(Relative);
-	newseg->setSize(Size(width, 1.0f));
-	newseg->setMinimumSize(absoluteToRelative_impl(NULL, Size(MinimumSegmentPixelWidth, 0.0f)));
+	newseg->setSize(UVector2(width, cegui_reldim(1.0f)));
+	newseg->setMinSize(UVector2(cegui_absdim(MinimumSegmentPixelWidth), cegui_absdim(0)));
 	newseg->setText(text);
 	newseg->setID(id);
+    newseg->setSizingEnabled(d_sizingEnabled);
+    newseg->setDragMovingEnabled(d_movingEnabled);
+    newseg->setClickable(d_sortingEnabled);
 
 	// subscribe events we listen to
 	newseg->subscribeEvent(ListHeaderSegment::EventSegmentSized, Event::Subscriber(&CEGUI::ListHeader::segmentSizedHandler, this));
@@ -730,7 +750,7 @@ ListHeaderSegment* ListHeader::createInitialisedSegment(const String& text, uint
 *************************************************************************/
 void ListHeader::layoutSegments(void)
 {
-	Point pos(-d_segmentOffset, 0.0f);
+	UVector2 pos(cegui_absdim(-d_segmentOffset), cegui_absdim(0.0f));
 
 	for (uint i = 0; i < getColumnCount(); ++i)
 	{
@@ -882,23 +902,16 @@ bool ListHeader::segmentMovedHandler(const EventArgs& e)
 	if (isHit(mousePos))
 	{
 		// get mouse position as something local
-		Point localMousePos(screenToWindow(mousePos));
+		Point localMousePos(CoordConverter::screenToWindow(*this, mousePos));
 
 		// set up to allow for current offsets
 		float currwidth = -d_segmentOffset;
 
-		// get required figures as pixels
-		if (getMetricsMode() == Relative)
-		{
-			localMousePos = relativeToAbsolute(localMousePos);
-			currwidth = relativeToAbsoluteX(currwidth);
-		}
-
 		// calculate column where dragged segment was dropped
         uint col;
 		for (col = 0; col < getColumnCount(); ++col)
 		{
-			currwidth += d_segments[col]->getAbsoluteWidth();
+			currwidth += d_segments[col]->getPixelSize().d_width;
 
 			if (localMousePos.d_x < currwidth)
 			{
@@ -937,7 +950,7 @@ bool ListHeader::segmentClickedHandler(const EventArgs& e)
 			setSortSegment(*seg);
 		}
 		// not a new segment, toggle current direction
-		else if (d_sortSegment != NULL)
+		else if (d_sortSegment)
 		{
 			ListHeaderSegment::SortDirection currDir = d_sortSegment->getSortDirection();
 
@@ -988,52 +1001,26 @@ bool ListHeader::segmentDragHandler(const EventArgs& e)
 	// what we do here is monitor the position and scroll if we can when mouse is outside area.
 
 	// get mouse position as something local
-	Point localMousePos(screenToWindow(MouseCursor::getSingleton().getPosition()));
-
-	MetricsMode mmode = getMetricsMode();
-
-	// get required figures as pixels
-	if (mmode == Relative)
-	{
-		localMousePos = relativeToAbsolute(localMousePos);
-	}
+	Point localMousePos(CoordConverter::screenToWindow(*this, MouseCursor::getSingleton().getPosition()));
 
 	// scroll left?
 	if (localMousePos.d_x < 0.0f)
 	{
 		if (d_segmentOffset > 0.0f)
 		{
-			float adjust = ScrollSpeed;
-			
-			if (mmode == Relative)
-			{
-				adjust = absoluteToRelativeX(adjust);
-			}
-			
-			setSegmentOffset(ceguimax(0.0f, d_segmentOffset - adjust));
+			setSegmentOffset(ceguimax(0.0f, d_segmentOffset - ScrollSpeed));
 		}
-
 	}
 	// scroll right?
-	else if (localMousePos.d_x >= getAbsoluteWidth())
+	else if (localMousePos.d_x >= d_pixelSize.d_width)
 	{
-		float adjust	= ScrollSpeed;
-		float pixOffset = d_segmentOffset;
-		float maxOffset = ceguimax(0.0f, getTotalSegmentsPixelExtent() - getAbsoluteWidth());
-
-		// convert values as required so calculations can be done in a consistent way
-		if (mmode == Relative)
-		{
-			maxOffset = absoluteToRelativeX(maxOffset);
-			pixOffset = relativeToAbsoluteX(pixOffset);
-			adjust = absoluteToRelativeX(adjust);
-		}
+		float maxOffset = ceguimax(0.0f, getTotalSegmentsPixelExtent() - d_pixelSize.d_width);
 
 		// if we have not scrolled to the limit
 		if (d_segmentOffset < maxOffset)
 		{
 			// scroll, but never beyond the limit
-			setSegmentOffset(ceguimin(maxOffset, d_segmentOffset + adjust));
+			setSegmentOffset(ceguimin(maxOffset, d_segmentOffset + ScrollSpeed));
 		}
 
 	}
@@ -1042,25 +1029,6 @@ bool ListHeader::segmentDragHandler(const EventArgs& e)
 }
 
 
-/*************************************************************************
-	Add ListHeader specific events	
-*************************************************************************/
-void ListHeader::addListHeaderEvents(void)
-{
-	addEvent(EventSortColumnChanged);
-	addEvent(EventSortDirectionChanged);
-	addEvent(EventSegmentSized);
-	addEvent(EventSegmentClicked);
-	addEvent(EventSplitterDoubleClicked);
-	addEvent(EventSegmentSequenceChanged);
-	addEvent(EventSegmentAdded);
-	addEvent(EventSegmentRemoved);
-	addEvent(EventSortSettingChanged);
-	addEvent(EventDragMoveSettingChanged);
-	addEvent(EventDragSizeSettingChanged);
-	addEvent(EventSegmentRenderOffsetChanged);
-}
-
 /*************************************************************************
 	Add ListHeader specific properties
 *************************************************************************/
@@ -1073,5 +1041,38 @@ void ListHeader::addHeaderProperties(void)
 	addProperty(&d_sortDirectionProperty);
 }
 
+/*************************************************************************
+    Create new segment
+*************************************************************************/
+ListHeaderSegment* ListHeader::createNewSegment(const String& name) const
+{
+    if (d_windowRenderer != 0)
+    {
+        ListHeaderWindowRenderer* wr = (ListHeaderWindowRenderer*)d_windowRenderer;
+        return wr->createNewSegment(name);
+    }
+    else
+    {
+        //return createNewSegment_impl(name);
+        throw InvalidRequestException("ListHeader::createNewSegment - This function must be implemented by the window renderer module");
+    }
+}
+
+/*************************************************************************
+    Destroy segment
+*************************************************************************/
+void ListHeader::destroyListSegment(ListHeaderSegment* segment) const
+{
+    if (d_windowRenderer != 0)
+    {
+        ListHeaderWindowRenderer* wr = (ListHeaderWindowRenderer*)d_windowRenderer;
+        return wr->destroyListSegment(segment);
+    }
+    else
+    {
+        //return destroyListSegment_impl(segment);
+        throw InvalidRequestException("ListHeader::destroyListSegment - This function must be implemented by the window renderer module");
+    }
+}
 
 } // End of  CEGUI namespace section
