@@ -1,41 +1,54 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIListbox.cpp
 	created:	13/4/2004
 	author:		Paul D Turner
 	
 	purpose:	Implementation of Listbox widget base class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "CEGUIExceptions.h"
 #include "CEGUIWindowManager.h"
 #include "elements/CEGUIListbox.h"
 #include "elements/CEGUIListboxItem.h"
 #include "elements/CEGUIScrollbar.h"
 #include "elements/CEGUITooltip.h"
-
+#include "CEGUICoordConverter.h"
 #include <algorithm>
 
 // Start of CEGUI namespace section
 namespace CEGUI
 {
 const String Listbox::EventNamespace("Listbox");
+const String Listbox::WidgetTypeName("CEGUI/Listbox");
+
+/*************************************************************************
+    ListboxWindowRenderer
+*************************************************************************/
+ListboxWindowRenderer::ListboxWindowRenderer(const String& name) :
+    WindowRenderer(name, Listbox::EventNamespace)
+{
+}
 
 /*************************************************************************
 	Definition of Properties for this class
@@ -51,14 +64,19 @@ ListboxProperties::ItemTooltips			Listbox::d_itemTooltipsProperty;
 	Constants
 *************************************************************************/
 // event names
-const String Listbox::EventListContentsChanged( (utf8*)"ListItemsChanged" );
-const String Listbox::EventSelectionChanged( (utf8*)"ItemSelectionChanged" );
-const String Listbox::EventSortModeChanged( (utf8*)"SortModeChanged" );
-const String Listbox::EventMultiselectModeChanged( (utf8*)"MuliselectModeChanged" );
-const String Listbox::EventVertScrollbarModeChanged( (utf8*)"VertScrollModeChanged" );
-const String Listbox::EventHorzScrollbarModeChanged( (utf8*)"HorzScrollModeChanged" );
+const String Listbox::EventListContentsChanged( "ListItemsChanged" );
+const String Listbox::EventSelectionChanged( "ItemSelectionChanged" );
+const String Listbox::EventSortModeChanged( "SortModeChanged" );
+const String Listbox::EventMultiselectModeChanged( "MuliselectModeChanged" );
+const String Listbox::EventVertScrollbarModeChanged( "VertScrollModeChanged" );
+const String Listbox::EventHorzScrollbarModeChanged( "HorzScrollModeChanged" );
+
+/*************************************************************************
+    Child Widget name suffix constants
+*************************************************************************/
+const String Listbox::VertScrollbarNameSuffix( "__auto_vscrollbar__" );
+const String Listbox::HorzScrollbarNameSuffix( "__auto_hscrollbar__" );
 
-	
 /*************************************************************************
 	Constructor for Listbox base class.
 *************************************************************************/
@@ -69,11 +87,8 @@ Listbox::Listbox(const String& type, const String& name)
 	d_forceVertScroll(false),
 	d_forceHorzScroll(false),
 	d_itemTooltips(false),
-	d_lastSelected(NULL)
+	d_lastSelected(0)
 {
-	// add new events specific to list box.
-	addListboxEvents();
-
 	addListboxProperties();
 }
 
@@ -90,17 +105,14 @@ Listbox::~Listbox(void)
 /*************************************************************************
 	Initialise the Window based object ready for use.
 *************************************************************************/
-void Listbox::initialise(void)
+void Listbox::initialiseComponents(void)
 {
-	// create the component sub-widgets
-	d_vertScrollbar = createVertScrollbar(getName() + "__auto_vscrollbar__");
-	d_horzScrollbar = createHorzScrollbar(getName() + "__auto_hscrollbar__");
+	// get the component sub-widgets
+	Scrollbar* vertScrollbar = getVertScrollbar();
+	Scrollbar* horzScrollbar = getHorzScrollbar();
 
-	addChildWindow(d_vertScrollbar);
-	addChildWindow(d_horzScrollbar);
-
-    d_vertScrollbar->subscribeEvent(Scrollbar::EventScrollPositionChanged, Event::Subscriber(&Listbox::handle_scrollChange, this));
-    d_horzScrollbar->subscribeEvent(Scrollbar::EventScrollPositionChanged, Event::Subscriber(&Listbox::handle_scrollChange, this));
+    vertScrollbar->subscribeEvent(Scrollbar::EventScrollPositionChanged, Event::Subscriber(&Listbox::handle_scrollChange, this));
+    horzScrollbar->subscribeEvent(Scrollbar::EventScrollPositionChanged, Event::Subscriber(&Listbox::handle_scrollChange, this));
 
 	configureScrollbars();
 	performChildWindowLayout();
@@ -132,7 +144,7 @@ size_t Listbox::getSelectedCount(void) const
 *************************************************************************/
 ListboxItem* Listbox::getFirstSelectedItem(void) const
 {
-	return getNextSelected(NULL);
+	return getNextSelected(0);
 }
 
 
@@ -142,7 +154,7 @@ ListboxItem* Listbox::getFirstSelectedItem(void) const
 ListboxItem* Listbox::getNextSelected(const ListboxItem* start_item) const
 {
 	// if start_item is NULL begin search at begining, else start at item after start_item
-	size_t index = (start_item == NULL) ? 0 : (getItemIndex(start_item) + 1);
+	size_t index = (start_item == 0) ? 0 : (getItemIndex(start_item) + 1);
 
 	while (index < d_listItems.size())
 	{
@@ -160,7 +172,7 @@ ListboxItem* Listbox::getNextSelected(const ListboxItem* start_item) const
 	}
 
 	// no more selected items.
-	return NULL;
+	return 0;
 }
 
 
@@ -175,7 +187,7 @@ ListboxItem* Listbox::getListboxItemFromIndex(size_t index) const
 	}
 	else
 	{
-		throw InvalidRequestException((utf8*)"Listbox::getListboxItemFromIndex - the specified index is out of range for this Listbox.");
+		throw InvalidRequestException("Listbox::getListboxItemFromIndex - the specified index is out of range for this Listbox.");
 	}
 }
 
@@ -193,7 +205,7 @@ size_t Listbox::getItemIndex(const ListboxItem* item) const
 	}
 	else
 	{
-		throw InvalidRequestException((utf8*)"Listbox::getItemIndex - the specified ListboxItem is not attached to this Listbox.");
+		throw InvalidRequestException("Listbox::getItemIndex - the specified ListboxItem is not attached to this Listbox.");
 	}
 
 }
@@ -210,7 +222,7 @@ bool Listbox::isItemSelected(size_t index) const
 	}
 	else
 	{
-		throw InvalidRequestException((utf8*)"Listbox::isItemSelected - the specified index is out of range for this Listbox.");
+		throw InvalidRequestException("Listbox::isItemSelected - the specified index is out of range for this Listbox.");
 	}
 
 }
@@ -222,7 +234,7 @@ bool Listbox::isItemSelected(size_t index) const
 ListboxItem* Listbox::findItemWithText(const String& text, const ListboxItem* start_item)
 {
 	// if start_item is NULL begin search at begining, else start at item after start_item
-	size_t index = (start_item == NULL) ? 0 : (getItemIndex(start_item) + 1);
+	size_t index = (!start_item) ? 0 : (getItemIndex(start_item) + 1);
 
 	while (index < d_listItems.size())
 	{
@@ -240,7 +252,7 @@ ListboxItem* Listbox::findItemWithText(const String& text, const ListboxItem* st
 	}
 
 	// no items matched.
-	return NULL;
+	return 0;
 }
 
 
@@ -273,7 +285,7 @@ void Listbox::resetList(void)
 *************************************************************************/
 void Listbox::addItem(ListboxItem* item)
 {
-	if (item != NULL)
+	if (item)
 	{
 		// establish ownership
 		item->setOwnerWindow(this);
@@ -308,7 +320,7 @@ void Listbox::insertItem(ListboxItem* item, const ListboxItem* position)
 	{
 		addItem(item);
 	}
-	else if (item != NULL)
+	else if (item)
 	{
 		// establish ownership
 		item->setOwnerWindow(this);
@@ -316,7 +328,7 @@ void Listbox::insertItem(ListboxItem* item, const ListboxItem* position)
 		// if position is NULL begin insert at begining, else insert after item 'position'
 		LBItemList::iterator ins_pos;
 
-		if (position == NULL)
+		if (!position)
 		{
 			ins_pos = d_listItems.begin();
 		}
@@ -327,7 +339,7 @@ void Listbox::insertItem(ListboxItem* item, const ListboxItem* position)
 			// throw if item 'position' is not in the list
 			if (ins_pos == d_listItems.end())
 			{
-				throw InvalidRequestException((utf8*)"Listbox::insertItem - the specified ListboxItem for parameter 'position' is not attached to this Listbox.");
+				throw InvalidRequestException("Listbox::insertItem - the specified ListboxItem for parameter 'position' is not attached to this Listbox.");
 			}
 
 		}
@@ -346,7 +358,7 @@ void Listbox::insertItem(ListboxItem* item, const ListboxItem* position)
 *************************************************************************/
 void Listbox::removeItem(const ListboxItem* item)
 {
-	if (item != NULL)
+	if (item)
 	{
 		LBItemList::iterator pos = std::find(d_listItems.begin(), d_listItems.end(), item);
 
@@ -354,7 +366,7 @@ void Listbox::removeItem(const ListboxItem* item)
 		if (pos != d_listItems.end())
 		{
 			// disown item
-			(*pos)->setOwnerWindow(NULL);
+			(*pos)->setOwnerWindow(0);
 
 			// remove item
 			d_listItems.erase(pos);
@@ -362,7 +374,7 @@ void Listbox::removeItem(const ListboxItem* item)
 			// if item was the last selected item, reset that to NULL
 			if (item == d_lastSelected)
 			{
-				d_lastSelected = NULL;
+				d_lastSelected = 0;
 			}
 
 			// if item is supposed to be deleted by us
@@ -504,7 +516,7 @@ void Listbox::setItemSelectState(ListboxItem* item, bool state)
 	}
 	else
 	{
-		throw InvalidRequestException((utf8*)"Listbox::setItemSelectState - the specified ListboxItem is not attached to this Listbox.");
+		throw InvalidRequestException("Listbox::setItemSelectState - the specified ListboxItem is not attached to this Listbox.");
 	}
 }
 
@@ -533,7 +545,7 @@ void Listbox::setItemSelectState(size_t item_index, bool state)
 	}
 	else
 	{
-		throw InvalidRequestException((utf8*)"Listbox::setItemSelectState - the value passed in the 'item_index' parameter is out of range for this Listbox.");
+		throw InvalidRequestException("Listbox::setItemSelectState - the value passed in the 'item_index' parameter is out of range for this Listbox.");
 	}
 
 }
@@ -550,84 +562,14 @@ void Listbox::handleUpdatedItemData(void)
 }
 
 
-/*************************************************************************
-	Perform the actual rendering for this Window.
-*************************************************************************/
-void Listbox::populateRenderCache()
-{
-    // get the derived class to render general stuff before we handle the items
-    cacheListboxBaseImagery();
-
-    //
-    // Render list items
-    //
-    Vector3	itemPos;
-    Size	itemSize;
-    Rect	itemClipper, itemRect;
-    float	widest = getWidestItemWidth();
-
-    // calculate position of area we have to render into
-    Rect itemsArea(getListRenderArea());
-
-    // set up some initial positional details for items
-    itemPos.d_x = itemsArea.d_left - d_horzScrollbar->getScrollPosition();
-    itemPos.d_y = itemsArea.d_top - d_vertScrollbar->getScrollPosition();
-    itemPos.d_z = System::getSingleton().getRenderer()->getZLayer(3) - System::getSingleton().getRenderer()->getCurrentZ();
-
-    float alpha = getEffectiveAlpha();
-
-    // loop through the items
-    size_t itemCount = getItemCount();
-
-    for (size_t i = 0; i < itemCount; ++i)
-    {
-        itemSize.d_height = d_listItems[i]->getPixelSize().d_height;
-
-        // allow item to have full width of box if this is wider than items
-        itemSize.d_width = ceguimax(itemsArea.getWidth(), widest);
-
-        // calculate destination area for this item.
-        itemRect.d_left	= itemPos.d_x;
-        itemRect.d_top	= itemPos.d_y;
-        itemRect.setSize(itemSize);
-        itemClipper = itemRect.getIntersection(itemsArea);
-
-        // skip this item if totally clipped
-        if (itemClipper.getWidth() == 0)
-        {
-            itemPos.d_y += itemSize.d_height;
-            continue;
-        }
-
-        // draw this item
-        d_listItems[i]->draw(d_renderCache, itemRect, itemPos.d_z, alpha, &itemClipper);
-
-        // update position ready for next item
-        itemPos.d_y += itemSize.d_height;
-    }
-
-}
-
-
 /*************************************************************************
 	display required integrated scroll bars according to current state
 	of the list box and update their values.
 *************************************************************************/
 void Listbox::configureScrollbars(void)
 {
-    Scrollbar* vertScrollbar;
-    Scrollbar* horzScrollbar;
-
-    try
-    {
-        vertScrollbar = static_cast<Scrollbar*>(WindowManager::getSingleton().getWindow(getName() + "__auto_vscrollbar__"));
-        horzScrollbar = static_cast<Scrollbar*>(WindowManager::getSingleton().getWindow(getName() + "__auto_hscrollbar__"));
-    }
-    catch (UnknownObjectException)
-    {
-        // no scrollbars?  Can't configure then!
-        return;
-    }
+    Scrollbar* vertScrollbar = getVertScrollbar();
+    Scrollbar* horzScrollbar = getHorzScrollbar();
 
 	float totalHeight	= getTotalItemsHeight();
 	float widestItem	= getWidestItemWidth();
@@ -804,7 +746,7 @@ ListboxItem* Listbox::getItemAtPoint(const Point& pt) const
 	// point must be within the rendering area of the Listbox.
 	if (renderArea.isPointInRect(pt))
 	{
-		float y = renderArea.d_top - d_vertScrollbar->getScrollPosition();
+		float y = renderArea.d_top - getVertScrollbar()->getScrollPosition();
 
 		// test if point is above first item
 		if (pt.d_y >= y)
@@ -822,21 +764,7 @@ ListboxItem* Listbox::getItemAtPoint(const Point& pt) const
 		}
 	}
 
-	return NULL;
-}
-
-
-/*************************************************************************
-	Add list box specific events
-*************************************************************************/
-void Listbox::addListboxEvents(void)
-{
-	addEvent(EventListContentsChanged);
-	addEvent(EventSelectionChanged);
-	addEvent(EventSortModeChanged);
-	addEvent(EventMultiselectModeChanged);
-	addEvent(EventVertScrollbarModeChanged);
-	addEvent(EventHorzScrollbarModeChanged);
+	return 0;
 }
 
 
@@ -935,21 +863,16 @@ void Listbox::onMouseButtonDown(MouseEventArgs& e)
 			modified = clearAllSelections_impl();
 		}
 
-		Point localPos(screenToWindow(e.position));
-
-		if (getMetricsMode() == Relative)
-		{
-			localPos = relativeToAbsolute(localPos);
-		}
+		Point localPos(CoordConverter::screenToWindow(*this, e.position));
 
 		ListboxItem* item = getItemAtPoint(localPos);
 
-		if (item != NULL)
+		if (item)
 		{
 			modified = true;
 
 			// select range or item, depending upon keys and last selected item
-			if (((e.sysKeys & Shift) && (d_lastSelected != NULL)) && d_multiselect)
+			if (((e.sysKeys & Shift) && (d_lastSelected != 0)) && d_multiselect)
 			{
 				selectRange(getItemIndex(item), getItemIndex(d_lastSelected));
 			}
@@ -959,7 +882,7 @@ void Listbox::onMouseButtonDown(MouseEventArgs& e)
 			}
 
 			// update last selected item
-			d_lastSelected = item->isSelected() ? item : NULL;
+			d_lastSelected = item->isSelected() ? item : 0;
 		}
 
 		// fire event if needed
@@ -983,13 +906,16 @@ void Listbox::onMouseWheel(MouseEventArgs& e)
 	// base class processing.
 	Window::onMouseWheel(e);
 
-	if (d_vertScrollbar->isVisible() && (d_vertScrollbar->getDocumentSize() > d_vertScrollbar->getPageSize()))
+    Scrollbar* vertScrollbar = getVertScrollbar();
+    Scrollbar* horzScrollbar = getHorzScrollbar();
+
+	if (vertScrollbar->isVisible() && (vertScrollbar->getDocumentSize() > vertScrollbar->getPageSize()))
 	{
-		d_vertScrollbar->setScrollPosition(d_vertScrollbar->getScrollPosition() + d_vertScrollbar->getStepSize() * -e.wheelChange);
+		vertScrollbar->setScrollPosition(vertScrollbar->getScrollPosition() + vertScrollbar->getStepSize() * -e.wheelChange);
 	}
-	else if (d_horzScrollbar->isVisible() && (d_horzScrollbar->getDocumentSize() > d_horzScrollbar->getPageSize()))
+	else if (horzScrollbar->isVisible() && (horzScrollbar->getDocumentSize() > horzScrollbar->getPageSize()))
 	{
-		d_horzScrollbar->setScrollPosition(d_horzScrollbar->getScrollPosition() + d_horzScrollbar->getStepSize() * -e.wheelChange);
+		horzScrollbar->setScrollPosition(horzScrollbar->getScrollPosition() + horzScrollbar->getStepSize() * -e.wheelChange);
 	}
 
 	e.handled = true;
@@ -1002,13 +928,13 @@ void Listbox::onMouseMove(MouseEventArgs& e)
 {
     if (d_itemTooltips)
     {
-        static ListboxItem* lastItem = NULL;
+        static ListboxItem* lastItem = 0;
 
-        Point posi = relativeToAbsolute(screenToWindow(e.position));
+        Point posi( CoordConverter::screenToWindow(*this, e.position) );
         ListboxItem* item = getItemAtPoint(posi);
         if (item != lastItem)
         {
-            if (item != NULL)
+            if (item)
             {
                 setTooltipText(item->getTooltipText());
             }
@@ -1041,10 +967,12 @@ void Listbox::onMouseMove(MouseEventArgs& e)
 *************************************************************************/
 void Listbox::ensureItemIsVisible(size_t item_index)
 {
+    Scrollbar* vertScrollbar = getVertScrollbar();
+
 	// handle simple "scroll to the bottom" case
 	if (item_index >= getItemCount())
 	{
-		d_vertScrollbar->setScrollPosition(d_vertScrollbar->getDocumentSize() - d_vertScrollbar->getPageSize());
+		vertScrollbar->setScrollPosition(vertScrollbar->getDocumentSize() - vertScrollbar->getPageSize());
 	}
 	else
 	{
@@ -1063,7 +991,7 @@ void Listbox::ensureItemIsVisible(size_t item_index)
 		bottom = top + d_listItems[i]->getPixelSize().d_height;
 
 		// account for current scrollbar value
-		float currPos = d_vertScrollbar->getScrollPosition();
+		float currPos = vertScrollbar->getScrollPosition();
 		top		-= currPos;
 		bottom	-= currPos;
 
@@ -1071,13 +999,13 @@ void Listbox::ensureItemIsVisible(size_t item_index)
 		if ((top < 0.0f) || ((bottom - top) > listHeight))
 		{
 			// scroll top of item to top of box.
-			d_vertScrollbar->setScrollPosition(currPos + top);
+			vertScrollbar->setScrollPosition(currPos + top);
 		}
 		// if bottom is below the view area
 		else if (bottom >= listHeight)
 		{
 			// position bottom of item at the bottom of the list
-			d_vertScrollbar->setScrollPosition(currPos + bottom - listHeight);		
+			vertScrollbar->setScrollPosition(currPos + bottom - listHeight);
 		}
 		
 		// Item is already fully visible - nothing more to do.
@@ -1153,7 +1081,7 @@ bool Listbox::resetList_impl(void)
 		// clear out the list.
 		d_listItems.clear();
 
-		d_lastSelected = NULL;
+		d_lastSelected = 0;
 
 		return true;
 	}
@@ -1170,6 +1098,42 @@ bool Listbox::handle_scrollChange(const EventArgs& args)
     return true;
 }
 
+/*************************************************************************
+    Return a pointer to the vertical scrollbar component widget for this
+    Listbox.
+*************************************************************************/
+Scrollbar* Listbox::getVertScrollbar() const
+{
+    return static_cast<Scrollbar*>(WindowManager::getSingleton().getWindow(
+                                   getName() + VertScrollbarNameSuffix));
+}
+
+/*************************************************************************
+    Return a pointer to the horizontal scrollbar component widget for this
+    Listbox.
+*************************************************************************/
+Scrollbar* Listbox::getHorzScrollbar() const
+{
+    return static_cast<Scrollbar*>(WindowManager::getSingleton().getWindow(
+                                   getName() + HorzScrollbarNameSuffix));
+}
+
+/*************************************************************************
+    Return a Rect object describing, in un-clipped pixels, the window
+    relative area that is to be used for rendering list items.
+*************************************************************************/
+Rect Listbox::getListRenderArea() const
+{
+    if (d_windowRenderer != 0)
+    {
+        ListboxWindowRenderer* wr = (ListboxWindowRenderer*)d_windowRenderer;
+        return wr->getListRenderArea();
+    }
+    else
+    {
+        throw InvalidRequestException("Listbox::getListRenderArea - This function must be implemented by the window renderer module");
+    }
+}
 
 //////////////////////////////////////////////////////////////////////////
 /*************************************************************************
