@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIMultiColumnListProperties.cpp
 	created:	11/7/2004
 	author:		Paul D Turner
 	
 	purpose:	Implements multi-column list properties.
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "elements/CEGUIMultiColumnListProperties.h"
 #include "elements/CEGUIMultiColumnList.h"
 #include "CEGUIPropertyHelper.h"
@@ -89,15 +93,15 @@ String	SortDirection::get(const PropertyReceiver* receiver) const
 	switch(static_cast<const MultiColumnList*>(receiver)->getSortDirection())
 	{
 	case ListHeaderSegment::Ascending:
-		return String((utf8*)"Ascending");
+		return String("Ascending");
 		break;
 
 	case ListHeaderSegment::Descending:
-		return String((utf8*)"Descending");
+		return String("Descending");
 		break;
 
 	default:
-		return String((utf8*)"None");
+		return String("None");
 		break;
 	}
 
@@ -108,11 +112,11 @@ void	SortDirection::set(PropertyReceiver* receiver, const String& value)
 {
 	ListHeaderSegment::SortDirection dir;
 
-	if (value == (utf8*)"Ascending")
+	if (value == "Ascending")
 	{
 		dir = ListHeaderSegment::Ascending;
 	}
-	else if (value == (utf8*)"Descending")
+	else if (value == "Descending")
 	{
 		dir = ListHeaderSegment::Descending;
 	}
@@ -178,43 +182,43 @@ String	SelectionMode::get(const PropertyReceiver* receiver) const
 	switch(static_cast<const MultiColumnList*>(receiver)->getSelectionMode())
 	{
 	case MultiColumnList::RowMultiple:
-		return String((utf8*)"RowMultiple");
+		return String("RowMultiple");
 		break;
 
 	case MultiColumnList::ColumnSingle:
-		return String((utf8*)"ColumnSingle");
+		return String("ColumnSingle");
 		break;
 
 	case MultiColumnList::ColumnMultiple:
-		return String((utf8*)"ColumnMultiple");
+		return String("ColumnMultiple");
 		break;
 
 	case MultiColumnList::CellSingle:
-		return String((utf8*)"CellSingle");
+		return String("CellSingle");
 		break;
 
 	case MultiColumnList::CellMultiple:
-		return String((utf8*)"CellMultiple");
+		return String("CellMultiple");
 		break;
 
 	case MultiColumnList::NominatedColumnSingle:
-		return String((utf8*)"NominatedColumnSingle");
+		return String("NominatedColumnSingle");
 		break;
 
 	case MultiColumnList::NominatedColumnMultiple:
-		return String((utf8*)"NominatedColumnMultiple");
+		return String("NominatedColumnMultiple");
 		break;
 
 	case MultiColumnList::NominatedRowSingle:
-		return String((utf8*)"NominatedRowSingle");
+		return String("NominatedRowSingle");
 		break;
 
 	case MultiColumnList::NominatedRowMultiple:
-		return String((utf8*)"NominatedRowMultiple");
+		return String("NominatedRowMultiple");
 		break;
 
 	default:
-		return String((utf8*)"RowSingle");
+		return String("RowSingle");
 		break;
 	}
 }
@@ -224,39 +228,39 @@ void	SelectionMode::set(PropertyReceiver* receiver, const String& value)
 {
 	MultiColumnList::SelectionMode mode;
 
-	if (value == (utf8*)"RowMultiple")
+	if (value == "RowMultiple")
 	{
 		mode = MultiColumnList::RowMultiple;
 	}
-	else if (value == (utf8*)"ColumnSingle")
+	else if (value == "ColumnSingle")
 	{
 		mode = MultiColumnList::ColumnSingle;
 	}
-	else if (value == (utf8*)"ColumnMultiple")
+	else if (value == "ColumnMultiple")
 	{
 		mode = MultiColumnList::ColumnMultiple;
 	}
-	else if (value == (utf8*)"CellSingle")
+	else if (value == "CellSingle")
 	{
 		mode = MultiColumnList::CellSingle;
 	}
-	else if (value == (utf8*)"CellMultiple")
+	else if (value == "CellMultiple")
 	{
 		mode = MultiColumnList::CellMultiple;
 	}
-	else if (value == (utf8*)"NominatedColumnSingle")
+	else if (value == "NominatedColumnSingle")
 	{
 		mode = MultiColumnList::NominatedColumnSingle;
 	}
-	else if (value == (utf8*)"NominatedColumnMultiple")
+	else if (value == "NominatedColumnMultiple")
 	{
 		mode = MultiColumnList::NominatedColumnMultiple;
 	}
-	else if (value == (utf8*)"NominatedRowSingle")
+	else if (value == "NominatedRowSingle")
 	{
 		mode = MultiColumnList::NominatedRowSingle;
 	}
-	else if (value == (utf8*)"NominatedRowMultiple")
+	else if (value == "NominatedRowMultiple")
 	{
 		mode = MultiColumnList::NominatedRowMultiple;
 	}
@@ -292,7 +296,7 @@ void ColumnHeader::set(PropertyReceiver* receiver, const String& value)
 	id = id.substr(id.find_first_of(":") + 1);
 
 	static_cast<MultiColumnList*>(receiver)->addColumn(
-		caption, PropertyHelper::stringToUint(id), PropertyHelper::stringToFloat(width));
+		caption, PropertyHelper::stringToUint(id), PropertyHelper::stringToUDim(width));
 }
 
 
