@@ -1,39 +1,54 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIMultiLineEditbox.cpp
 	created:	30/6/2004
 	author:		Paul D Turner
 	
 	purpose:	Implementation of the Multi-line edit box base class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "elements/CEGUIMultiLineEditbox.h"
 #include "elements/CEGUIScrollbar.h"
 #include "CEGUITextUtils.h"
 #include "CEGUIImage.h"
 #include "CEGUIExceptions.h"
-
+#include "CEGUICoordConverter.h"
+#include "CEGUIWindowManager.h"
 
 // Start of CEGUI namespace section
 namespace CEGUI
 {
 const String MultiLineEditbox::EventNamespace("MultiLineEditbox");
+const String MultiLineEditbox::WidgetTypeName("CEGUI/MultiLineEditbox");
+
+
+/*************************************************************************
+    MultiLineEditboxWindowRenderer
+*************************************************************************/
+MultiLineEditboxWindowRenderer::MultiLineEditboxWindowRenderer(const String& name) :
+    WindowRenderer(name, MultiLineEditbox::EventNamespace)
+{
+}
 
 /*************************************************************************
 	TODO:
@@ -50,34 +65,30 @@ MultiLineEditboxProperties::CaratIndex				MultiLineEditbox::d_caratIndexProperty
 MultiLineEditboxProperties::SelectionStart			MultiLineEditbox::d_selectionStartProperty;
 MultiLineEditboxProperties::SelectionLength			MultiLineEditbox::d_selectionLengthProperty;
 MultiLineEditboxProperties::MaxTextLength			MultiLineEditbox::d_maxTextLengthProperty;
-MultiLineEditboxProperties::NormalTextColour		MultiLineEditbox::d_normalTextColourProperty;
-MultiLineEditboxProperties::SelectedTextColour		MultiLineEditbox::d_selectedTextColourProperty;
-MultiLineEditboxProperties::ActiveSelectionColour	MultiLineEditbox::d_activeSelectionColourProperty;
-MultiLineEditboxProperties::InactiveSelectionColour	MultiLineEditbox::d_inactiveSelectionColourProperty;
+MultiLineEditboxProperties::SelectionBrushImage     MultiLineEditbox::d_selectionBrushProperty;
 
 
 /*************************************************************************
 	Constants
 *************************************************************************/
 // event names
-const String MultiLineEditbox::EventReadOnlyModeChanged( (utf8*)"ReadOnlyChanged" );
-const String MultiLineEditbox::EventWordWrapModeChanged( (utf8*)"WordWrapModeChanged" );
-const String MultiLineEditbox::EventMaximumTextLengthChanged( (utf8*)"MaximumTextLengthChanged" );
-const String MultiLineEditbox::EventCaratMoved( (utf8*)"CaratMoved" );
-const String MultiLineEditbox::EventTextSelectionChanged( (utf8*)"TextSelectionChanged" );
-const String MultiLineEditbox::EventEditboxFull( (utf8*)"EditboxFullEvent" );
-const String MultiLineEditbox::EventVertScrollbarModeChanged( (utf8*)"VertScrollbarModeChanged" );
-const String MultiLineEditbox::EventHorzScrollbarModeChanged( (utf8*)"HorzScrollbarModeChanged" );
-
-// default colours
-const argb_t MultiLineEditbox::DefaultNormalTextColour			= 0xFFFFFFFF;
-const argb_t MultiLineEditbox::DefaultSelectedTextColour		= 0xFF000000;
-const argb_t MultiLineEditbox::DefaultNormalSelectionColour		= 0xFF6060FF;
-const argb_t MultiLineEditbox::DefaultInactiveSelectionColour	= 0xFF808080;
+const String MultiLineEditbox::EventReadOnlyModeChanged( "ReadOnlyChanged" );
+const String MultiLineEditbox::EventWordWrapModeChanged( "WordWrapModeChanged" );
+const String MultiLineEditbox::EventMaximumTextLengthChanged( "MaximumTextLengthChanged" );
+const String MultiLineEditbox::EventCaratMoved( "CaratMoved" );
+const String MultiLineEditbox::EventTextSelectionChanged( "TextSelectionChanged" );
+const String MultiLineEditbox::EventEditboxFull( "EditboxFullEvent" );
+const String MultiLineEditbox::EventVertScrollbarModeChanged( "VertScrollbarModeChanged" );
+const String MultiLineEditbox::EventHorzScrollbarModeChanged( "HorzScrollbarModeChanged" );
 
 // Static data initialisation
-String MultiLineEditbox::d_lineBreakChars((utf8*)"\n");
+String MultiLineEditbox::d_lineBreakChars("\n");
 
+/*************************************************************************
+    Child Widget name suffix constants
+*************************************************************************/
+const String MultiLineEditbox::VertScrollbarNameSuffix( "__auto_vscrollbar__" );
+const String MultiLineEditbox::HorzScrollbarNameSuffix( "__auto_hscrollbar__" );
 
 /*************************************************************************
 	Constructor for the MultiLineEditbox base class.
@@ -95,15 +106,8 @@ MultiLineEditbox::MultiLineEditbox(const String& type, const String& name) :
 	d_widestExtent(0.0f),
 	d_forceVertScroll(false),
 	d_forceHorzScroll(false),
-	d_selectionBrush(NULL),
-	d_normalTextColour(DefaultNormalTextColour),
-	d_selectTextColour(DefaultSelectedTextColour),
-	d_selectBrushColour(DefaultNormalSelectionColour),
-	d_inactiveSelectBrushColour(DefaultInactiveSelectionColour)
+	d_selectionBrush(0)
 {
-	// add events specific to this widget.
-	addMultiLineEditboxEvents();
-
 	addMultiLineEditboxProperties();
 
 	// we always need a terminating \n
@@ -122,17 +126,14 @@ MultiLineEditbox::~MultiLineEditbox(void)
 /*************************************************************************
 	Initialise the Window based object ready for use.	
 *************************************************************************/
-void MultiLineEditbox::initialise(void)
+void MultiLineEditbox::initialiseComponents(void)
 {
 	// create the component sub-widgets
-	d_vertScrollbar = createVertScrollbar(getName() + "__auto_vscrollbar__");
-	d_horzScrollbar = createHorzScrollbar(getName() + "__auto_hscrollbar__");
+	Scrollbar* vertScrollbar = getVertScrollbar();
+	Scrollbar* horzScrollbar = getHorzScrollbar();
 
-	addChildWindow(d_vertScrollbar);
-	addChildWindow(d_horzScrollbar);
-
-    d_vertScrollbar->subscribeEvent(Scrollbar::EventScrollPositionChanged, Event::Subscriber(&MultiLineEditbox::handle_scrollChange, this));
-    d_horzScrollbar->subscribeEvent(Scrollbar::EventScrollPositionChanged, Event::Subscriber(&MultiLineEditbox::handle_scrollChange, this));
+    vertScrollbar->subscribeEvent(Scrollbar::EventScrollPositionChanged, Event::Subscriber(&MultiLineEditbox::handle_scrollChange, this));
+    horzScrollbar->subscribeEvent(Scrollbar::EventScrollPositionChanged, Event::Subscriber(&MultiLineEditbox::handle_scrollChange, this));
 
 	formatText();
 	performChildWindowLayout();
@@ -175,45 +176,6 @@ size_t MultiLineEditbox::getSelectionLength(void) const
 }
 
 
-/*************************************************************************
-	Add multi-line edit box specific events	
-*************************************************************************/
-void MultiLineEditbox::addMultiLineEditboxEvents(void)
-{
-	addEvent(EventReadOnlyModeChanged);
-	addEvent(EventWordWrapModeChanged);
-	addEvent(EventMaximumTextLengthChanged);
-	addEvent(EventCaratMoved);
-	addEvent(EventTextSelectionChanged);
-	addEvent(EventEditboxFull);
-	addEvent(EventVertScrollbarModeChanged);
-	addEvent(EventHorzScrollbarModeChanged);
-}
-
-
-/*************************************************************************
-	Perform the actual rendering for this Window.	
-*************************************************************************/
-void MultiLineEditbox::populateRenderCache()
-{
-	// get the derived class to render general stuff before we handle the text itself
-	cacheEditboxBaseImagery();
-
-	//
-	// Render edit box text
-	//
-	Rect textarea(getTextRenderArea());
-
-	cacheTextLines(textarea);
-
-	if (hasInputFocus() && !isReadOnly())
-	{
-		cacheCaratImagery(textarea);
-	}
-
-}
-
-
 /*************************************************************************
 	Specify whether the edit box is read-only.	
 *************************************************************************/
@@ -320,57 +282,16 @@ void MultiLineEditbox::setMaxTextLength(size_t max_len)
 }
 
 
-/*************************************************************************
-	Set the colour to be used for rendering edit box text in the normal,
-	unselected state.	
-*************************************************************************/
-void MultiLineEditbox::setNormalTextColour(const colour& col)
-{
-	d_normalTextColour = col;
-	requestRedraw();
-}
-
-
-/*************************************************************************
-	Set the colour to be used for rendering the edit box text when
-	within the selected region.	
-*************************************************************************/
-void MultiLineEditbox::setSelectedTextColour(const colour& col)
-{
-	d_selectTextColour = col;
-	requestRedraw();
-}
-
-
-/*************************************************************************
-	Set the colour to be used for rendering the edit box selection
-	highlight when the edit box is active.	
-*************************************************************************/
-void MultiLineEditbox::setNormalSelectBrushColour(const colour& col)
-{
-	d_selectBrushColour = col;
-	requestRedraw();
-}
-
-
-/*************************************************************************
-	Set the colour to be used for rendering the edit box selection
-	highlight when the edit box is inactive.	
-*************************************************************************/
-void MultiLineEditbox::setInactiveSelectBrushColour(const colour& col)
-{
-	d_inactiveSelectBrushColour = col;
-	requestRedraw();
-}
-
-
 /*************************************************************************
 	Scroll the view so that the current carat position is visible.
 *************************************************************************/
 void MultiLineEditbox::ensureCaratIsVisible(void)
 {
+    Scrollbar* vertScrollbar = getVertScrollbar();
+    Scrollbar* horzScrollbar = getHorzScrollbar();
+
 	// calculate the location of the carat
-	const Font* fnt = getFont();
+	Font* fnt = getFont();
 	size_t caratLine = getLineNumberFromIndex(d_caratPos);
 
 	if (caratLine < d_lines.size())
@@ -383,29 +304,29 @@ void MultiLineEditbox::ensureCaratIsVisible(void)
 		float xpos = fnt->getTextExtent(d_text.substr(d_lines[caratLine].d_startIdx, caratLineIdx));
 
 		// adjust position for scroll bars
-		xpos -= d_horzScrollbar->getScrollPosition();
-		ypos -= d_vertScrollbar->getScrollPosition();
+		xpos -= horzScrollbar->getScrollPosition();
+		ypos -= vertScrollbar->getScrollPosition();
 
 		// if carat is above window, scroll up
 		if (ypos < 0)
 		{
-			d_vertScrollbar->setScrollPosition(d_vertScrollbar->getScrollPosition() + ypos);
+			vertScrollbar->setScrollPosition(vertScrollbar->getScrollPosition() + ypos);
 		}
 		// if carat is below the window, scroll down
 		else if ((ypos += fnt->getLineSpacing()) > textArea.getHeight())
 		{
-			d_vertScrollbar->setScrollPosition(d_vertScrollbar->getScrollPosition() + (ypos - textArea.getHeight()) + fnt->getLineSpacing());
+			vertScrollbar->setScrollPosition(vertScrollbar->getScrollPosition() + (ypos - textArea.getHeight()) + fnt->getLineSpacing());
 		}
 
 		// if carat is left of the window, scroll left
 		if (xpos < 0)
 		{
-			d_horzScrollbar->setScrollPosition(d_horzScrollbar->getScrollPosition() + xpos - 50);
+			horzScrollbar->setScrollPosition(horzScrollbar->getScrollPosition() + xpos - 50);
 		}
 		// if carat is right of the window, scroll right
 		else if (xpos > textArea.getWidth())
 		{
-			d_horzScrollbar->setScrollPosition(d_horzScrollbar->getScrollPosition() + (xpos - textArea.getWidth()) + 50);
+			horzScrollbar->setScrollPosition(horzScrollbar->getScrollPosition() + (xpos - textArea.getWidth()) + 50);
 		}
 
 	}
@@ -436,6 +357,8 @@ void MultiLineEditbox::setWordWrapping(bool setting)
 *************************************************************************/
 void MultiLineEditbox::configureScrollbars(void)
 {
+    Scrollbar* vertScrollbar = getVertScrollbar();
+    Scrollbar* horzScrollbar = getHorzScrollbar();
 	float totalHeight	= (float)d_lines.size() * getFont()->getLineSpacing();
 	float widestItem	= d_widestExtent;
 
@@ -445,16 +368,16 @@ void MultiLineEditbox::configureScrollbars(void)
 	// show or hide vertical scroll bar as required (or as specified by option)
 	if ((totalHeight > getTextRenderArea().getHeight()) || d_forceVertScroll)
 	{
-		d_vertScrollbar->show();
+		vertScrollbar->show();
 
 		// show or hide horizontal scroll bar as required (or as specified by option)
 		if ((widestItem > getTextRenderArea().getWidth()) || d_forceHorzScroll)
 		{
-			d_horzScrollbar->show();
+			horzScrollbar->show();
 		}
 		else
 		{
-			d_horzScrollbar->hide();
+			horzScrollbar->hide();
 		}
 
 	}
@@ -463,23 +386,23 @@ void MultiLineEditbox::configureScrollbars(void)
 		// show or hide horizontal scroll bar as required (or as specified by option)
 		if ((widestItem > getTextRenderArea().getWidth()) || d_forceHorzScroll)
 		{
-			d_horzScrollbar->show();
+			horzScrollbar->show();
 
 			// show or hide vertical scroll bar as required (or as specified by option)
 			if ((totalHeight > getTextRenderArea().getHeight()) || d_forceVertScroll)
 			{
-				d_vertScrollbar->show();
+				vertScrollbar->show();
 			}
 			else
 			{
-				d_vertScrollbar->hide();
+				vertScrollbar->hide();
 			}
 
 		}
 		else
 		{
-			d_vertScrollbar->hide();
-			d_horzScrollbar->hide();
+			vertScrollbar->hide();
+			horzScrollbar->hide();
 		}
 
 	}
@@ -489,136 +412,15 @@ void MultiLineEditbox::configureScrollbars(void)
 	//
 	Rect renderArea(getTextRenderArea());
 
-	d_vertScrollbar->setDocumentSize(totalHeight);
-	d_vertScrollbar->setPageSize(renderArea.getHeight());
-	d_vertScrollbar->setStepSize(ceguimax(1.0f, renderArea.getHeight() / 10.0f));
-	d_vertScrollbar->setScrollPosition(d_vertScrollbar->getScrollPosition());
+	vertScrollbar->setDocumentSize(totalHeight);
+	vertScrollbar->setPageSize(renderArea.getHeight());
+	vertScrollbar->setStepSize(ceguimax(1.0f, renderArea.getHeight() / 10.0f));
+	vertScrollbar->setScrollPosition(vertScrollbar->getScrollPosition());
 
-	d_horzScrollbar->setDocumentSize(widestItem);
-	d_horzScrollbar->setPageSize(renderArea.getWidth());
-	d_horzScrollbar->setStepSize(ceguimax(1.0f, renderArea.getWidth() / 10.0f));
-	d_horzScrollbar->setScrollPosition(d_horzScrollbar->getScrollPosition());
-}
-
-
-/*************************************************************************
-	Render text lines.	
-*************************************************************************/
-void MultiLineEditbox::cacheTextLines(const Rect& dest_area)
-{
-    // text is already formatted, we just grab the lines and render them with the required alignment.
-    Rect drawArea(dest_area);
-    drawArea.offset(Point(-d_horzScrollbar->getScrollPosition(), -d_vertScrollbar->getScrollPosition()));
-
-    Renderer* renderer = System::getSingleton().getRenderer();
-    const Font* fnt = getFont();
-
-    if (fnt)
-    {
-        // get layers to use for rendering
-        float textZ = renderer->getZLayer(4) - renderer->getCurrentZ();
-        float selZ  = renderer->getZLayer(3) - renderer->getCurrentZ();
-
-        // calculate final colours to use.
-        ColourRect colours;
-        float alpha = getEffectiveAlpha();
-        colour normalTextCol = d_normalTextColour;
-        normalTextCol.setAlpha(normalTextCol.getAlpha() * alpha);
-        colour selectTextCol = d_selectTextColour;
-        selectTextCol.setAlpha(selectTextCol.getAlpha() * alpha);
-        colour selectBrushCol = hasInputFocus() ? d_selectBrushColour : d_inactiveSelectBrushColour;
-        selectBrushCol.setAlpha(selectBrushCol.getAlpha() * alpha);
-
-        // for each formatted line.
-        for (size_t i = 0; i < d_lines.size(); ++i)
-        {
-            Rect lineRect(drawArea);
-            const LineInfo& currLine = d_lines[i];
-            String lineText(d_text.substr(currLine.d_startIdx, currLine.d_length));
-
-            // if it is a simple 'no selection area' case
-            if ((currLine.d_startIdx >= d_selectionEnd) ||
-                ((currLine.d_startIdx + currLine.d_length) <= d_selectionStart) ||
-                (d_selectionBrush == NULL))
-            {
-                colours.setColours(normalTextCol);
-                // render the complete line.
-                d_renderCache.cacheText(lineText, fnt, LeftAligned, lineRect, textZ, colours, &dest_area);
-            }
-            // we have at least some selection highlighting to do
-            else
-            {
-                // Start of actual rendering section.
-                String sect;
-                size_t sectIdx = 0, sectLen;
-                float selStartOffset = 0.0f, selAreaWidth = 0.0f;
-
-                // render any text prior to selected region of line.
-                if (currLine.d_startIdx < d_selectionStart)
-                {
-                    // calculate length of text section
-                    sectLen = d_selectionStart - currLine.d_startIdx;
-
-                    // get text for this section
-                    sect = lineText.substr(sectIdx, sectLen);
-                    sectIdx += sectLen;
-
-                    // get the pixel offset to the beginning of the selection area highlight.
-                    selStartOffset = fnt->getTextExtent(sect);
-
-                    // draw this portion of the text
-                    colours.setColours(normalTextCol);
-                    d_renderCache.cacheText(sect, fnt, LeftAligned, lineRect, textZ, colours, &dest_area);
-
-                    // set position ready for next portion of text
-                    lineRect.d_left += selStartOffset;
-                }
-
-                // calculate the length of the selected section
-                sectLen = ceguimin(d_selectionEnd - currLine.d_startIdx, currLine.d_length) - sectIdx;
-
-                // get the text for this section
-                sect = lineText.substr(sectIdx, sectLen);
-                sectIdx += sectLen;
-
-                // get the extent to use as the width of the selection area highlight
-                selAreaWidth = fnt->getTextExtent(sect);
-
-                // draw the text for this section
-                colours.setColours(selectTextCol);
-                d_renderCache.cacheText(sect, fnt, LeftAligned, lineRect, textZ, colours, &dest_area);
-
-                // render any text beyond selected region of line
-                if (sectIdx < currLine.d_length)
-                {
-                    // update render position to the end of the selected area.
-                    lineRect.d_left += selAreaWidth;
-
-                    // calculate length of this section
-                    sectLen = currLine.d_length - sectIdx;
-
-                    // get the text for this section
-                    sect = lineText.substr(sectIdx, sectLen);
-
-                    // render the text for this section.
-                    colours.setColours(normalTextCol);
-                    d_renderCache.cacheText(sect, fnt, LeftAligned, lineRect, textZ, colours, &dest_area);
-                }
-
-                // calculate area for the selection brush on this line
-                lineRect.d_left = drawArea.d_left + selStartOffset;
-                lineRect.d_right = lineRect.d_left + selAreaWidth;
-                lineRect.d_bottom = lineRect.d_top + fnt->getLineSpacing();
-
-                // render the selection area brush for this line
-                colours.setColours(selectBrushCol);
-                d_renderCache.cacheImage(*d_selectionBrush, lineRect, selZ, colours, &dest_area);
-            }
-
-            // update master position for next line in paragraph.
-            drawArea.d_top += fnt->getLineSpacing();
-        }
-    }
+	horzScrollbar->setDocumentSize(widestItem);
+	horzScrollbar->setPageSize(renderArea.getWidth());
+	horzScrollbar->setStepSize(ceguimax(1.0f, renderArea.getWidth() / 10.0f));
+	horzScrollbar->setScrollPosition(horzScrollbar->getScrollPosition());
 }
 
 
@@ -633,9 +435,9 @@ void MultiLineEditbox::formatText(void)
 
 	String paraText;
 
-	const Font* fnt = getFont();
+	Font* fnt = getFont();
 
-	if (fnt != NULL)
+	if (fnt)
 	{
 		float areaWidth = getTextRenderArea().getWidth();
 
@@ -774,12 +576,7 @@ size_t MultiLineEditbox::getTextIndexFromPosition(const Point& pt) const
 	//
 	// calculate final window position to be checked
 	//
-	Point wndPt = screenToWindow(pt);
-
-	if (getMetricsMode() == Relative)
-	{
-		wndPt = relativeToAbsolute(wndPt);
-	}
+	Point wndPt = CoordConverter::screenToWindow(*this, pt);
 
 	Rect textArea(getTextRenderArea());
 
@@ -787,8 +584,8 @@ size_t MultiLineEditbox::getTextIndexFromPosition(const Point& pt) const
 	wndPt.d_y -= textArea.d_top;
 
 	// factor in scroll bar values
-	wndPt.d_x += d_horzScrollbar->getScrollPosition();
-	wndPt.d_y += d_vertScrollbar->getScrollPosition();
+	wndPt.d_x += getHorzScrollbar()->getScrollPosition();
+	wndPt.d_y += getVertScrollbar()->getScrollPosition();
 
 	size_t lineNumber = static_cast<size_t>(wndPt.d_y / getFont()->getLineSpacing());
 
@@ -844,7 +641,7 @@ size_t MultiLineEditbox::getLineNumberFromIndex(size_t index) const
 
 	}
 
-	throw InvalidRequestException((utf8*)"MultiLineEditbox::getLineNumberFromIndex - Unable to identify a line from the given, invalid, index.");
+	throw InvalidRequestException("MultiLineEditbox::getLineNumberFromIndex - Unable to identify a line from the given, invalid, index.");
 }
 
 
@@ -1215,6 +1012,56 @@ void MultiLineEditbox::handleNewLine(uint sysKeys)
 }
 
 
+/*************************************************************************
+    Processing to move caret one page up
+*************************************************************************/
+void MultiLineEditbox::handlePageUp(uint sysKeys)
+{
+    size_t caratLine = getLineNumberFromIndex(d_caratPos);
+    size_t nbLine = static_cast<size_t>(getTextRenderArea().getHeight() / getFont()->getLineSpacing());
+    size_t newline = 0;
+    if (nbLine < caratLine)
+    {
+        newline = caratLine - nbLine;
+    }
+    setCaratIndex(d_lines[newline].d_startIdx);
+    if (sysKeys & Shift)
+    {
+        setSelection(d_caratPos, d_selectionEnd);
+    }
+    else
+    {
+        clearSelection();
+    }
+    ensureCaratIsVisible();
+}
+
+
+/*************************************************************************
+    Processing to move caret one page down
+*************************************************************************/
+void MultiLineEditbox::handlePageDown(uint sysKeys)
+{
+    size_t caratLine = getLineNumberFromIndex(d_caratPos);
+    size_t nbLine =  static_cast<size_t>(getTextRenderArea().getHeight() / getFont()->getLineSpacing());
+    size_t newline = caratLine + nbLine;
+    if (d_lines.size() > 0)
+    {
+        newline = newline < d_lines.size() - 1 ? newline : d_lines.size() -1;
+    }
+    setCaratIndex(d_lines[newline].d_startIdx + d_lines[newline].d_length - 1);
+    if (sysKeys & Shift)
+    {
+        setSelection(d_selectionStart, d_caratPos);
+    }
+    else
+    {
+        clearSelection();
+    }
+    ensureCaratIsVisible();
+}
+
+
 /*************************************************************************
 	Handler for when a mouse button is pushed
 *************************************************************************/
@@ -1477,6 +1324,14 @@ void MultiLineEditbox::onKeyDown(KeyEventArgs& e)
 			}
 			break;
 
+        case Key::PageUp:
+            handlePageUp(e.sysKeys);
+            break;
+
+        case Key::PageDown:
+            handlePageDown(e.sysKeys);
+            break;
+
         // default case is now to leave event as (possibly) unhandled.
         default:
             return;
@@ -1539,13 +1394,16 @@ void MultiLineEditbox::onMouseWheel(MouseEventArgs& e)
 	// base class processing.
 	Window::onMouseWheel(e);
 
-	if (d_vertScrollbar->isVisible() && (d_vertScrollbar->getDocumentSize() > d_vertScrollbar->getPageSize()))
+    Scrollbar* vertScrollbar = getVertScrollbar();
+    Scrollbar* horzScrollbar = getHorzScrollbar();
+
+	if (vertScrollbar->isVisible() && (vertScrollbar->getDocumentSize() > vertScrollbar->getPageSize()))
 	{
-		d_vertScrollbar->setScrollPosition(d_vertScrollbar->getScrollPosition() + d_vertScrollbar->getStepSize() * -e.wheelChange);
+		vertScrollbar->setScrollPosition(vertScrollbar->getScrollPosition() + vertScrollbar->getStepSize() * -e.wheelChange);
 	}
-	else if (d_horzScrollbar->isVisible() && (d_horzScrollbar->getDocumentSize() > d_horzScrollbar->getPageSize()))
+	else if (horzScrollbar->isVisible() && (horzScrollbar->getDocumentSize() > horzScrollbar->getPageSize()))
 	{
-		d_horzScrollbar->setScrollPosition(d_horzScrollbar->getScrollPosition() + d_horzScrollbar->getStepSize() * -e.wheelChange);
+		horzScrollbar->setScrollPosition(horzScrollbar->getScrollPosition() + horzScrollbar->getStepSize() * -e.wheelChange);
 	}
 
 	e.handled = true;
@@ -1650,10 +1508,7 @@ void MultiLineEditbox::addMultiLineEditboxProperties(void)
 	addProperty(&d_selectionStartProperty);
 	addProperty(&d_selectionLengthProperty);
 	addProperty(&d_maxTextLengthProperty);
-	addProperty(&d_normalTextColourProperty);
-	addProperty(&d_selectedTextColourProperty);
-	addProperty(&d_activeSelectionColourProperty);
-	addProperty(&d_inactiveSelectionColourProperty);
+    addProperty(&d_selectionBrushProperty);
 }
 
 /*************************************************************************
@@ -1666,6 +1521,50 @@ bool MultiLineEditbox::handle_scrollChange(const EventArgs& args)
     return true;
 }
 
+/*************************************************************************
+    Return a pointer to the vertical scrollbar component widget.
+*************************************************************************/
+Scrollbar* MultiLineEditbox::getVertScrollbar() const
+{
+    return static_cast<Scrollbar*>(WindowManager::getSingleton().getWindow(
+                                   getName() + VertScrollbarNameSuffix));
+}
 
+/*************************************************************************
+    Return a pointer to the horizontal scrollbar component widget.
+*************************************************************************/
+Scrollbar* MultiLineEditbox::getHorzScrollbar() const
+{
+    return static_cast<Scrollbar*>(WindowManager::getSingleton().getWindow(
+                                   getName() + HorzScrollbarNameSuffix));
+}
+
+/*************************************************************************
+    Get the text rendering area
+*************************************************************************/
+Rect MultiLineEditbox::getTextRenderArea() const
+{
+    if (d_windowRenderer != 0)
+    {
+        MultiLineEditboxWindowRenderer* wr = (MultiLineEditboxWindowRenderer*)d_windowRenderer;
+        return wr->getTextRenderArea();
+    }
+    else
+    {
+        //return getTextRenderArea_impl();
+        throw InvalidRequestException("MultiLineEditbox::getTextRenderArea - This function must be implemented by the window renderer module");
+    }
+}
+
+const Image* MultiLineEditbox::getSelectionBrushImage() const
+{
+    return d_selectionBrush;
+}
+
+void MultiLineEditbox::setSelectionBrushImage(const Image* image)
+{
+    d_selectionBrush = image;
+    requestRedraw();
+}
 
 } // End of  CEGUI namespace section
