@@ -1,28 +1,32 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIMultiLineEditboxProperties.cpp
 	created:	10/7/2004
 	author:		Paul D Turner
 	
 	purpose:	Implements properties for the MultiLineEditbox class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "elements/CEGUIMultiLineEditboxProperties.h"
 #include "elements/CEGUIMultiLineEditbox.h"
 #include "CEGUIPropertyHelper.h"
@@ -113,53 +117,18 @@ void MaxTextLength::set(PropertyReceiver* receiver, const String& value)
 }
 
 
-String NormalTextColour::get(const PropertyReceiver* receiver) const
+String SelectionBrushImage::get(const PropertyReceiver* receiver) const
 {
-	return PropertyHelper::colourToString(static_cast<const MultiLineEditbox*>(receiver)->getNormalTextColour());
+    const Image* img = static_cast<const MultiLineEditbox*>(receiver)->getSelectionBrushImage();
+    return img ? PropertyHelper::imageToString(img) : String("");
 }
 
-
-void NormalTextColour::set(PropertyReceiver* receiver, const String& value)
+void SelectionBrushImage::set(PropertyReceiver* receiver, const String &value)
 {
-	static_cast<MultiLineEditbox*>(receiver)->setNormalTextColour(PropertyHelper::stringToColour(value));
+    static_cast<MultiLineEditbox*>(receiver)->setSelectionBrushImage(PropertyHelper::stringToImage(value));
 }
 
 
-String SelectedTextColour::get(const PropertyReceiver* receiver) const
-{
-	return PropertyHelper::colourToString(static_cast<const MultiLineEditbox*>(receiver)->getSelectedTextColour());
-}
-
-
-void SelectedTextColour::set(PropertyReceiver* receiver, const String& value)
-{
-	static_cast<MultiLineEditbox*>(receiver)->setSelectedTextColour(PropertyHelper::stringToColour(value));
-}
-
-
-String ActiveSelectionColour::get(const PropertyReceiver* receiver) const
-{
-	return PropertyHelper::colourToString(static_cast<const MultiLineEditbox*>(receiver)->getNormalSelectBrushColour());
-}
-
-
-void ActiveSelectionColour::set(PropertyReceiver* receiver, const String& value)
-{
-	static_cast<MultiLineEditbox*>(receiver)->setNormalSelectBrushColour(PropertyHelper::stringToColour(value));
-}
-
-
-String InactiveSelectionColour::get(const PropertyReceiver* receiver) const
-{
-	return PropertyHelper::colourToString(static_cast<const MultiLineEditbox*>(receiver)->getInactiveSelectBrushColour());
-}
-
-
-void InactiveSelectionColour::set(PropertyReceiver* receiver, const String& value)
-{
-	static_cast<MultiLineEditbox*>(receiver)->setInactiveSelectBrushColour(PropertyHelper::stringToColour(value));
-}
-
 } // End of  MultiLineEditboxProperties namespace section
 
 } // End of  CEGUI namespace section
