@@ -1,35 +1,42 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIScrollablePane.cpp
 	created:	1/3/2005
 	author:		Paul D Turner
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "elements/CEGUIScrollablePane.h"
 #include "elements/CEGUIScrolledContainer.h"
 #include "elements/CEGUIScrollbar.h"
 #include "CEGUIWindowManager.h"
+#include "CEGUIExceptions.h"
 #include <math.h>
 
 // Start of CEGUI namespace section
 namespace CEGUI
 {
+    const String ScrollablePane::WidgetTypeName("CEGUI/ScrollablePane");
+
     //////////////////////////////////////////////////////////////////////////
     // Event name constants
     const String ScrollablePane::EventNamespace("ScrollablePane");
@@ -38,6 +45,10 @@ namespace CEGUI
     const String ScrollablePane::EventHorzScrollbarModeChanged("HorzScrollbarModeChanged");
     const String ScrollablePane::EventAutoSizeSettingChanged("AutoSizeSettingChanged");
     const String ScrollablePane::EventContentPaneScrolled("ContentPaneScrolled");
+    // Child Widget name suffix constants
+    const String ScrollablePane::VertScrollbarNameSuffix( "__auto_vscrollbar__" );
+    const String ScrollablePane::HorzScrollbarNameSuffix( "__auto_hscrollbar__" );
+    const String ScrollablePane::ScrolledContainerNameSuffix( "__auto_container__" );
     // Property objects
     ScrollablePaneProperties::ForceHorzScrollbar   ScrollablePane::d_horzScrollbarProperty;
     ScrollablePaneProperties::ForceVertScrollbar   ScrollablePane::d_vertScrollbarProperty;
@@ -51,6 +62,15 @@ namespace CEGUI
     ScrollablePaneProperties::VertScrollPosition   ScrollablePane::d_vertScrollPositionProperty;
     //////////////////////////////////////////////////////////////////////////
 
+    /*************************************************************************
+        ScrollablePaneWindowRenderer
+    *************************************************************************/
+    ScrollablePaneWindowRenderer::ScrollablePaneWindowRenderer(const String& name) :
+        WindowRenderer(name, ScrollablePane::EventNamespace)
+    {
+    }
+
+    //////////////////////////////////////////////////////////////////////////
     ScrollablePane::ScrollablePane(const String& type, const String& name) :
         Window(type, name),
         d_forceVertScroll(false),
@@ -59,13 +79,16 @@ namespace CEGUI
         d_vertStep(0.1f),
         d_vertOverlap(0.01f),
         d_horzStep(0.1f),
-        d_horzOverlap(0.01f),
-        d_vertScrollbar(0),
-        d_horzScrollbar(0),
-        d_container(0)
+        d_horzOverlap(0.01f)
     {
-        addScrollablePaneEvents();
         addScrollablePaneProperties();
+		
+		// create scrolled container widget
+        ScrolledContainer* container =
+            static_cast<ScrolledContainer*>(WindowManager::getSingleton().createWindow(
+                ScrolledContainer::WidgetTypeName, d_name + ScrolledContainerNameSuffix));
+        // add scrolled container widget as child
+        addChildWindow(container);
     }
 
     ScrollablePane::~ScrollablePane(void)
@@ -74,8 +97,7 @@ namespace CEGUI
 
     const ScrolledContainer* ScrollablePane::getContentPane(void) const
     {
-        assert (d_container != 0);
-        return d_container;
+        return getScrolledContainer();
     }
 
     bool ScrollablePane::isVertScrollbarAlwaysShown(void) const
@@ -114,26 +136,22 @@ namespace CEGUI
 
     bool ScrollablePane::isContentPaneAutoSized(void) const
     {
-        assert(d_container != 0);
-        return d_container->isContentPaneAutoSized();
+        return getScrolledContainer()->isContentPaneAutoSized();
     }
 
     void ScrollablePane::setContentPaneAutoSized(bool setting)
     {
-        assert(d_container != 0);
-        d_container->setContentPaneAutoSized(setting);
+        getScrolledContainer()->setContentPaneAutoSized(setting);
     }
 
     const Rect& ScrollablePane::getContentPaneArea(void) const
     {
-        assert(d_container != 0);
-        return d_container->getContentArea();
+        return getScrolledContainer()->getContentArea();
     }
 
     void ScrollablePane::setContentPaneArea(const Rect& area)
     {
-        assert(d_container != 0);
-        d_container->setContentArea(area);
+        getScrolledContainer()->setContentArea(area);
     }
 
     float ScrollablePane::getHorizontalStepSize(void) const
@@ -160,15 +178,15 @@ namespace CEGUI
 
     float ScrollablePane::getHorizontalScrollPosition(void) const
     {
-        assert(d_horzScrollbar != 0);
-        float docSz = d_horzScrollbar->getDocumentSize();
-        return (docSz != 0) ? d_horzScrollbar->getScrollPosition() / docSz : 0.0f;
+        Scrollbar* horzScrollbar = getHorzScrollbar();
+        float docSz = horzScrollbar->getDocumentSize();
+        return (docSz != 0) ? horzScrollbar->getScrollPosition() / docSz : 0.0f;
     }
 
     void ScrollablePane::setHorizontalScrollPosition(float position)
     {
-        assert(d_horzScrollbar != 0);
-        d_horzScrollbar->setScrollPosition(d_horzScrollbar->getDocumentSize() * position);
+        Scrollbar* horzScrollbar = getHorzScrollbar();
+        horzScrollbar->setScrollPosition(horzScrollbar->getDocumentSize() * position);
     }
 
     float ScrollablePane::getVerticalStepSize(void) const
@@ -195,62 +213,46 @@ namespace CEGUI
 
     float ScrollablePane::getVerticalScrollPosition(void) const
     {
-        assert(d_vertScrollbar != 0);
-        float docSz = d_vertScrollbar->getDocumentSize();
-        return (docSz != 0) ? d_vertScrollbar->getScrollPosition() / docSz : 0.0f;
+        Scrollbar* vertScrollbar = getVertScrollbar();
+        float docSz = vertScrollbar->getDocumentSize();
+        return (docSz != 0) ? vertScrollbar->getScrollPosition() / docSz : 0.0f;
     }
 
     void ScrollablePane::setVerticalScrollPosition(float position)
     {
-        assert(d_vertScrollbar != 0);
-        d_vertScrollbar->setScrollPosition(d_vertScrollbar->getDocumentSize() * position);
+        Scrollbar* vertScrollbar = getVertScrollbar();
+        vertScrollbar->setScrollPosition(vertScrollbar->getDocumentSize() * position);
     }
 
-    void ScrollablePane::initialise(void)
+    void ScrollablePane::initialiseComponents(void)
     {
-        String widgetName;
-        // create horizontal scrollbar
-        widgetName = d_name + "__auto_hscrollbar__";
-        d_horzScrollbar = createHorizontalScrollbar(widgetName);
-        // perform consistency checks on what we got back
-        assert(d_horzScrollbar != 0);
-        assert(d_horzScrollbar->getName() == widgetName);
+        // get horizontal scrollbar
+        Scrollbar* horzScrollbar = getHorzScrollbar();
 
-        // create vertical scrollbar
-        widgetName = d_name + "__auto_vscrollbar__";
-        d_vertScrollbar = createVerticalScrollbar(widgetName);
-        // perform consistency checks on what we got back
-        assert(d_vertScrollbar != 0);
-        assert(d_vertScrollbar->getName() == widgetName);
+        // get vertical scrollbar
+        Scrollbar* vertScrollbar = getVertScrollbar();
 
-        // create scrolled container widget
-        d_container = 
-            static_cast<ScrolledContainer*>(WindowManager::getSingleton().createWindow(
-                ScrolledContainer::WidgetTypeName, d_name + "__auto_container__"));
-
-        // add child controls
-        addChildWindow(d_horzScrollbar);
-        addChildWindow(d_vertScrollbar);
-        addChildWindow(d_container);
+        // get scrolled container widget
+        ScrolledContainer* container = getScrolledContainer();
 
         // do a bit of initialisation
-        d_horzScrollbar->setAlwaysOnTop(true);
-        d_vertScrollbar->setAlwaysOnTop(true);
+        horzScrollbar->setAlwaysOnTop(true);
+        vertScrollbar->setAlwaysOnTop(true);
         // container pane is always same size as this parent pane,
         // scrolling is actually implemented via positioning and clipping tricks.
-        d_container->setSize(Relative, Size(1.0f, 1.0f));
+        container->setSize(UVector2(cegui_reldim(1.0f), cegui_reldim(1.0f)));
 
         // subscribe to events we need to hear about
-        d_vertScrollbar->subscribeEvent(
+        vertScrollbar->subscribeEvent(
             Scrollbar::EventScrollPositionChanged,
             Event::Subscriber(&ScrollablePane::handleScrollChange, this));
-        d_horzScrollbar->subscribeEvent(
+        horzScrollbar->subscribeEvent(
             Scrollbar::EventScrollPositionChanged,
             Event::Subscriber(&ScrollablePane::handleScrollChange, this));
-        d_container->subscribeEvent(
+        d_contentChangedConn = container->subscribeEvent(
             ScrolledContainer::EventContentChanged,
             Event::Subscriber(&ScrollablePane::handleContentAreaChange, this));
-        d_container->subscribeEvent(
+        d_autoSizeChangedConn = container->subscribeEvent(
             ScrolledContainer::EventAutoSizeSettingChanged,
             Event::Subscriber(&ScrollablePane::handleAutoSizePaneChanged, this));
 
@@ -258,31 +260,21 @@ namespace CEGUI
         configureScrollbars();
     }
 
-    void ScrollablePane::addScrollablePaneEvents(void)
-    {
-        addEvent(EventAutoSizeSettingChanged);
-        addEvent(EventContentPaneChanged);
-        addEvent(EventHorzScrollbarModeChanged);
-        addEvent(EventVertScrollbarModeChanged);
-        addEvent(EventContentPaneScrolled);
-    }
-
     void ScrollablePane::configureScrollbars(void)
     {
         // controls should all be valid by this stage
-        assert(d_container != 0);
-        assert(d_vertScrollbar != 0);
-        assert(d_horzScrollbar != 0);
+        Scrollbar* vertScrollbar = getVertScrollbar();
+        Scrollbar* horzScrollbar = getHorzScrollbar();
 
         // enable required scrollbars
-        d_vertScrollbar->setVisible(isVertScrollbarNeeded());
-        d_horzScrollbar->setVisible(isHorzScrollbarNeeded());
+        vertScrollbar->setVisible(isVertScrollbarNeeded());
+        horzScrollbar->setVisible(isHorzScrollbarNeeded());
 
         // Check if the addition of the horizontal scrollbar means we
         // now also need the vertical bar.
-        if (d_horzScrollbar->isVisible())
+        if (horzScrollbar->isVisible())
         {
-            d_vertScrollbar->setVisible(isVertScrollbarNeeded());
+            vertScrollbar->setVisible(isVertScrollbarNeeded());
         }
 
         performChildWindowLayout();
@@ -291,51 +283,45 @@ namespace CEGUI
         Rect viewableArea(getViewableArea());
 
         // set up vertical scroll bar values
-        d_vertScrollbar->setDocumentSize(fabsf(d_contentRect.getHeight()));
-        d_vertScrollbar->setPageSize(viewableArea.getHeight());
-        d_vertScrollbar->setStepSize(ceguimax(1.0f, viewableArea.getHeight() * d_vertStep));
-        d_vertScrollbar->setOverlapSize(ceguimax(1.0f, viewableArea.getHeight() * d_vertOverlap));
-        d_vertScrollbar->setScrollPosition(d_vertScrollbar->getScrollPosition());
+        vertScrollbar->setDocumentSize(fabsf(d_contentRect.getHeight()));
+        vertScrollbar->setPageSize(viewableArea.getHeight());
+        vertScrollbar->setStepSize(ceguimax(1.0f, viewableArea.getHeight() * d_vertStep));
+        vertScrollbar->setOverlapSize(ceguimax(1.0f, viewableArea.getHeight() * d_vertOverlap));
+        vertScrollbar->setScrollPosition(vertScrollbar->getScrollPosition());
 
         // set up horizontal scroll bar values
-        d_horzScrollbar->setDocumentSize(fabsf(d_contentRect.getWidth()));
-        d_horzScrollbar->setPageSize(viewableArea.getWidth());
-        d_horzScrollbar->setStepSize(ceguimax(1.0f, viewableArea.getWidth() * d_horzStep));
-        d_horzScrollbar->setOverlapSize(ceguimax(1.0f, viewableArea.getWidth() * d_horzOverlap));
-        d_horzScrollbar->setScrollPosition(d_horzScrollbar->getScrollPosition());
+        horzScrollbar->setDocumentSize(fabsf(d_contentRect.getWidth()));
+        horzScrollbar->setPageSize(viewableArea.getWidth());
+        horzScrollbar->setStepSize(ceguimax(1.0f, viewableArea.getWidth() * d_horzStep));
+        horzScrollbar->setOverlapSize(ceguimax(1.0f, viewableArea.getWidth() * d_horzOverlap));
+        horzScrollbar->setScrollPosition(horzScrollbar->getScrollPosition());
     }
 
     bool ScrollablePane::isHorzScrollbarNeeded(void) const
     {
-        assert(d_container != 0);
-
         return ((fabs(d_contentRect.getWidth()) > getViewableArea().getWidth()) || d_forceHorzScroll);
     }
 
     bool ScrollablePane::isVertScrollbarNeeded(void) const
     {
-        assert(d_container != 0);
-
         return ((fabs(d_contentRect.getHeight()) > getViewableArea().getHeight()) || d_forceVertScroll);
     }
 
     void ScrollablePane::updateContainerPosition(void)
     {
-        assert(d_container != 0);
-        assert(d_horzScrollbar != 0);
-        assert(d_vertScrollbar != 0);
-
         // basePos is the position represented by the scrollbars
         // (these are negated so pane is scrolled in the correct directions)
-        Point basePos(-d_horzScrollbar->getScrollPosition(), -d_vertScrollbar->getScrollPosition());
+        UVector2 basePos(cegui_absdim(-getHorzScrollbar()->getScrollPosition()),
+                         cegui_absdim(-getVertScrollbar()->getScrollPosition()));
 
         // this bias is the absolute position that 0 on the scrollbars represent.
         // effectively removes un-used empty space from the pane.
-        Point bias(d_contentRect.d_left, d_contentRect.d_top);
+        UVector2 bias(cegui_absdim(d_contentRect.d_left),
+                      cegui_absdim(d_contentRect.d_top));
 
         // set the new container pane position to be what the scrollbars request
         // minus any bias generated by the location of the content.
-        d_container->setPosition(Absolute, basePos - bias);
+        getScrolledContainer()->setPosition(basePos - bias);
     }
 
     void ScrollablePane::onContentPaneChanged(WindowEventArgs& e)
@@ -373,12 +359,11 @@ namespace CEGUI
 
     bool ScrollablePane::handleContentAreaChange(const EventArgs& e)
     {
-        assert(d_container != 0);
-        assert(d_horzScrollbar != 0);
-        assert(d_vertScrollbar != 0);
+        Scrollbar* vertScrollbar = getVertScrollbar();
+        Scrollbar* horzScrollbar = getHorzScrollbar();
 
         // get updated extents of the content
-        Rect contentArea(d_container->getContentArea());
+        Rect contentArea(getScrolledContainer()->getContentArea());
 
         // calculate any change on the top and left edges.
         float xChange = contentArea.d_left - d_contentRect.d_left;
@@ -390,8 +375,8 @@ namespace CEGUI
         configureScrollbars();
 
         // update scrollbar positions (which causes container pane to be moved as needed).
-        d_horzScrollbar->setScrollPosition(d_horzScrollbar->getScrollPosition() - xChange);
-        d_vertScrollbar->setScrollPosition(d_vertScrollbar->getScrollPosition() - yChange);
+        horzScrollbar->setScrollPosition(horzScrollbar->getScrollPosition() - xChange);
+        vertScrollbar->setScrollPosition(vertScrollbar->getScrollPosition() - yChange);
 
         // this call may already have been made if the scroll positions changed.  The call
         // is required here for cases where the top/left 'bias' has changed; in which
@@ -421,8 +406,8 @@ namespace CEGUI
         // null is not a valid window pointer!
         assert(wnd != 0);
 
-        // See if this is an internally generated window (will have "__auto_" in the name)
-        if (wnd->getName().find("__auto_") != String::npos)
+        // See if this is an internally generated window (will have AutoWidgetNameSuffix in the name)
+        if (wnd->getName().find(AutoWidgetNameSuffix) != String::npos)
         {
             // This is an internal widget, so should be added normally.
             Window::addChild_impl(wnd);
@@ -431,8 +416,7 @@ namespace CEGUI
         else
         {
             // container should always be valid by the time we're adding client controls
-            assert(d_container != 0);
-            d_container->addChildWindow(wnd);
+            getScrolledContainer()->addChildWindow(wnd);
         }
     }
 
@@ -441,8 +425,8 @@ namespace CEGUI
         // null is not a valid window pointer!
         assert(wnd != 0);
 
-        // See if this is an internally generated window (will have "__auto_" in the name)
-        if (wnd->getName().find("__auto_") != String::npos)
+        // See if this is an internally generated window (will have AutoWidgetNameSuffix in the name)
+        if (wnd->getName().find(AutoWidgetNameSuffix) != String::npos)
         {
             // This is an internal widget, so should be removed normally.
             Window::removeChild_impl(wnd);
@@ -451,8 +435,7 @@ namespace CEGUI
         else
         {
             // container should always be valid by the time we're handling client controls
-            assert(d_container != 0);
-            d_container->removeChildWindow(wnd);
+            getScrolledContainer()->removeChildWindow(wnd);
         }
     }
 
@@ -470,13 +453,16 @@ namespace CEGUI
         // base class processing.
         Window::onMouseWheel(e);
 
-        if (d_vertScrollbar->isVisible() && (d_vertScrollbar->getDocumentSize() > d_vertScrollbar->getPageSize()))
+        Scrollbar* vertScrollbar = getVertScrollbar();
+        Scrollbar* horzScrollbar = getHorzScrollbar();
+
+        if (vertScrollbar->isVisible() && (vertScrollbar->getDocumentSize() > vertScrollbar->getPageSize()))
         {
-            d_vertScrollbar->setScrollPosition(d_vertScrollbar->getScrollPosition() + d_vertScrollbar->getStepSize() * -e.wheelChange);
+            vertScrollbar->setScrollPosition(vertScrollbar->getScrollPosition() + vertScrollbar->getStepSize() * -e.wheelChange);
         }
-        else if (d_horzScrollbar->isVisible() && (d_horzScrollbar->getDocumentSize() > d_horzScrollbar->getPageSize()))
+        else if (horzScrollbar->isVisible() && (horzScrollbar->getDocumentSize() > horzScrollbar->getPageSize()))
         {
-            d_horzScrollbar->setScrollPosition(d_horzScrollbar->getScrollPosition() + d_horzScrollbar->getStepSize() * -e.wheelChange);
+            horzScrollbar->setScrollPosition(horzScrollbar->getScrollPosition() + horzScrollbar->getStepSize() * -e.wheelChange);
         }
 
         e.handled = true;
@@ -496,4 +482,47 @@ namespace CEGUI
         addProperty(&d_vertScrollPositionProperty);
     }
 
+    Scrollbar* ScrollablePane::getVertScrollbar() const
+    {
+        return static_cast<Scrollbar*>(WindowManager::getSingleton().getWindow(
+                                    getName() + VertScrollbarNameSuffix));
+    }
+
+    Scrollbar* ScrollablePane::getHorzScrollbar() const
+    {
+        return static_cast<Scrollbar*>(WindowManager::getSingleton().getWindow(
+                                    getName() + HorzScrollbarNameSuffix));
+    }
+
+    ScrolledContainer* ScrollablePane::getScrolledContainer() const
+    {
+        return static_cast<ScrolledContainer*>(
+                    WindowManager::getSingleton().getWindow(
+                    getName() + ScrolledContainerNameSuffix));
+    }
+
+    Rect ScrollablePane::getViewableArea() const
+    {
+        if (d_windowRenderer != 0)
+        {
+            ScrollablePaneWindowRenderer* wr = (ScrollablePaneWindowRenderer*)d_windowRenderer;
+            return wr->getViewableArea();
+        }
+        else
+        {
+            //return getViewableArea_impl();
+            throw InvalidRequestException("ScrollablePane::getViewableArea - This function must be implemented by the window renderer module");
+        }
+    }
+
+    void ScrollablePane::destroy(void)
+    {
+        // detach from events on content pane
+        d_contentChangedConn->disconnect();
+        d_autoSizeChangedConn->disconnect();
+
+        // now do the cleanup
+        Window::destroy();
+    }
+
 } // End of  CEGUI namespace section
