@@ -1,36 +1,50 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUIScrollbar.cpp
 	created:	13/4/2004
 	author:		Paul D Turner
 	
 	purpose:	Implementation of Scrollbar widget base class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "elements/CEGUIScrollbar.h"
 #include "elements/CEGUIThumb.h"
-
+#include "CEGUIWindowManager.h"
+#include "CEGUIExceptions.h"
 
 // Start of CEGUI namespace section
 namespace CEGUI
 {
 const String Scrollbar::EventNamespace("Scrollbar");
+const String Scrollbar::WidgetTypeName("CEGUI/Scrollbar");
+
+/*************************************************************************
+    ScrollbarWindowRenderer
+*************************************************************************/
+ScrollbarWindowRenderer::ScrollbarWindowRenderer(const String& name) :
+    WindowRenderer(name, Scrollbar::EventNamespace)
+{
+}
 
 /*************************************************************************
 	Definition of Properties for this class
@@ -45,10 +59,17 @@ ScrollbarProperties::ScrollPosition	Scrollbar::d_scrollPositionProperty;
 /*************************************************************************
 	Event name constants
 *************************************************************************/
-const String Scrollbar::EventScrollPositionChanged( (utf8*)"ScrollPosChanged" );
-const String Scrollbar::EventThumbTrackStarted( (utf8*)"ThumbTrackStarted" );
-const String Scrollbar::EventThumbTrackEnded( (utf8*)"ThumbTrackEnded" );
-const String Scrollbar::EventScrollConfigChanged( (utf8*)"ScrollConfigChanged" );
+const String Scrollbar::EventScrollPositionChanged( "ScrollPosChanged" );
+const String Scrollbar::EventThumbTrackStarted( "ThumbTrackStarted" );
+const String Scrollbar::EventThumbTrackEnded( "ThumbTrackEnded" );
+const String Scrollbar::EventScrollConfigChanged( "ScrollConfigChanged" );
+
+/*************************************************************************
+    Child Widget name suffix constants
+*************************************************************************/
+const String Scrollbar::ThumbNameSuffix( "__auto_thumb__" );
+const String Scrollbar::IncreaseButtonNameSuffix( "__auto_incbtn__" );
+const String Scrollbar::DecreaseButtonNameSuffix( "__auto_decbtn__" );
 
 
 /*************************************************************************
@@ -62,7 +83,6 @@ Scrollbar::Scrollbar(const String& type, const String& name) :
 	d_overlapSize(0.0f),
 	d_position(0.0f)
 {
-	addScrollbarEvents();
 	addScrollbarProperties();
 }
 
@@ -78,24 +98,21 @@ Scrollbar::~Scrollbar(void)
 /*************************************************************************
 	Initialises the Scrollbar object ready for use.
 *************************************************************************/
-void Scrollbar::initialise(void)
+void Scrollbar::initialiseComponents(void)
 {
 	// Set up thumb
-	d_thumb = createThumb(getName() + "__auto_thumb__");
-	addChildWindow(d_thumb);
-	d_thumb->subscribeEvent(Thumb::EventThumbPositionChanged, Event::Subscriber(&CEGUI::Scrollbar::handleThumbMoved, this));
-	d_thumb->subscribeEvent(Thumb::EventThumbTrackStarted, Event::Subscriber(&CEGUI::Scrollbar::handleThumbTrackStarted, this));
-	d_thumb->subscribeEvent(Thumb::EventThumbTrackEnded, Event::Subscriber(&CEGUI::Scrollbar::handleThumbTrackEnded, this));
+	Thumb* thumb = getThumb();
+	thumb->subscribeEvent(Thumb::EventThumbPositionChanged, Event::Subscriber(&CEGUI::Scrollbar::handleThumbMoved, this));
+	thumb->subscribeEvent(Thumb::EventThumbTrackStarted, Event::Subscriber(&CEGUI::Scrollbar::handleThumbTrackStarted, this));
+	thumb->subscribeEvent(Thumb::EventThumbTrackEnded, Event::Subscriber(&CEGUI::Scrollbar::handleThumbTrackEnded, this));
 
 	// set up Increase button
-	d_increase = createIncreaseButton(getName() + "__auto_incbtn__");
-	addChildWindow(d_increase);
-	d_increase->subscribeEvent(PushButton::EventMouseButtonDown, Event::Subscriber(&CEGUI::Scrollbar::handleIncreaseClicked, this));
+	PushButton* increase = getIncreaseButton();
+	increase->subscribeEvent(PushButton::EventMouseButtonDown, Event::Subscriber(&CEGUI::Scrollbar::handleIncreaseClicked, this));
 
 	// set up Decrease button
-	d_decrease = createDecreaseButton(getName() + "__auto_decbtn__");
-	addChildWindow(d_decrease);
-	d_decrease->subscribeEvent(PushButton::EventMouseButtonDown, Event::Subscriber(&CEGUI::Scrollbar::handleDecreaseClicked, this));
+	PushButton* decrease = getDecreaseButton();
+	decrease->subscribeEvent(PushButton::EventMouseButtonDown, Event::Subscriber(&CEGUI::Scrollbar::handleDecreaseClicked, this));
 
 	// do initial layout
 	performChildWindowLayout();
@@ -193,18 +210,6 @@ void Scrollbar::setScrollPosition(float position)
 }
 
 
-/*************************************************************************
-	Add scroll bar specific events
-*************************************************************************/
-void Scrollbar::addScrollbarEvents(void)
-{
-	addEvent(EventScrollPositionChanged);
-	addEvent(EventThumbTrackStarted);
-	addEvent(EventThumbTrackEnded);
-	addEvent(EventScrollConfigChanged);
-}
-
-
 /*************************************************************************
 	Handler triggered when the scroll position changes
 *************************************************************************/
@@ -237,6 +242,7 @@ void Scrollbar::onThumbTrackEnded(WindowEventArgs& e)
 *************************************************************************/
 void Scrollbar::onScrollConfigChanged(WindowEventArgs& e)
 {
+	performChildWindowLayout();
 	fireEvent(EventScrollConfigChanged, e, EventNamespace);
 }
 
@@ -369,8 +375,105 @@ void Scrollbar::addScrollbarProperties(void)
 	addProperty(&d_stepSizeProperty);
 	addProperty(&d_overlapSizeProperty);
 	addProperty(&d_scrollPositionProperty);
+
+    // we ban all these properties from xml for auto windows
+    if (isAutoWindow())
+    {
+        banPropertyFromXML(&d_documentSizeProperty);
+        banPropertyFromXML(&d_pageSizeProperty);
+        banPropertyFromXML(&d_stepSizeProperty);
+        banPropertyFromXML(&d_overlapSizeProperty);
+        banPropertyFromXML(&d_scrollPositionProperty);
+
+        // scrollbars tend to have their visibility toggled alot, so we ban that as well
+        banPropertyFromXML(&d_visibleProperty);
+    }
 }
 
 
+/*************************************************************************
+    Return a pointer to the 'increase' PushButtoncomponent widget for this
+    Scrollbar.
+*************************************************************************/
+PushButton* Scrollbar::getIncreaseButton() const
+{
+    return static_cast<PushButton*>(WindowManager::getSingleton().getWindow(
+                                    getName() + IncreaseButtonNameSuffix));
+}
+
+
+/*************************************************************************
+    Return a pointer to the 'decrease' PushButton component widget for this
+    Scrollbar.
+*************************************************************************/
+PushButton* Scrollbar::getDecreaseButton() const
+{
+    return static_cast<PushButton*>(WindowManager::getSingleton().getWindow(
+                                    getName() + DecreaseButtonNameSuffix));
+}
+
+
+/*************************************************************************
+    Return a pointer to the Thumb component widget for this Scrollbar.
+*************************************************************************/
+Thumb* Scrollbar::getThumb() const
+{
+    return static_cast<Thumb*>(WindowManager::getSingleton().getWindow(
+                               getName() + ThumbNameSuffix));
+}
+
+/*************************************************************************
+    update the size and location of the thumb to properly represent the
+    current state of the scroll bar
+*************************************************************************/
+void Scrollbar::updateThumb(void)
+{
+    if (d_windowRenderer != 0)
+    {
+        ScrollbarWindowRenderer* wr = (ScrollbarWindowRenderer*)d_windowRenderer;
+        wr->updateThumb();
+    }
+    else
+    {
+        //updateThumb_impl();
+        throw InvalidRequestException("Scrollbar::updateThumb - This function must be implemented by the window renderer module");
+    }
+}
+
+/*************************************************************************
+    return value that best represents current scroll bar position given
+    the current location of the thumb.
+*************************************************************************/
+float Scrollbar::getValueFromThumb(void) const
+{
+    if (d_windowRenderer != 0)
+    {
+        ScrollbarWindowRenderer* wr = (ScrollbarWindowRenderer*)d_windowRenderer;
+        return wr->getValueFromThumb();
+    }
+    else
+    {
+        //return getValueFromThumb_impl();
+        throw InvalidRequestException("Scrollbar::getValueFromThumb - This function must be implemented by the window renderer module");
+    }
+}
+
+/*************************************************************************
+    Given window location 'pt', return a value indicating what change
+    should be made to the scroll bar.
+*************************************************************************/
+float Scrollbar::getAdjustDirectionFromPoint(const Point& pt) const
+{
+    if (d_windowRenderer != 0)
+    {
+        ScrollbarWindowRenderer* wr = (ScrollbarWindowRenderer*)d_windowRenderer;
+        return wr->getAdjustDirectionFromPoint(pt);
+    }
+    else
+    {
+        //return getAdjustDirectionFromPoint_impl(pt);
+        throw InvalidRequestException("Scrollbar::getAdjustDirectionFromPoint - This function must be implemented by the window renderer module");
+    }
+}
 
 } // End of  CEGUI namespace section
