@@ -1,35 +1,50 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUISlider.cpp
 	created:	13/4/2004
 	author:		Paul D Turner
 	
 	purpose:	Implementation of Slider widget base class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "elements/CEGUISlider.h"
 #include "elements/CEGUIThumb.h"
+#include "CEGUIWindowManager.h"
+#include "CEGUIExceptions.h"
 
 // Start of CEGUI namespace section
 namespace CEGUI
 {
 const String Slider::EventNamespace("Slider");
+const String Slider::WidgetTypeName("CEGUI/Slider");
+
+/*************************************************************************
+    SliderWindowRenderer
+*************************************************************************/
+SliderWindowRenderer::SliderWindowRenderer(const String& name) :
+    WindowRenderer(name, Slider::EventNamespace)
+{
+}
 
 /*************************************************************************
 	Definition of Properties for this class
@@ -42,10 +57,14 @@ SliderProperties::ClickStepSize	Slider::d_clickStepSizeProperty;
 /*************************************************************************
 	Event name constants
 *************************************************************************/
-const String Slider::EventValueChanged( (utf8*)"ValueChanged" );
-const String Slider::EventThumbTrackStarted( (utf8*)"ThumbTrackStarted" );
-const String Slider::EventThumbTrackEnded( (utf8*)"ThumbTrackEnded" );
+const String Slider::EventValueChanged( "ValueChanged" );
+const String Slider::EventThumbTrackStarted( "ThumbTrackStarted" );
+const String Slider::EventThumbTrackEnded( "ThumbTrackEnded" );
 
+/*************************************************************************
+    Child Widget name suffix constants
+*************************************************************************/
+const String Slider::ThumbNameSuffix( "__auto_thumb__" );
 
 /*************************************************************************
 	Slider base class constructor
@@ -54,10 +73,8 @@ Slider::Slider(const String& type, const String& name) :
 	Window(type, name),
 	d_value(0.0f),
 	d_maxValue(1.0f),
-	d_step(0.01f),
-	d_thumb(NULL)
+	d_step(0.01f)
 {
-	addSliderEvents();
 	addSliderProperties();
 }
 
@@ -73,16 +90,15 @@ Slider::~Slider(void)
 /*************************************************************************
 	Initialises the Window based object ready for use.	
 *************************************************************************/
-void Slider::initialise(void)
+void Slider::initialiseComponents(void)
 {
-	// create and attach thumb
-	d_thumb = createThumb(getName() + "__auto_thumb__");
-	addChildWindow(d_thumb);
+	// get thumb
+	Thumb* thumb = getThumb();
 
 	// bind handler to thumb events
-	d_thumb->subscribeEvent(Thumb::EventThumbPositionChanged, Event::Subscriber(&CEGUI::Slider::handleThumbMoved, this));
-	d_thumb->subscribeEvent(Thumb::EventThumbTrackStarted, Event::Subscriber(&CEGUI::Slider::handleThumbTrackStarted, this));
-	d_thumb->subscribeEvent(Thumb::EventThumbTrackEnded, Event::Subscriber(&CEGUI::Slider::handleThumbTrackEnded, this));
+	thumb->subscribeEvent(Thumb::EventThumbPositionChanged, Event::Subscriber(&CEGUI::Slider::handleThumbMoved, this));
+	thumb->subscribeEvent(Thumb::EventThumbTrackStarted, Event::Subscriber(&CEGUI::Slider::handleThumbTrackStarted, this));
+	thumb->subscribeEvent(Thumb::EventThumbTrackEnded, Event::Subscriber(&CEGUI::Slider::handleThumbTrackEnded, this));
 
 	performChildWindowLayout();
 }
@@ -137,17 +153,6 @@ void Slider::setCurrentValue(float value)
 }
 
 
-/*************************************************************************
-	Add slider specific events	
-*************************************************************************/
-void Slider::addSliderEvents(void)
-{
-	addEvent(EventValueChanged);
-	addEvent(EventThumbTrackStarted);
-	addEvent(EventThumbTrackEnded);
-}
-
-
 /*************************************************************************
 	Handler triggered when the slider value changes
 *************************************************************************/
@@ -262,5 +267,67 @@ void Slider::addSliderProperties(void)
 	addProperty(&d_maximumValueProperty);
 }
 
+/*************************************************************************
+    Return a pointer to the Thumb component widget..
+*************************************************************************/
+Thumb* Slider::getThumb() const
+{
+    return static_cast<Thumb*>(WindowManager::getSingleton().getWindow(
+                               getName() + ThumbNameSuffix));
+}
+
+/*************************************************************************
+    update the size and location of the thumb to properly represent the
+    current state of the scroll bar
+*************************************************************************/
+void Slider::updateThumb(void)
+{
+    if (d_windowRenderer != 0)
+    {
+        SliderWindowRenderer* wr = (SliderWindowRenderer*)d_windowRenderer;
+        wr->updateThumb();
+    }
+    else
+    {
+        //updateThumb_impl();
+        throw InvalidRequestException("Slider::updateThumb - This function must be implemented by the window renderer module");
+    }
+}
+
+/*************************************************************************
+    return value that best represents current scroll bar position given
+    the current location of the thumb.
+*************************************************************************/
+float Slider::getValueFromThumb(void) const
+{
+    if (d_windowRenderer != 0)
+    {
+        SliderWindowRenderer* wr = (SliderWindowRenderer*)d_windowRenderer;
+        return wr->getValueFromThumb();
+    }
+    else
+    {
+        //return getValueFromThumb_impl();
+        throw InvalidRequestException("Slider::getValueFromThumb - This function must be implemented by the window renderer module");
+    }
+}
+
+/*************************************************************************
+    Given window location 'pt', return a value indicating what change
+    should be made to the scroll bar.
+*************************************************************************/
+float Slider::getAdjustDirectionFromPoint(const Point& pt) const
+{
+    if (d_windowRenderer != 0)
+    {
+        SliderWindowRenderer* wr = (SliderWindowRenderer*)d_windowRenderer;
+        return wr->getAdjustDirectionFromPoint(pt);
+    }
+    else
+    {
+        //return getAdjustDirectionFromPoint_impl(pt);
+        throw InvalidRequestException("Slider::getAdjustDirectionFromPoint - This function must be implemented by the window renderer module");
+    }
+}
 
 } // End of  CEGUI namespace section
