@@ -1,71 +1,96 @@
-/************************************************************************
+/***********************************************************************
 	filename: 	CEGUITabControl.cpp
 	created:	08/08/2004
 	author:		Steve Streeting
 	
 	purpose:	Implementation of Tab Control widget base class
 *************************************************************************/
-/*************************************************************************
-    Crazy Eddie's GUI System (http://www.cegui.org.uk)
-    Copyright (C)2004 - 2005 Paul D Turner (paul@cegui.org.uk)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*************************************************************************/
+/***************************************************************************
+ *   Copyright (C) 2004 - 2006 Paul D Turner & The CEGUI Development Team
+ *
+ *   Permission is hereby granted, free of charge, to any person obtaining
+ *   a copy of this software and associated documentation files (the
+ *   "Software"), to deal in the Software without restriction, including
+ *   without limitation the rights to use, copy, modify, merge, publish,
+ *   distribute, sublicense, and/or sell copies of the Software, and to
+ *   permit persons to whom the Software is furnished to do so, subject to
+ *   the following conditions:
+ *
+ *   The above copyright notice and this permission notice shall be
+ *   included in all copies or substantial portions of the Software.
+ *
+ *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ *   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ *   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ *   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ *   OTHER DEALINGS IN THE SOFTWARE.
+ ***************************************************************************/
 #include "CEGUIExceptions.h"
 #include "elements/CEGUITabControl.h"
 #include "elements/CEGUITabButton.h"
-#include "elements/CEGUIStatic.h"
+#include "elements/CEGUIPushButton.h"
 #include "elements/CEGUIGUISheet.h"
 #include "CEGUIFont.h"
 #include "CEGUIWindowManager.h"
+#include "CEGUIPropertyHelper.h"
 
 // Start of CEGUI namespace section
 namespace CEGUI
 {
 const String TabControl::EventNamespace("TabControl");
+const String TabControl::WidgetTypeName("CEGUI/TabControl");
+
+/*************************************************************************
+    TabControlWindowRenderer
+*************************************************************************/
+TabControlWindowRenderer::TabControlWindowRenderer(const String& name) :
+    WindowRenderer(name, TabControl::EventNamespace)
+{
+}
 
 /*************************************************************************
 	Definition of Properties for this class
 *************************************************************************/
 TabControlProperties::TabHeight		            TabControl::d_tabHeightProperty;
-TabControlProperties::AbsoluteTabHeight	        TabControl::d_absoluteTabHeightProperty;
-TabControlProperties::RelativeTabHeight	        TabControl::d_relativeTabHeightProperty;
 TabControlProperties::TabTextPadding		    TabControl::d_tabTextPaddingProperty;
-TabControlProperties::AbsoluteTabTextPadding	TabControl::d_absoluteTabTextPaddingProperty;
-TabControlProperties::RelativeTabTextPadding	TabControl::d_relativeTabTextPaddingProperty;
+TabControlProperties::TabPanePosition		    TabControl::d_tabPanePosition;
 
 /*************************************************************************
 	Constants
 *************************************************************************/
 // event names
-const String TabControl::EventSelectionChanged( (utf8*)"TabSelectionChanged" );
+const String TabControl::EventSelectionChanged( "TabSelectionChanged" );
+
+/*************************************************************************
+    Child Widget name suffix constants
+*************************************************************************/
+const String TabControl::ContentPaneNameSuffix( "__auto_TabPane__" );
+const String TabControl::TabButtonNameSuffix( "__auto_btn" );
+const String TabControl::TabButtonPaneNameSuffix( "__auto_TabPane__Buttons" );
+const String TabControl::ButtonScrollLeftSuffix( "__auto_TabPane__ScrollLeft" );
+const String TabControl::ButtonScrollRightSuffix( "__auto_TabPane__ScrollRight" );
+
+/*************************************************************************
+    Miscelaneous private strings
+*************************************************************************/
+static const String EnableTop = "EnableTop";
+static const String EnableBottom = "EnableBottom";
+static const String n0 = "0";
+static const String n1 = "1";
 
-	
 /*************************************************************************
 	Constructor for TabControl base class.
 *************************************************************************/
 TabControl::TabControl(const String& type, const String& name)
-	: Window(type, name),
-    d_tabButtonPane(NULL),
-    d_tabContentPane(NULL),
-    d_nextTabIndex(0)
+    : Window(type, name),
+    d_tabHeight(0, -1), // means 'to be initialized later'
+    d_tabPadding(0, 5),
+    d_firstTabOffset(0),
+    d_tabPanePos(Top)
 {
-	addTabControlEvents();
 	addTabControlProperties();
-    setRelativeTabHeight(0.05f);
-    setAbsoluteTabTextPadding(5);
 }
 
 
@@ -80,44 +105,51 @@ TabControl::~TabControl(void)
 /*************************************************************************
 	Initialise the Window based object ready for use.
 *************************************************************************/
-void TabControl::initialise(void)
+void TabControl::initialiseComponents(void)
 {
-	// create the component sub-widgets
-	d_tabContentPane = createTabContentPane(getName() + "__auto_TabPane__");
-    d_tabButtonPane = createTabButtonPane(getName() + "__auto_TabPane__Buttons");
-
-	addChildWindow(d_tabContentPane);
-    addChildWindow(d_tabButtonPane);
-
 	performChildWindowLayout();
+
+    String name = getName () + ButtonScrollLeftSuffix;
+    if (WindowManager::getSingleton().isWindowPresent (name))
+        WindowManager::getSingleton().getWindow (name)->subscribeEvent (
+            PushButton::EventClicked, Event::Subscriber(
+                &CEGUI::TabControl::handleScrollPane, this));
+
+    name = getName() + ButtonScrollRightSuffix;
+    if (WindowManager::getSingleton().isWindowPresent (name))
+        WindowManager::getSingleton().getWindow (name)->subscribeEvent (
+            PushButton::EventClicked, Event::Subscriber(
+                &CEGUI::TabControl::handleScrollPane, this));
 }
 /*************************************************************************
 Get the number of tabs
 *************************************************************************/
-uint TabControl::getTabCount(void) const
+size_t TabControl::getTabCount(void) const
 {   
-    return d_tabContentPane->getChildCount();
+    return getTabPane()->getChildCount();
 }
 /*************************************************************************
 Get the tab with a given name
 *************************************************************************/
 Window*	TabControl::getTabContents(const String& name) const
 {
-    return d_tabContentPane->getChild(name);
+    return getTabPane()->getChild(name);
 }
 /*************************************************************************
 Get the tab at a given ID
 *************************************************************************/
 Window*	TabControl::getTabContents(uint ID) const
 {
-    return d_tabContentPane->getChild(ID);
+    return getTabPane()->getChild(ID);
 }
 /*************************************************************************
 Get the tab for the given index
 *************************************************************************/
-Window*	TabControl::getTabContentsAtIndex(uint index) const
+Window*	TabControl::getTabContentsAtIndex(size_t index) const
 {
-    return d_tabButtonIndexMap.find(index)->second->getTargetWindow();
+    if (index >= d_tabButtonVector.size ())
+        return 0;
+    return d_tabButtonVector [index]->getTargetWindow();
 }
 
 /*************************************************************************
@@ -132,22 +164,13 @@ bool TabControl::isTabContentsSelected(Window* wnd) const
 /*************************************************************************
 Return whether the tab content window is currently selected.
 *************************************************************************/
-uint TabControl::getSelectedTabIndex() const
+size_t TabControl::getSelectedTabIndex() const
 {
-    uint index;
-    TabButtonIndexMap::const_iterator i, iend;
-    iend = d_tabButtonIndexMap.end();
-    for (i = d_tabButtonIndexMap.begin(); i != iend; ++i)
-    {
-        // get corresponding tab button and content window
-        TabButton* tb = i->second;
-        if (tb->isSelected())
-        {
-            index = i->first;
-			break;
-        }
-	}
-	return index;
+    for (size_t i = 0; i < d_tabButtonVector.size (); ++i)
+        if (d_tabButtonVector [i]->isSelected ())
+            return i;
+
+	throw UnknownObjectException("TabControl::getSelectedTabIndex - Current tab not in list?");
 }
 
 /*************************************************************************
@@ -155,129 +178,64 @@ Set the selected tab by window name
 *************************************************************************/
 void TabControl::setSelectedTab(const String &name)
 {
-    // get window
-    Window* wnd = d_tabContentPane->getChild(name);
-
-    selectTab_impl(wnd);
+    selectTab_impl(getTabPane()->getChild(name));
 }
 /*************************************************************************
 Set the selected tab by window ID
 *************************************************************************/
 void TabControl::setSelectedTab(uint ID)
 {
-    // get window
-    Window* wnd = d_tabContentPane->getChild(ID);
-
-    selectTab_impl(wnd);
+    selectTab_impl(getTabPane()->getChild(ID));
 }
 /*************************************************************************
 Set the selected tab by window name
 *************************************************************************/
-void TabControl::setSelectedTabAtIndex(uint index)
+void TabControl::setSelectedTabAtIndex(size_t index)
 {
-	Window* wnd = getTabContentsAtIndex(index);
-	selectTab_impl(wnd);
+	selectTab_impl(getTabContentsAtIndex(index));
+}
+
+/*************************************************************************
+Make the tab by window name visible
+*************************************************************************/
+void TabControl::makeTabVisible(const String &name)
+{
+    makeTabVisible_impl(getTabPane()->getChild(name));
 }
 /*************************************************************************
-Get the tab height
+Make the tab by window ID visible
 *************************************************************************/
-float TabControl::getTabHeight(void) const
+void TabControl::makeTabVisible(uint ID)
 {
-    MetricsMode mode = getMetricsMode();
-    if (mode == Relative)
-    {
-        return d_rel_tabHeight;
-    }
-    else
-    {
-        return d_abs_tabHeight;
-    }
+    makeTabVisible_impl(getTabPane()->getChild(ID));
+}
+/*************************************************************************
+Make the tab by window name visible
+*************************************************************************/
+void TabControl::makeTabVisibleAtIndex(size_t index)
+{
+	makeTabVisible_impl(getTabContentsAtIndex(index));
 }
 
 /*************************************************************************
 Set the tab height
 *************************************************************************/
-void TabControl::setRelativeTabHeight(float height)
+void TabControl::setTabHeight(const UDim& height)
 {
-    d_rel_tabHeight = height;
-    d_abs_tabHeight = relativeToAbsoluteY(height);
+    d_tabHeight = height;
 
     performChildWindowLayout();
 }
-/*************************************************************************
-Set the tab height
-*************************************************************************/
-void TabControl::setAbsoluteTabHeight(float height)
-{
-    d_abs_tabHeight = height;
-    d_rel_tabHeight = absoluteToRelativeY(height);
-
-    performChildWindowLayout();
-}
-/*************************************************************************
-Set the tab height
-*************************************************************************/
-void TabControl::setTabHeight(float height)
-{
-    if (getMetricsMode() == Relative)
-    {
-        setRelativeTabHeight(height);
-    }
-    else
-    {
-        setAbsoluteTabHeight(height);
-    }
-}
-/*************************************************************************
-Get the tab text padding
-*************************************************************************/
-float TabControl::getTabTextPadding(void) const
-{
-    MetricsMode mode = getMetricsMode();
-    if (mode == Relative)
-    {
-        return d_rel_tabPadding;
-    }
-    else
-    {
-        return d_abs_tabPadding;
-    }
-}
 
 /*************************************************************************
 Set the tab text padding
 *************************************************************************/
-void TabControl::setRelativeTabTextPadding(float height)
+void TabControl::setTabTextPadding(const UDim& padding)
 {
-    d_rel_tabPadding = height;
-    d_abs_tabPadding = relativeToAbsoluteY(height);
+    d_tabPadding = padding;
 
     performChildWindowLayout();
 }
-/*************************************************************************
-Set the tab text padding
-*************************************************************************/
-void TabControl::setAbsoluteTabTextPadding(float height)
-{
-    d_abs_tabPadding = height;
-    d_rel_tabPadding = absoluteToRelativeY(height);
-
-    performChildWindowLayout();
-}
-/*************************************************************************
-Set the tab text padding
-*************************************************************************/
-void TabControl::setTabTextPadding(float height)
-{
-    if (getMetricsMode() == Relative)
-    {
-        setRelativeTabTextPadding(height);
-    }
-    else
-    {
-        setAbsoluteTabTextPadding(height);
-    }
-}
 
 /*************************************************************************
 Add a new tab
@@ -287,17 +245,18 @@ void TabControl::addTab(Window* wnd)
     // Create a new TabButton
     addButtonForTabContent(wnd);
     // Add the window to the content pane
-    d_tabContentPane->addChildWindow(wnd);
+    getTabPane()->addChildWindow(wnd);
     // Auto-select?
     if (getTabCount() == 1)
-    {
         setSelectedTab(wnd->getName());
-    }
     else
-    {
         // initialise invisible content
         wnd->setVisible(false);
-    }
+
+	// when adding the 1st page, autosize tab pane height
+    if (d_tabHeight.d_scale == 0 && d_tabHeight.d_offset == -1)
+        d_tabHeight.d_offset = 8 + getFont()->getFontHeight ();
+
     // Just request redraw
     performChildWindowLayout();
     requestRedraw();
@@ -312,26 +271,22 @@ Remove a tab
 void TabControl::removeTab(const String& name)
 {
     // do nothing if given window is not attached as a tab.
-    if (!d_tabContentPane->isChild(name))
+    if (!getTabPane()->isChild(name))
         return;
 
-    Window* wnd = d_tabContentPane->getChild(name);
+    Window* wnd = getTabPane()->getChild(name);
+
     // Was this selected?
     bool reselect = wnd->isVisible();
     // Tab buttons are the 2nd onward children
-    d_tabContentPane->removeChildWindow(name);
+    getTabPane()->removeChildWindow(name);
 
     // remove button too
     removeButtonForTabContent(wnd);
 
-    if (reselect)
-    {
+    if (reselect && (getTabCount() > 0))
         // Select another tab
-        if (getTabCount() > 0)
-        {
-            setSelectedTab(d_tabContentPane->getChildAtIdx(0)->getName());
-        }
-    }
+        setSelectedTab(getTabPane()->getChildAtIdx(0)->getName());
 
     performChildWindowLayout();
 
@@ -343,14 +298,15 @@ Remove a tab by ID
 void TabControl::removeTab(uint ID)
 {
     // do nothing if given window is not attached as a tab.
-    if (!d_tabContentPane->isChild(ID))
+    if (!getTabPane()->isChild(ID))
         return;
 
-    Window* wnd = d_tabContentPane->getChild(ID);
+    Window* wnd = getTabPane()->getChild(ID);
+
     // Was this selected?
     bool reselect = wnd->isVisible();
     // Tab buttons are the 2nd onward children
-    d_tabContentPane->removeChildWindow(ID);
+    getTabPane()->removeChildWindow(ID);
 
     // remove button too
     removeButtonForTabContent(wnd);
@@ -360,7 +316,7 @@ void TabControl::removeTab(uint ID)
         // Select another tab
         if (getTabCount() > 0)
         {
-            setSelectedTab(d_tabContentPane->getChildAtIdx(0)->getName());
+            setSelectedTab(getTabPane()->getChildAtIdx(0)->getName());
         }
     }
 
@@ -380,17 +336,17 @@ void TabControl::addButtonForTabContent(Window* wnd)
     tb->setFont(getFont());
     // Set target window
     tb->setTargetWindow(wnd);
-    // Set index
-    tb->setTabIndex(d_nextTabIndex++);
     // Instert into map
-    d_tabButtonIndexMap.insert(
-        TabButtonIndexMap::value_type(tb->getTabIndex(), tb));
+    d_tabButtonVector.push_back(tb);
     // add the button
-    d_tabButtonPane->addChildWindow(tb);
+    getTabButtonPane()->addChildWindow(tb);
     // Subscribe to clicked event so that we can change tab
     tb->subscribeEvent(TabButton::EventClicked, 
         Event::Subscriber(&TabControl::handleTabButtonClicked, this));
-
+    tb->subscribeEvent(TabButton::EventDragged,
+        Event::Subscriber(&TabControl::handleDraggedPane, this));
+    tb->subscribeEvent(TabButton::EventScrolled,
+        Event::Subscriber(&TabControl::handleWheeledPane, this));
 }
 
 /*************************************************************************
@@ -398,51 +354,11 @@ void TabControl::addButtonForTabContent(Window* wnd)
 *************************************************************************/
 TabButton* TabControl::getButtonForTabContents(Window* wnd) const
 {
-    TabButtonIndexMap::const_iterator i, iend;
-    iend = d_tabButtonIndexMap.end();
-    for (i = d_tabButtonIndexMap.begin(); i != iend; ++i)
-    {
-        // get corresponding tab button and content window
-        TabButton* tb = i->second;
-        Window* child = tb->getTargetWindow();
-        if (child == wnd)
-        {
-			return tb;
-        }
-	}
-	throw UnknownObjectException((utf8*)"TabControl::getButtonForTabContents - The Window object is not a tab contents.");
-}
-/*************************************************************************
-	Calculate size and position for a tab button
-*************************************************************************/
-void TabControl::calculateTabButtonSizePosition(TabButton* btn, uint targetIndex)
-{
-    // relative height is always 1.0 for buttons since they are embedded in a
-    // panel of the correct height already
-    btn->setHeight(Relative, 1.0f);
-    btn->setYPosition(Relative, 0.0f);
-    // x position is based on previous button
-    if (targetIndex > 0)
-    {
-		TabButtonIndexMap::iterator iter = d_tabButtonIndexMap.begin();
-		std::advance(iter, targetIndex - 1);
-		Window* prevButton = iter->second;
+    for (size_t i = 0; i < d_tabButtonVector.size (); ++i)
+        if (d_tabButtonVector [i]->getTargetWindow () == wnd)
+            return d_tabButtonVector [i];
 
-		// position is prev pos + width
-        btn->setXPosition(Relative, 
-            prevButton->getXPosition(Relative) 
-            + prevButton->getWidth(Relative));
-    }
-    else
-    {
-        // First button
-        btn->setXPosition(Relative, 0);
-    }
-    // Width is based on font size (expressed as absolute)
-    const Font* fnt = btn->getFont();
-    btn->setWidth(Absolute, 
-        fnt->getTextExtent(btn->getText()) + getAbsoluteTabTextPadding()*2);
-    btn->requestRedraw();
+	throw UnknownObjectException("TabControl::getButtonForTabContents - The Window object is not a tab contents.");
 }
 /*************************************************************************
 Remove tab button
@@ -451,10 +367,16 @@ void TabControl::removeButtonForTabContent(Window* wnd)
 {
     // get
     TabButton* tb = static_cast<TabButton*>(
-        d_tabButtonPane->getChild(makeButtonName(wnd)));
+        getTabButtonPane()->getChild(makeButtonName(wnd)));
     // remove
-    d_tabButtonIndexMap.erase(tb->getTabIndex());
-    d_tabButtonPane->removeChildWindow(tb);
+    TabButtonVector::iterator i;
+    for (i = d_tabButtonVector.begin (); i < d_tabButtonVector.end (); ++i)
+        if (*i == tb)
+        {
+            d_tabButtonVector.erase (i);
+            break;
+        }
+    getTabButtonPane()->removeChildWindow(tb);
 	// destroy
 	WindowManager::getSingleton().destroyWindow(tb);
 }
@@ -463,9 +385,11 @@ Remove tab button
 *************************************************************************/
 String TabControl::makeButtonName(Window* wnd)
 {
-    // derive button name
-    String buttonName = (utf8*)"__auto_btn";
-    buttonName.append(wnd->getName());
+    // Create the button name as "'auto' parent + 'auto' button + tab name"
+    String buttonName = getTabButtonPane()->getName();
+    buttonName.append(TabButtonNameSuffix);
+    size_t pathEndPos = wnd->getName().find_last_of("/");
+    buttonName.append(wnd->getName().substr(pathEndPos == String::npos ? 0 : pathEndPos + 1));
     return buttonName;
 }
 /*************************************************************************
@@ -473,24 +397,21 @@ Select tab implementation
 *************************************************************************/
 void TabControl::selectTab_impl(Window* wnd)
 {
+    makeTabVisible_impl(wnd);
+
     bool modified = false;
-    bool foundSelected = false;
     // Iterate in order of tab index
-    TabButtonIndexMap::iterator i, iend;
-    iend = d_tabButtonIndexMap.end();
-    for (i = d_tabButtonIndexMap.begin(); i != iend; ++i)
+    for (size_t i = 0; i < d_tabButtonVector.size(); ++i)
     {
         // get corresponding tab button and content window
-        TabButton* tb = i->second;
+        TabButton* tb = d_tabButtonVector [i];
         Window* child = tb->getTargetWindow();
         // Should we be selecting?
         bool selectThis = (child == wnd);
         // Are we modifying this tab?
         modified = modified || (tb->isSelected() != selectThis);
-        foundSelected = foundSelected || selectThis;
         // Select tab & set visible if this is the window, not otherwise
         tb->setSelected(selectThis);
-        tb->setRightOfSelected(foundSelected);
         child->setVisible(selectThis);
     }
     // Trigger event?
@@ -501,16 +422,67 @@ void TabControl::selectTab_impl(Window* wnd)
     }
 }
 /*************************************************************************
+Make tab visible implementation
+*************************************************************************/
+void TabControl::makeTabVisible_impl(Window* wnd)
+{
+    TabButton *tb = 0;
+
+    for (size_t i = 0; i < d_tabButtonVector.size (); ++i)
+    {
+        // get corresponding tab button and content window
+        tb = d_tabButtonVector [i];
+        Window* child = tb->getTargetWindow();
+        if (child == wnd)
+            break;
+        tb = 0;
+    }
+
+    if (!tb)
+        return;
+
+    float ww = getPixelSize ().d_width;
+    float x = tb->getXPosition().asAbsolute (ww);
+    float w = tb->getPixelSize ().d_width;
+    float lx = 0, rx = ww;
+
+    Window *scrollLeftBtn = 0, *scrollRightBtn = 0;
+    String name = getName() + ButtonScrollLeftSuffix;
+    if (WindowManager::getSingleton().isWindowPresent (name))
+    {
+        scrollLeftBtn = WindowManager::getSingleton().getWindow (name);
+        lx = scrollLeftBtn->getArea ().d_max.d_x.asAbsolute (ww);
+        scrollLeftBtn->setWantsMultiClickEvents (false);
+    }
+
+    name = getName() + ButtonScrollRightSuffix;
+    if (WindowManager::getSingleton().isWindowPresent (name))
+    {
+        scrollRightBtn = WindowManager::getSingleton().getWindow (name);
+        rx = scrollRightBtn->getXPosition ().asAbsolute (ww);
+        scrollRightBtn->setWantsMultiClickEvents (false);
+    }
+
+    if (x < lx)
+        d_firstTabOffset += lx - x;
+    else
+    {
+        if (x + w <= rx)
+            return; // nothing to do
+
+        d_firstTabOffset += rx - (x + w);
+    }
+
+    performChildWindowLayout ();
+}
+/*************************************************************************
 Add tab control properties
 *************************************************************************/
 void TabControl::addTabControlProperties(void)
 {
     addProperty(&d_tabHeightProperty);
-    addProperty(&d_relativeTabHeightProperty);
-    addProperty(&d_absoluteTabHeightProperty);
     addProperty(&d_tabTextPaddingProperty);
-    addProperty(&d_relativeTabTextPaddingProperty);
-    addProperty(&d_absoluteTabTextPaddingProperty);
+    addProperty(&d_tabPanePosition);
 }
 /*************************************************************************
 Internal version of adding a child window
@@ -518,7 +490,7 @@ Internal version of adding a child window
 void TabControl::addChild_impl(Window* wnd)
 {
     // Look for __auto_TabPane__ in the name (hopefully no-one will use this!)
-    if (wnd->getName().find((const utf8*)"__auto_TabPane__") != String::npos)
+    if (wnd->getName().find(ContentPaneNameSuffix) != String::npos)
     {
         // perform normal addChild
         Window::addChild_impl(wnd);
@@ -538,7 +510,7 @@ void TabControl::removeChild_impl(Window* wnd)
     if (!wnd) return;
 
     // Look for __auto_TabPane__ in the name (hopefully no-one will use this!)
-    if (wnd->getName().find((const utf8*)"__auto_TabPane__") != String::npos)
+    if (wnd->getName().find(ContentPaneNameSuffix) != String::npos)
     {
         // perform normal removeChild
         Window::removeChild_impl(wnd);
@@ -563,56 +535,114 @@ Font changed event
 void TabControl::onFontChanged(WindowEventArgs& e)
 {
     // Propagate font change to buttons
-    TabButtonIndexMap::iterator i, iend;
-    iend = d_tabButtonIndexMap.end();
-    for (i = d_tabButtonIndexMap.end(); i != iend; ++i)
-    {
-        i->second->setFont(getFont());
-    }
+    for (size_t i = 0; i < d_tabButtonVector.size(); ++i)
+        d_tabButtonVector [i]->setFont (getFont ());
 }
 /*************************************************************************
-Add events for this class
+	Calculate size and position for a tab button
 *************************************************************************/
-void TabControl::addTabControlEvents(void)
+void TabControl::calculateTabButtonSizePosition(size_t index)
 {
-    addEvent(EventSelectionChanged);
+    TabButton* btn = d_tabButtonVector [index];
+    // relative height is always 1.0 for buttons since they are embedded in a
+    // panel of the correct height already
+    btn->setHeight(cegui_reldim(1.0f));
+    btn->setYPosition(cegui_absdim(0.0f));
+    // x position is based on previous button
+    if (!index)
+        // First button
+        btn->setXPosition(cegui_absdim(d_firstTabOffset));
+    else
+    {
+		Window* prevButton = d_tabButtonVector [index - 1];
+
+		// position is prev pos + width
+        btn->setXPosition(prevButton->getArea().d_max.d_x);
+    }
+    // Width is based on font size (expressed as absolute)
+    Font* fnt = btn->getFont();
+    btn->setWidth(cegui_absdim(fnt->getTextExtent(btn->getText())) +
+                        getTabTextPadding() + getTabTextPadding());
+
+    float left_x = btn->getXPosition ().d_offset;
+    btn->setVisible ((left_x < getPixelSize ().d_width) &&
+                     (left_x + btn->getPixelSize ().d_width > 0));
+    btn->requestRedraw();
 }
 /*************************************************************************
 Layout the widgets
 *************************************************************************/
 void TabControl::performChildWindowLayout()
 {
+    Window* tabButtonPane = getTabButtonPane();
+    Window* tabContentPane = getTabPane();
+
+    // Enable top/bottom edges of the tabPane control,
+    // if supported by looknfeel
+    if (tabContentPane->isPropertyPresent (EnableTop))
+        tabContentPane->setProperty (EnableTop, (d_tabPanePos == Top) ? n0 : n1);
+    if (tabContentPane->isPropertyPresent (EnableBottom))
+        tabContentPane->setProperty (EnableBottom, (d_tabPanePos == Top) ? n1 : n0);
+    if (tabButtonPane->isPropertyPresent (EnableTop))
+        tabButtonPane->setProperty (EnableTop, (d_tabPanePos == Top) ? n0 : n1);
+    if (tabButtonPane->isPropertyPresent (EnableBottom))
+        tabButtonPane->setProperty (EnableBottom, (d_tabPanePos == Top) ? n1 : n0);
+
     Window::performChildWindowLayout();
 
-    if (d_tabButtonPane)
-    {
-        // Set the size of the tab button area (full width, height from tab height)
-        d_tabButtonPane->setSize(Relative, Size(1.0f, d_rel_tabHeight) );
-        d_tabButtonPane->setPosition(Relative, Point(0.0f, 0.0f) );
-        // Calculate the positions and sizes of the tab buttons
-        TabButtonIndexMap::iterator i, iend;
-        iend = d_tabButtonIndexMap.end();
-        uint x = 0;
-        for (i = d_tabButtonIndexMap.begin(); i != iend; ++i, ++x)
-        {
-            TabButton* btn = i->second;
-            calculateTabButtonSizePosition(btn, x);
-        }
-    }
-    if (d_tabContentPane)
-    {
-        // Set the size of the content area
-        d_tabContentPane->setSize(Relative, Size(1.0f, 1.0f - d_rel_tabHeight) );
-        d_tabContentPane->setPosition(Relative, Point(0.0f, d_rel_tabHeight) );
-    }
+    // Calculate the size & position of the tab scroll buttons
+    Window *scrollLeftBtn = 0, *scrollRightBtn = 0;
+    String name = getName() + ButtonScrollLeftSuffix;
+    if (WindowManager::getSingleton().isWindowPresent (name))
+        scrollLeftBtn = WindowManager::getSingleton().getWindow (name);
 
-}
-/*************************************************************************
-Create tab button pane
-*************************************************************************/
-Window*	TabControl::createTabButtonPane(const String& name) const
-{
-	return WindowManager::getSingleton().createWindow(GUISheet::WidgetTypeName, name);
+    name = getName() + ButtonScrollRightSuffix;
+    if (WindowManager::getSingleton().isWindowPresent (name))
+        scrollRightBtn = WindowManager::getSingleton().getWindow (name);
+
+    // Calculate the positions and sizes of the tab buttons
+    if (d_firstTabOffset > 0)
+        d_firstTabOffset = 0;
+
+    for (;;)
+    {
+        size_t i;
+        for (i = 0; i < d_tabButtonVector.size (); ++i)
+            calculateTabButtonSizePosition (i);
+
+        if (d_tabButtonVector.empty ())
+        {
+            if (scrollRightBtn)
+                scrollRightBtn->setVisible (false);
+            if (scrollLeftBtn)
+                scrollLeftBtn->setVisible (false);
+            break;
+        }
+
+        // Now check if tab pane wasn't scrolled too far
+        --i;
+        float xmax = d_tabButtonVector [i]->getXPosition ().d_offset +
+            d_tabButtonVector [i]->getPixelSize ().d_width;
+        float width = tabContentPane->getPixelSize ().d_width;
+
+        // If right button margin exceeds right window margin,
+        // or leftmost button is at offset 0, finish
+        if ((xmax > (width - 0.5)) || (d_firstTabOffset == 0))
+        {
+            if (scrollLeftBtn)
+                scrollLeftBtn->setVisible (d_firstTabOffset < 0);
+            if (scrollRightBtn)
+                scrollRightBtn->setVisible (xmax > width);
+            break;
+        }
+
+        // Scroll the tab pane until the rightmost button
+        // touches the right margin
+        d_firstTabOffset += width - xmax;
+        // If we scrolled it too far, set leftmost button offset to 0
+        if (d_firstTabOffset > 0)
+            d_firstTabOffset = 0;
+    }
 }
 /*************************************************************************
 Text changed on a content window
@@ -621,7 +651,7 @@ bool TabControl::handleContentWindowTextChanged(const EventArgs& args)
 {
     // update text
     const WindowEventArgs& wargs = static_cast<const WindowEventArgs&>(args);
-    Window* tabButton = d_tabButtonPane->getChild(
+    Window* tabButton = getTabButtonPane()->getChild(
         makeButtonName(wargs.window));
     tabButton->setText(wargs.window->getText());
     // sort out the layout
@@ -642,20 +672,140 @@ bool TabControl::handleTabButtonClicked(const EventArgs& args)
 	return true;
 }
 
-int TabControl::writeChildWindowsXML(OutStream& out_stream) const
+/*************************************************************************
+    Return a pointer to the tab button pane (Window)for
+    this TabControl.
+*************************************************************************/
+Window* TabControl::getTabButtonPane() const
 {
-    int childOutputCount = Window::writeChildWindowsXML(out_stream);
+    return WindowManager::getSingleton().getWindow(getName() +
+                                                   TabButtonPaneNameSuffix);
+}
+
+/*************************************************************************
+    Return a pointer to the TabPane component widget for
+    this TabControl.
+*************************************************************************/
+Window* TabControl::getTabPane() const
+{
+    return WindowManager::getSingleton().getWindow(getName() + ContentPaneNameSuffix);
+}
+
+int TabControl::writeChildWindowsXML(XMLSerializer& xml_stream) const
+{
+    int childOutputCount = Window::writeChildWindowsXML(xml_stream);
 
     // since TabControl content is actually added to the component tab
     // content pane window, this overridden function exists to dump those
     // out as if they were our own children.
-    for (uint i = 0; i < getTabCount(); ++i)
+    for (size_t i = 0; i < getTabCount(); ++i)
     {
-        getTabContentsAtIndex(i)->writeXMLToStream(out_stream);
+        getTabContentsAtIndex(i)->writeXMLToStream(xml_stream);
         ++childOutputCount;
     }
 
     return childOutputCount;
 }
 
+/*************************************************************************
+    create and return a pointer to a TabButton widget for use as a
+    clickable tab header
+*************************************************************************/
+TabButton* TabControl::createTabButton(const String& name) const
+{
+    if (d_windowRenderer != 0)
+    {
+        TabControlWindowRenderer* wr = (TabControlWindowRenderer*)d_windowRenderer;
+        return wr->createTabButton(name);
+    }
+    else
+    {
+        //return createTabButton_impl(name);
+        throw InvalidRequestException("TabControl::createTabButton - This function must be implemented by the window renderer module");
+    }
+}
+
+/*************************************************************************
+	change the positioning of the tab pane.
+*************************************************************************/
+void TabControl::setTabPanePosition(TabPanePosition pos)
+{
+	d_tabPanePos = pos;
+    performChildWindowLayout();
+}
+
+/*************************************************************************
+    scroll the tab pane to left or right depending of the clicked button
+*************************************************************************/
+bool TabControl::handleScrollPane(const EventArgs& e)
+{
+    const WindowEventArgs& wargs = static_cast<const WindowEventArgs&>(e);
+
+    size_t i;
+    float delta = 0;
+    // Find the leftmost visible button
+    for (i = 0; i < d_tabButtonVector.size(); ++i)
+    {
+        if (d_tabButtonVector [i]->isVisible (true))
+            break;
+        delta = d_tabButtonVector [i]->getPixelSize ().d_width;
+    }
+
+    if (wargs.window->getName () == getName() + ButtonScrollLeftSuffix)
+    {
+        if (delta == 0.0f && i < d_tabButtonVector.size ())
+            delta = d_tabButtonVector [i]->getPixelSize ().d_width;
+
+        // scroll button pane to the right
+        d_firstTabOffset += delta;
+    }
+    else if (i < d_tabButtonVector.size ())
+        // scroll button pane to left
+        d_firstTabOffset -= d_tabButtonVector [i]->getPixelSize ().d_width;
+
+    performChildWindowLayout();
+	return true;
+}
+
+bool TabControl::handleDraggedPane(const EventArgs& e)
+{
+    const MouseEventArgs& me = static_cast<const MouseEventArgs&>(e);
+
+    if (me.button == MiddleButton)
+    {
+        // This is the middle-mouse-click event, remember initial drag position
+        Window *but_pane = getTabButtonPane();
+        d_btGrabPos = (me.position.d_x - but_pane->getPixelRect ().d_left) -
+            d_firstTabOffset;
+    }
+    else if (me.button == NoButton)
+    {
+        // Regular mouse move event
+        Window *but_pane = getTabButtonPane();
+        float new_to = (me.position.d_x - but_pane->getPixelRect ().d_left) -
+            d_btGrabPos;
+        if ((new_to < d_firstTabOffset - 0.9) ||
+            (new_to > d_firstTabOffset + 0.9))
+        {
+            d_firstTabOffset = new_to;
+            performChildWindowLayout();
+        }
+    }
+
+    return true;
+}
+
+bool TabControl::handleWheeledPane(const EventArgs& e)
+{
+    const MouseEventArgs& me = static_cast<const MouseEventArgs&>(e);
+
+    Window *but_pane = getTabButtonPane();
+    float delta = but_pane->getPixelRect ().getWidth () / 20;
+    
+    d_firstTabOffset -= me.wheelChange * delta;
+    performChildWindowLayout();
+
+    return true;
+}
+
 } // End of  CEGUI namespace section
